package se.su.dsv.scipro.crosscutting;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.collect.FluentIterable;
import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.FinalSeminarApprovalRequestEvent;
import se.su.dsv.scipro.reviewing.RoughDraftApprovalRequestedEvent;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.HashSet;
import java.util.Set;

@Singleton
public class ReviewerSupportMailer {
    private final MailEventService mailEventService;
    private final GeneralSystemSettingsService systemSettingsService;

    @Inject
    public ReviewerSupportMailer(final EventBus eventBus, MailEventService mailEventService, GeneralSystemSettingsService systemSettingsService) {
        this.mailEventService = mailEventService;
        this.systemSettingsService = systemSettingsService;
        eventBus.register(this);
    }

    @Subscribe
    public void onFinalSeminarApprovalRequest(final FinalSeminarApprovalRequestEvent finalSeminarApprovalRequestEvent) {
        final Project project = finalSeminarApprovalRequestEvent.getProject();
        if (project.getReviewer() == null) {
            final String comment = finalSeminarApprovalRequestEvent.getFinalSeminarApproval().getCurrentDecision().getComment();
            sendMailToReviewerSupport(project, comment, ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED);
        }
    }

    @Subscribe
    public void onRoughDraftApprovalRequest(final RoughDraftApprovalRequestedEvent roughDraftApprovalRequestedEvent) {
        final Project project = roughDraftApprovalRequestedEvent.getProject();
        if (project.getReviewer() == null) {
            final String comment = roughDraftApprovalRequestedEvent.getRoughDraftApproval().getCurrentDecision().getComment();
            sendMailToReviewerSupport(project, comment, ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REQUESTED);
        }
    }

    private void sendMailToReviewerSupport(Project project, final String comment, ProjectEvent.Event event) {
        GeneralSystemSettings systemSettings = systemSettingsService.getGeneralSystemSettingsInstance();

        MailEvent mail = new MailEvent();
        mail.setFromEmail(systemSettings.getSystemFromMail());
        mail.setFromName(systemSettings.getMailFromName());
        mail.setNonUserRecipients(getReviewerSupport(systemSettings));
        mail.setSubject("Important: Reviewer needed!");
        mail.setMessageBody(event.name() + "\n"
                + " Project: " + project.getTitle() + "\n"
                + " Level: " + project.getProjectType().getName() + "\n"
                + " Supervisor: " + project.getHeadSupervisor().getFullName() + "\n"
                + " Research area: " + ((project.getResearchArea() == null) ? "N/A" : project.getResearchArea().getTitle()) + "\n"
                + " Authors: " + FluentIterable.from(project.getProjectParticipants())
                .transform(new Function<User, String>() {
                    @Override
                    public String apply(final User author) {
                        return author.getFullName();
                    }
                })
                .join(Joiner.on(", ")) + "\n"
                + " Comment: " + comment + "\n\n"
                + " Assign a reviewer to this project asap!");
        mailEventService.save(mail);
    }

    private Set<String> getReviewerSupport(GeneralSystemSettings systemSettings) {
        Set<String> reviewerSupport = new HashSet<>(1);
        reviewerSupport.add(systemSettings.getReviewerSupportMail());
        return reviewerSupport;
    }
}
