package se.su.dsv.scipro.data.facade;

import se.su.dsv.scipro.data.enums.MailChoice;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.mail.EmailRecipient;
import se.su.dsv.scipro.mail.Recipient;
import se.su.dsv.scipro.mail.UserRecipient;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.match.IdeaParticipation;
import se.su.dsv.scipro.match.IdeaService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.io.Serializable;
import java.util.*;

public class MailFacade implements Serializable {

    @Inject
    private ProjectService projectService;
    @Inject
    private IdeaService ideaService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    private Set<Recipient> addProjectAuthors(ProjectStatus ps, Set<ProjectType> pc, Date startDate, Date endDate) {
        final ProjectService.Filter filter = setProjectParams(ps, pc, startDate, endDate);
        return authorsFromProjects(filter);
    }

    private Set<Recipient> addProjectSupervisors(ProjectStatus ps, Set<ProjectType> pc, Date startDate, Date endDate) {
        final ProjectService.Filter filter = setProjectParams(ps, pc, startDate, endDate);
        return supervisorsFromProjects(filter);
    }

    private Set<Recipient> addProjectCoSupervisors(ProjectStatus ps, Set<ProjectType> pc, Date startDate, Date endDate) {
        final ProjectService.Filter filter = setProjectParams(ps, pc, startDate, endDate);
        return coSupervisorsFromProjects(filter);
    }

    private Set<Recipient> addProjectReviewers(ProjectStatus ps, Set<ProjectType> pc, Date startDate, Date endDate) {
        final ProjectService.Filter filter = setProjectParams(ps, pc, startDate, endDate);
        return reviewersFromProjects(filter);
    }

    private ProjectService.Filter setProjectParams(ProjectStatus status, Set<ProjectType> pc, Date startDate, Date endDate) {
        final ProjectService.Filter projectParams = new ProjectService.Filter();
        if (status != null) {
            projectParams.setStatuses(new HashSet<>(Collections.singletonList(status)));
        }
        if (pc != null && !pc.isEmpty()) {
            projectParams.setProjectTypes(pc);
        }
        if (startDate != null) {
            projectParams.setCreatedAfter(startDate);
        }
        if (endDate != null) {
            projectParams.setCreatedBefore(endDate);
        }
        return projectParams;
    }

    private Set<Recipient> authorsFromProjects(final ProjectService.Filter filter) {
        List<Project> listOfProjects = projectService.findAll(filter);
        Set<Recipient> recipients = new HashSet<>();
        for (Project p : listOfProjects) {
            for (User s : p.getProjectParticipants()) {
                recipients.add(new UserRecipient(s));
            }
        }
        recipients.addAll(addThesisSupport());
        return recipients;
    }

    private Set<Recipient> supervisorsFromProjects(final ProjectService.Filter filter) {
        List<Project> listOfProjects = projectService.findAll(filter);
        Set<Recipient> recipients = new HashSet<>();
        for (Project p : listOfProjects) {
            recipients.add(new UserRecipient(p.getHeadSupervisor()));
        }
        recipients.addAll(addThesisSupport());
        return recipients;
    }

    private Set<Recipient> coSupervisorsFromProjects(final ProjectService.Filter filter) {
        List<Project> listOfProjects = projectService.findAll(filter);
        Set<Recipient> recipients = new HashSet<>();
        for (Project p : listOfProjects) {
            for (User user : p.getCoSupervisors()){
                recipients.add(new UserRecipient(user));
            }
        }
        recipients.addAll(addThesisSupport());
        return recipients;
    }

    private Set<Recipient> reviewersFromProjects(final ProjectService.Filter filter) {
        List<Project> listOProjects = projectService.findAll(filter);
        Set<Recipient> recipients = new HashSet<>();
        for (Project p : listOProjects) {
            for (User user : p.getReviewers()){
                recipients.add(new UserRecipient(user));
            }
        }
        recipients.addAll(addThesisSupport());
        return recipients;
    }

    private Set<Recipient> addThesisSupport() {
        String thesisSupportMail = generalSystemSettingsService.getGeneralSystemSettingsInstance().getThesisSupportMail();
        if (thesisSupportMail != null) {
            return Collections.<Recipient>singleton(new EmailRecipient(thesisSupportMail));
        }
        else {
            return Collections.emptySet();
        }
    }

    public Set<Recipient> getRecipients(MailChoice choice, Set<ProjectType> pc, Date startDate, Date endDate) {
        switch (choice) {
            case THESIS_SUPPORT:
                return addThesisSupport();
            case AUTHORS_ACTIVE_PROJECT:
                return addProjectAuthors(ProjectStatus.ACTIVE, pc, startDate, endDate);
            case ACTIVE_SUPERVISORS:
                return addProjectSupervisors(ProjectStatus.ACTIVE, pc, startDate, endDate);
            case ACTIVE_CO_SUPERVISORS:
                return addProjectCoSupervisors(ProjectStatus.ACTIVE, pc, startDate, endDate);
            case ACTIVE_REVIEWERS:
                return addProjectReviewers(ProjectStatus.ACTIVE, pc, startDate, endDate);
            case ALL_FOLLOWERS:
                Set<Recipient> allFollowers = new HashSet<>();
                allFollowers.addAll(addProjectSupervisors(ProjectStatus.ACTIVE, pc, startDate, endDate));
                allFollowers.addAll(addProjectReviewers(ProjectStatus.ACTIVE, pc, startDate, endDate));
                allFollowers.addAll(addProjectCoSupervisors(ProjectStatus.ACTIVE, pc, startDate, endDate));
                return allFollowers;
            case AUTHORS_MATCHED_IDEA:
                IdeaService.Filter matchedFilter = setIdeaParams(Arrays.asList(Idea.Status.COMPLETED, Idea.Status.MATCHED), pc, startDate, endDate, null);
                return authorsFromIdeas(matchedFilter);
            case AUTHORS_SUBMITTED_IDEA:
                IdeaService.Filter submittedFilter = setIdeaParams(null, pc, startDate, endDate, Collections.singletonList(Type.STUDENT));
                return authorsFromIdeas(submittedFilter);
            case SUPERVISORS_MATCHED_IDEA:
                IdeaService.Filter supervisorMatchedFilter = setIdeaParams(Arrays.asList(Idea.Status.COMPLETED, Idea.Status.MATCHED), pc, startDate, endDate, null);
                return supervisorsFromIdeas(supervisorMatchedFilter);
            case SUPERVISORS_SUBMITTED_IDEA:
                IdeaService.Filter supervisorSubmittedFilter = setIdeaParams(null, pc, startDate, endDate, Collections.singletonList(Type.SUPERVISOR));
                return supervisorsFromIdeas(supervisorSubmittedFilter);
            default:
                return new HashSet<>();
        }
    }

    private IdeaService.Filter setIdeaParams(Collection<Idea.Status> statuses, Set<ProjectType> pc, Date startDate, Date endDate, Collection<Type> types) {
        final IdeaService.Filter ideaParams = new IdeaService.Filter();
        if (statuses != null && !statuses.isEmpty()) {
            ideaParams.setStatuses(statuses);
        }
        if (pc != null && !pc.isEmpty()) {
            ideaParams.setLevels(pc);
        }
        if (types != null && !types.isEmpty()) {
            ideaParams.setTypes(types);
        }
        if (startDate != null) {
            ideaParams.setCreatedAfter(startDate);
        }
        if (endDate != null) {
            ideaParams.setCreatedBefore(endDate);
        }
        return ideaParams;
    }

    private Set<Recipient> authorsFromIdeas(final IdeaService.Filter filter) {
        Set<Recipient> recipients = new HashSet<>();
        Iterable<Idea> ideas = ideaService.findAll(filter);
        for (Idea idea : ideas) {
            for (IdeaParticipation ip : idea.getIdeaParticipations()) {
                recipients.add(new UserRecipient(ip.getUser()));
            }
        }
        recipients.addAll(addThesisSupport());
        return recipients;
    }

    private Set<Recipient> supervisorsFromIdeas(final IdeaService.Filter filter) {
        Set<Recipient> recipients = new HashSet<>();
        Iterable<Idea> ideas = ideaService.findAll(filter);
        for (Idea idea : ideas) {
            if (idea.getMatch() != null && idea.getMatch().getSupervisor() != null) {
                recipients.add(new UserRecipient(idea.getMatch().getSupervisor()));
            }
        }
        recipients.addAll(addThesisSupport());
        return recipients;
    }
}
