package se.su.dsv.scipro.finalseminar;

import com.mysema.query.annotations.QueryInit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.LazyDeletableDomainObject;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.*;

@Data
@EqualsAndHashCode(of = "id", callSuper = true)
@Entity
@Table(name = "final_seminar")
@Cacheable(true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class FinalSeminar extends LazyDeletableDomainObject {
    public static final String U_SINGULAR = "Final seminar";

    public static final int DEFAULT_MAX_OPPONENTS = 2;
    public static final int DEFAULT_MAX_PARTICIPANTS = 5;
    public static final String FINAL_SEMINAR = "finalSeminar";

    @Id
    @GeneratedValue
    private Long id;

    @OneToOne(optional = false)
    @QueryInit({"projectType", "headSupervisor"})
    private Project project;

    @Basic(optional = false)
    private Date startDate;

    @Basic(optional = false)
    private String room;

    @OneToMany(mappedBy = FINAL_SEMINAR, orphanRemoval = true, cascade = CascadeType.ALL)
    private Set<FinalSeminarActiveParticipation> activeParticipations = new HashSet<>();

    @OneToMany(mappedBy = FINAL_SEMINAR, orphanRemoval = true, cascade = CascadeType.ALL)
    private Set<FinalSeminarOpposition> oppositions = new HashSet<>();

    @OneToMany(mappedBy = FINAL_SEMINAR, orphanRemoval = true, cascade = CascadeType.ALL)
    private Set<FinalSeminarRespondent> respondents = new HashSet<>();

    /*
     * Cascading delete, set document to nul will delete the filedescription but
     * not the actual file. Use FinarSeminarUploadController.deleteSeminarReport
     * to delete the document
     */
    @OneToOne(optional = true, orphanRemoval = true, cascade = CascadeType.ALL)
    private FileDescription document;

    private Date documentUploadDate;

    @ManyToOne(optional = false)
    private Language presentationLanguage;

    @ManyToOne(optional = false)
    private Language reportLanguage;

    private int maxOpponents = DEFAULT_MAX_OPPONENTS;
    private int maxParticipants = DEFAULT_MAX_PARTICIPANTS;

    @Basic
    @Getter
    private String creationReason;

    public FinalSeminar() {

    }

    public FinalSeminar(int maxOpponents, int maxParticipants) {
        this.maxOpponents = maxOpponents;
        this.maxParticipants = maxParticipants;
    }

    public FinalSeminar(Project project) {
        this(project.getProjectTypeMaxOpponents(), project.getProjectTypeMaxActiveParticipants());
        this.project = project;
    }

    public void setStartDate(Date startDate) {
        this.startDate = (Date) startDate.clone();
    }

    public Project getProject() {
        return project;
    }

    public void setActiveParticipations(Collection<FinalSeminarActiveParticipation> activeParticipations) {
        this.activeParticipations.clear();
        this.activeParticipations.addAll(activeParticipations);
    }

    public Set<FinalSeminarActiveParticipation> getActiveParticipations() {
        return Collections.unmodifiableSet(activeParticipations);
    }

    public void addActiveParticipant(FinalSeminarActiveParticipation participation) {
        this.activeParticipations.add(participation);
    }

    public void removeActiveParticipant(FinalSeminarActiveParticipation participation) {
        this.activeParticipations.remove(participation);
    }

    public void setOppositions(Collection<FinalSeminarOpposition> oppositions) {
        this.oppositions.clear();
        this.oppositions.addAll(oppositions);
    }

    public Set<FinalSeminarOpposition> getOppositions() {
        return Collections.unmodifiableSet(oppositions);
    }

    public void addOpposition(FinalSeminarOpposition opposition) {
        this.oppositions.add(opposition);
    }

    public int getMinOpponents() {
        return project.getMinOpponentsOnFinalSeminar();
    }

    public int getMinActiveParticipants() {
        return project.getMinFinalSeminarActiveParticipation();
    }

    public List<Member> getMembers() {
        List<Member> members = project.getMembers();

        for (FinalSeminarActiveParticipation participation : getActiveParticipations()) {
            members.add(new Member(participation.getUser(), Member.Type.ACTIVE_PARTICIPANT));
        }

        for (FinalSeminarOpposition opposition : getOppositions()) {
            members.add(new Member(opposition.getUser(), Member.Type.OPPONENT));
        }

        return members;
    }

    public User getHeadSupervisor() {
        return getProject().getHeadSupervisor();
    }

    public ProjectType getProjectType() {
        return project.getProjectType();
    }

    @SuppressWarnings("UnusedDeclaration")
    public String getProjectTitle() {
        return project.getTitle();
    }

    public void setCancelled(boolean cancelled) {
        setDeleted(cancelled);
    }

    public boolean isCancelled() {
        return isDeleted();
    }

    public void removeOpposition(FinalSeminarOpposition opposition) {
        this.oppositions.remove(opposition);
    }

    @SuppressWarnings("UnusedDeclaration")
    public Collection<User> getNotGradedOpponents() {
        return getNotGradedParticipations(oppositions);
    }

    @SuppressWarnings("UnusedDeclaration")
    public Collection<User> getNotGradedActiveParticipations() {
        return getNotGradedParticipations(activeParticipations);
    }

    @SuppressWarnings("UnusedDeclaration")
    public Collection<User> getNotGradedRespondents() {
        return getNotGradedParticipations(respondents);
    }

    private Collection<User> getNotGradedParticipations(Set<? extends FinalSeminarParticipation> participations) {
        List<User> result = new ArrayList<>();
        for (FinalSeminarParticipation participation : participations) {
            if(participation.getGrade() == null) {
                result.add(participation.getUser());
            }
        }
        return result;
    }

    public Set<User> getOpponents(){
        Set<User> opponents = new HashSet<>();
        for (FinalSeminarOpposition fso : oppositions){
            opponents.add(fso.getUser());
        }
        return opponents;
    }

    public Set<User> getActiveParticipants(){
        Set<User> activeParticipants = new HashSet<>();
        for (FinalSeminarActiveParticipation fsap : activeParticipations){
            activeParticipants.add(fsap.getUser());
        }
        return activeParticipants;
    }

    public void removeActiveParticipant(User user) {
        Iterator<FinalSeminarActiveParticipation> iterator = activeParticipations.iterator();
        while (iterator.hasNext()) {
            FinalSeminarActiveParticipation next = iterator.next();
            if (next.getUser().equals(user)) {
                iterator.remove();
            }
        }
    }
}