package se.su.dsv.scipro.match;

import com.google.inject.persist.Transactional;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

import static com.mysema.query.support.Expressions.allOf;

class ApplicationPeriodServiceImpl extends AbstractServiceImpl<ApplicationPeriod, Long> implements ApplicationPeriodService {

    @Inject
    ApplicationPeriodServiceImpl(Provider<EntityManager> em) {
        super(em, ApplicationPeriod.class, QApplicationPeriod.applicationPeriod);
    }

    @Override
    public List<ApplicationPeriod> getCurrentPeriods() {
        return getCurrentPeriods(new Date());
    }

    private List<ApplicationPeriod> getCurrentPeriods(Date currentDate) {
        return findAll(startedBefore(currentDate).and(endedAfter(currentDate)));
    }

    @Override
    public List<ApplicationPeriod> getCurrentPeriods(User student) {
        Date currentDate = new Date();
        Set<ApplicationPeriod> phs = new HashSet<>(getCurrentPeriods(currentDate));
        QApplicationPeriodExemption exemption = QApplicationPeriod.applicationPeriod.applicationPeriodExemptions.any();
        Iterable<ApplicationPeriod> periodsWithExemption =
                findAll(allOf(exemption.user.eq(student), exemption.start.before(new Date()), exemption.until.after(new Date())));
        for (ApplicationPeriod applicationPeriod : periodsWithExemption) {
            phs.add(applicationPeriod);
        }
        return new ArrayList<>(phs);
    }

    @Override
    public Collection<ApplicationPeriod> getPreviousPeriods(ApplicationPeriod applicationPeriod, int limit) {
        return findAll(startedBefore(applicationPeriod.getStartDate()),
                new PageRequest(0, limit, new Sort(Sort.Direction.DESC, "startDate")));
    }

    private List<ApplicationPeriod> getPreviousPeriodsByProjectType(ProjectType pc) {
        return findAll(endedBefore(new Date()).and(haveProjectType(pc)));
    }

    @Override
    @Transactional
    public ApplicationPeriod getCurrentPeriod(ProjectType pc) {
        Date currentDate = new Date();
        return findOne(startedBefore(currentDate).and(endedAfter(currentDate)).and(haveProjectType(pc)));
    }

    @Override
    public ApplicationPeriod getCurrentPeriod(ProjectType pc, User student) {
        Date currentDate = new Date();
        QApplicationPeriodExemption exemption = QApplicationPeriod.applicationPeriod.applicationPeriodExemptions.any();
        Iterable<ApplicationPeriod> exemptedPeriods = findAll(QApplicationPeriod.applicationPeriod.projectTypes.any().projectType.eq(pc)
                .and(exemption.user.eq(student).and(exemption.start.before(currentDate)).and(exemption.until.after(currentDate))));
        Iterator<ApplicationPeriod> iterator = exemptedPeriods.iterator();
        if (iterator.hasNext()) {
            return iterator.next();
        }
        return getCurrentPeriod(pc);
    }

    @Override
    public ApplicationPeriod getPreviousPeriodByProjectType(ProjectType projectType) {
        List<ApplicationPeriod> periods = getPreviousPeriodsByProjectType(projectType);
        if (periods.isEmpty()) {
            return null;
        }
        ApplicationPeriod previous = periods.get(0);
        for (ApplicationPeriod ap : periods) {
            if (ap.getEndDate().after(previous.getEndDate())) {
                previous = ap;
            }
        }
        return previous;

    }

    @Override
    public boolean courseStartHasPassed(ApplicationPeriod ap) {
        return ap.getCourseStartDate().compareTo(new Date()) < 0;
    }

    @Override
    public boolean endDateIsReached(ApplicationPeriod ap) {
        return ap.getEndDate().compareTo(new Date()) < 0;
    }

    @Override
    public boolean isOpen(ApplicationPeriod ap) {
        Date now = new Date();
        return ap.getStartDate().before(now) && ap.getEndDate().after(now);
    }

    @Override
    public boolean hasOpenExemption(ApplicationPeriod ap, User student) {
        for (ApplicationPeriodExemption ape : ap.getApplicationPeriodExemptions()) {
            Date now = new Date();
            if (ape.getUser().equals(student) && ape.getStart().before(now) && ape.getUntil().after(now)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean hasExemption(User user, ApplicationPeriod applicationPeriod) {
        ApplicationPeriod reloaded = findOne(applicationPeriod.getId());
        for (ApplicationPeriodExemption ape : reloaded.getApplicationPeriodExemptions()) {
            if (ape.getUser().equals(user)) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean doesPeriodOverlap(final ApplicationPeriod newPeriod) {
        for(ApplicationPeriod foundPeriod : findAll()) {
            if(isEditing(newPeriod, foundPeriod)) {
                continue;
            }

            if(projectTypeMatch(foundPeriod.getProjectTypes(), newPeriod.getProjectTypes()) &&
                    periodsOverlap(foundPeriod.getStartDate(), foundPeriod.getEndDate(), newPeriod.getStartDate(), newPeriod.getEndDate())) {
                return true;
            }
        }
        return false;
    }

    private boolean periodsOverlap(Date p1Start, Date p1End, Date p2Start, Date p2End) {
        return p2Start.before(p1End) && p2End.after(p1Start);
    }

    private boolean isEditing(final ApplicationPeriod newPeriod, final ApplicationPeriod foundPeriod) {
        return (newPeriod.getId() != null &&  newPeriod.equals(foundPeriod));
    }

    private boolean projectTypeMatch(final Set<ProjectType> fromNewPeriod, final Set<ProjectType> fromOldPeriod) {
        for (ProjectType projectType : fromNewPeriod) {
            if(fromOldPeriod.contains(projectType)) {
                return true;
            }
        }
        return false;
    }

    public static BooleanExpression haveProjectType(ProjectType pc) {
        return QApplicationPeriod.applicationPeriod.projectTypes.any().projectType.eq(pc);
    }

    public static BooleanExpression endedBefore(Date currentDate) {
        return QApplicationPeriod.applicationPeriod.endDate.before(currentDate);
    }

    public static BooleanExpression startedBefore(Date currentDate) {
        return QApplicationPeriod.applicationPeriod.startDate.before(currentDate);
    }

    public static BooleanExpression endedAfter(Date currentDate) {
        return QApplicationPeriod.applicationPeriod.endDate.after(currentDate);
    }

}
