package se.su.dsv.scipro.milestones.service.impl;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.jpa.JPASubQuery;
import com.mysema.query.types.Predicate;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.QMilestone;
import se.su.dsv.scipro.milestones.dataobjects.QMilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.MileStoneEvent;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.List;

import static com.mysema.query.support.Expressions.allOf;
import static com.mysema.query.support.Expressions.anyOf;

public class MilestoneServiceImpl extends AbstractServiceImpl<Milestone, Long> implements MileStoneService {

    public static final long EMPTY_RESULT = 0L;
    private final NotificationController notificationController;
    private final ProjectService projectService;
    private final MilestoneActivityTemplateService milestoneActivityTemplateService;

    @Inject
    public MilestoneServiceImpl(final NotificationController notificationController,
                                final ProjectService projectService,
                                final MilestoneActivityTemplateService milestoneActivityTemplateService,
                                Provider<EntityManager> em) {
        super(em, Milestone.class, QMilestone.milestone);
        this.notificationController = notificationController;
        this.projectService = projectService;
        this.milestoneActivityTemplateService = milestoneActivityTemplateService;
    }

    @Override
    public long countProjectMilestones(Project project, boolean confirmed) {
        QMilestone milestone = QMilestone.milestone;
        return count(allOf(milestone.project.eq(project), milestone.confirmed.eq(confirmed), milestone.user.isNull(), milestone.activity.deleted.isFalse()));
    }

    private long countAllMilestones(Project project, boolean confirmed){
        QMilestone milestone = QMilestone.milestone;
        return count(allOf(milestone.project.eq(project), milestone.confirmed.eq(confirmed)));
    }

    @Override
    public long countMileStones(Project project, User student, boolean confirmed) {
        QMilestone milestone = QMilestone.milestone;
        return count(allOf(milestone.project.eq(project), milestone.confirmed.eq(confirmed), milestone.user.eq(student)));
    }

    @Override
    public void setConfirmed(Project project, MilestoneActivityTemplate activity, boolean confirmed) {
        if (confirmed) {
            for (MilestoneActivityTemplate earlierActivity : getEarlierIncompleteProjectMilestoneActivities(project, activity)) {
                setConfirmed(project, null, earlierActivity, true);
            }
        }
        setConfirmed(project, null, activity, confirmed);
    }

    private Iterable<? extends MilestoneActivityTemplate> getEarlierIncompleteProjectMilestoneActivities(final Project project, final MilestoneActivityTemplate activity) {
        QMilestoneActivityTemplate q = QMilestoneActivityTemplate.milestoneActivityTemplate;
        QMilestone milestone = QMilestone.milestone;
        return from(q).where(
                q.deleted.isFalse(),
                q.milestonePhaseTemplate.deleted.isFalse(),
                q.type.eq(MilestoneActivityTemplate.Type.PROJECT),
                q.projectTypes.contains(project.getProjectType()),
                anyOf(
                        allOf(q.sortOrder.lt(activity.getSortOrder()), q.milestonePhaseTemplate.eq(activity.getMilestonePhaseTemplate())),
                        q.milestonePhaseTemplate.sortOrder.lt(activity.getMilestonePhaseTemplate().getSortOrder())

                ),
                new JPASubQuery().from(milestone)
                        .where(milestone.activity.eq(q), milestone.confirmed.isTrue(), milestone.project.eq(project))
                        .notExists())
                .orderBy(q.milestonePhaseTemplate.sortOrder.asc(), q.sortOrder.asc())
                .list(q);
    }

    @Override
    public void setConfirmed(Project project, User student, MilestoneActivityTemplate activity, boolean confirmed) {

        Project localProject = project;

        if (!localProject.hasModule(ProjectModule.MILESTONES)) {
            return;
        }

        Milestone milestone = student == null ? getMileStone(localProject, activity) : getMileStone(localProject, student, activity);
        milestone.setConfirmed(confirmed);
        milestone = save(milestone);

        long localProjectMilestoneActivityTemplateCount = milestoneActivityTemplateService.countActivities(MilestoneActivityTemplate.Type.PROJECT, localProject.getProjectType(), localProject.getDateCreated());
        long individualStudentActivityMilestoneTemplateCount = milestoneActivityTemplateService.countActivities(MilestoneActivityTemplate.Type.STUDENT, localProject.getProjectType(), localProject.getDateCreated());
        long totalCompletedMilestones = countAllMilestones(localProject, true);

        if ((localProjectMilestoneActivityTemplateCount+(individualStudentActivityMilestoneTemplateCount*localProject.getProjectParticipants().size()))==totalCompletedMilestones){ //all milestones confirmed
            localProject = projectService.complete(localProject);
        }

        NotificationSource source = new NotificationSource();
        source.setMessage(activity.getTitle());
        if (student == null) {
            notificationController.notifyProject(localProject, confirmed ? ProjectEvent.Event.MILESTONE_CONFIRMED : ProjectEvent.Event.MILESTONE_REVOKED, source);
        } else {
            notificationController.notifyMileStone(milestone, confirmed ? MileStoneEvent.Event.MILESTONE_CONFIRMED : MileStoneEvent.Event.MILESTONE_REVOKED, source);
        }

    }

    @Override
    public boolean isConfirmed(Project project, MilestoneActivityTemplate activity) {
        return getMileStone(project, activity).isConfirmed();
    }

    @Override
    public boolean isConfirmed(Project project, User student, MilestoneActivityTemplate activity) {
        return getMileStone(project, student, activity).isConfirmed();
    }

    @Override
    public Milestone getMileStone(Project project, MilestoneActivityTemplate activity) {
        QMilestone qMilestone = QMilestone.milestone;
        Milestone milestone = findOne(allOf(qMilestone.project.eq(project), qMilestone.activity.eq(activity), qMilestone.user.isNull()));
        if (milestone == null) {
            milestone = new Milestone(project, activity);
        }
        return milestone;
    }

    @Override
    public Milestone getMileStone(Project project, User student, MilestoneActivityTemplate activity) {
        QMilestone qMilestone = QMilestone.milestone;
        Milestone milestone = findOne(allOf(qMilestone.project.eq(project), qMilestone.activity.eq(activity), qMilestone.user.eq(student)));
        if (milestone == null) {
            milestone = new Milestone(project, student, activity);
        }
        return milestone;
    }

    @Override
    public long countCompletedMileStones(MilestoneActivityTemplate activity, Filter filter) {
        if (filter.getProjectTypes().isEmpty()) {
            return EMPTY_RESULT;
        }
        QMilestone mileStone = QMilestone.milestone;
        return count(
                mileStone.activity.eq(activity)
                        .and(mileStone.confirmed.isTrue()
                                .and(toPredicate(filter))));
    }

    @Override
    public long countCompletedMileStones(MilestoneActivityTemplate activity, Filter filter, ProjectType projectType) {
        QMilestone mileStone = QMilestone.milestone;
        BooleanBuilder bb = new BooleanBuilder();
        bb.and(mileStone.activity.eq(activity));
        bb.and(mileStone.confirmed.isTrue());
        bb.and(mileStone.project.projectType.eq(projectType));
        bb.and(mileStone.lastModified.before(filter.getMileStoneCompletedBefore()));
        bb.and(withProjectStatus(filter.getProjectStatuses()));
        startedBeforeFilter(filter, mileStone, bb);
        return count(bb);
    }

    @Override
    public List<Milestone> findCompletedMilestones(MilestoneActivityTemplate activity, Filter filter, Pageable pageable) {
        QMilestone mileStone = QMilestone.milestone;
        return findAll(mileStone.activity.eq(activity).and(mileStone.confirmed.isTrue()
                .and(toPredicate(filter))), pageable);
    }

    private static Predicate toPredicate(Filter filter) {
        QMilestone mileStone = QMilestone.milestone;
        BooleanBuilder booleanBuilder = new BooleanBuilder();
        booleanBuilder.and(withProjectType(filter.getProjectTypes()));
        booleanBuilder.and(mileStone.lastModified.before(filter.getMileStoneCompletedBefore()));
        booleanBuilder.and(withProjectStatus(filter.getProjectStatuses()));
        startedBeforeFilter(filter, mileStone, booleanBuilder);
        return booleanBuilder;
    }

    private static void startedBeforeFilter(Filter filter, QMilestone mileStone, BooleanBuilder booleanBuilder) {
        if (filter.getProjectStartedAfter() != null) {
            booleanBuilder.and(mileStone.project.dateCreated.after(filter.getProjectStartedAfter()));
        }
        if (filter.getProjectStartedBefore() != null) {
            booleanBuilder.and(mileStone.project.dateCreated.before(filter.getProjectStartedBefore()));
        }
    }

    private static BooleanExpression withProjectStatus(Collection<ProjectStatus> projectStatuses) {
        if (projectStatuses.isEmpty()) {
            return QMilestone.milestone.isNull();
        } else {
            return QMilestone.milestone.project.projectStatus.in(projectStatuses);
        }
    }

    private static BooleanExpression withProjectType(Collection<ProjectType> projectTypes) {
        if (projectTypes.isEmpty()) {
            return QMilestone.milestone.isNull();
        } else {
            return QMilestone.milestone.project.projectType.in(projectTypes);
        }
    }
}
