package se.su.dsv.scipro.peer;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.Transactional;
import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static com.mysema.query.types.ExpressionUtils.allOf;

public class PeerRequestServiceImpl extends AbstractServiceImpl<PeerRequest, Long> implements PeerRequestService {

    private final EventBus eventBus;

    @Inject
    public PeerRequestServiceImpl(Provider<EntityManager> em, final EventBus eventBus) {
        super(em, PeerRequest.class, QPeerRequest.peerRequest);
        this.eventBus = eventBus;
    }

    @Override
    public List<PeerRequest> findAll(Filter params, Pageable pageable) {
        return findAll(fromParams(params), pageable);
    }

    @Override
    public long count(Filter params) {
        return count(fromParams(params));
    }

    private BooleanBuilder fromParams(Filter params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getSupervisor() != null) {
            bb.and(supervisorFilter(params.getSupervisor()));
        }
        if (params.getAuthor() != null) {
            bb.and(authorFilter(params.getAuthor()));
        }
        if (params.getStatuses() != null) {
            bb.and(statusFilter(params.getStatuses()));
        }
        if (params.getCreatedAfter() != null) {
            bb.and(createdAfterFilter(params.getCreatedAfter()));
        }
        if (params.getCreatedBefore() != null) {
            bb.and(createdBeforeFilter(params.getCreatedBefore()));
        }
        if (params.getProject() != null) {
            bb.and(projectFilter(params.getProject()));
        }
        if (params.getProjectTypes() != null) {
            bb.and(projectTypesFilter(params.getProjectTypes()));
        }
        if (params.getLanguage() != null) {
            bb.and(languageFilter(params.getLanguage()));
        }
        return bb;
    }

    private BooleanExpression supervisorFilter(User supervisor) {
        return QPeerRequest.peerRequest.project.headSupervisor.eq(supervisor);
    }

    private BooleanExpression authorFilter(User author) {
        return QPeerRequest.peerRequest.requester.eq(author);
    }

    private BooleanBuilder statusFilter(Collection<RequestStatus> statuses) {
        BooleanBuilder e = new BooleanBuilder();
        if (statuses != null && !statuses.isEmpty()) {
            for (RequestStatus status : statuses) {
                e.or(hasStatus(status));
            }
            return e;
        } else {
            e.and(QPeerRequest.peerRequest.status.isNull());
            return e;
        }
    }

    private BooleanExpression languageFilter(Language language) {
        return QPeerRequest.peerRequest.language.eq(language);
    }

    private BooleanBuilder projectTypesFilter(Collection<ProjectType> projectTypes) {
        BooleanBuilder e = new BooleanBuilder();
        if (projectTypes != null && !projectTypes.isEmpty()) {
            for (ProjectType projectType : projectTypes) {
                e.or(hasProjectType(projectType));
            }
            return e;
        } else {
            e.and(QPeerRequest.peerRequest.project.projectType.isNull());
            return e;
        }
    }

    private BooleanExpression createdAfterFilter(Date createdAfter) {
        return QPeerRequest.peerRequest.dateCreated.after(createdAfter);
    }

    private BooleanExpression createdBeforeFilter(Date createdBefore) {
        return QPeerRequest.peerRequest.dateCreated.before(createdBefore);
    }

    private BooleanExpression projectFilter(Project project) {
        return QPeerRequest.peerRequest.project.eq(project);
    }

    @Override
    public List<PeerRequest> findAvailableRequests(User user, RequestStatus status, ProjectType pc, Pageable pageable) {
        if (status != null && pc != null) {
            return findAll(hasStatus(status).and(projectTypeIs(pc)
                    .and(QPeerRequest.peerRequest.project.projectStatus.eq(ProjectStatus.ACTIVE)))
                    .and(QPeerRequest.peerRequest.project.projectParticipants.contains(user).not()), pageable);
        } else {
            return findAll(pageable);
        }
    }

    @Override
    public List<PeerRequest> findByStatus(RequestStatus status) {
        return findAll(QPeerRequest.peerRequest.status.eq(status));
    }

    @Override
    public List<PeerRequest> findByRequesterAndProject(User requester, Project project) {
        return from(QPeerRequest.peerRequest).where(allOf(isRequester(requester), QPeerRequest.peerRequest.project.eq(project)))
                .orderBy(QPeerRequest.peerRequest.dateCreated.desc()).list(QPeerRequest.peerRequest);
    }

    public long countByStatus(RequestStatus status) {
        return count(hasStatus(status));
    }


    @Override
    public long countAvailableRequests(User user, RequestStatus status, ProjectType pc) {
        if (status != null && pc != null) {
            return count(hasStatus(status).and(projectTypeIs(pc))
                    .and(QPeerRequest.peerRequest.project.projectStatus.eq(ProjectStatus.ACTIVE))
                    .and(QPeerRequest.peerRequest.project.projectParticipants.contains(user).not()));
        } else {
            return count();
        }
    }

    private BooleanExpression isRequester(User student) {
        if (student != null) {
            return QPeerRequest.peerRequest.requester.eq(student);
        } else {
            return QPeerRequest.peerRequest.requester.isNotNull();
        }
    }

    private BooleanExpression hasStatus(RequestStatus status) {
        return QPeerRequest.peerRequest.status.eq(status);
    }

    private BooleanExpression hasProjectType(ProjectType projectType) {
        return QPeerRequest.peerRequest.project.projectType.eq(projectType);
    }

    private BooleanExpression projectTypeIs(ProjectType pc) {
        return QPeerRequest.peerRequest.project.projectType.eq(pc);
    }

    @Transactional
    @Override
    public void deleteRequest(PeerRequest req) {
        PeerRequest request = findOne(req.getId());
        List<PeerReview> reviews = request.getPeerReviews();
        if (request.getStatus() == RequestStatus.WAITING && reviews.isEmpty()) {
            delete(request);
        } else if (request.getStatus() == RequestStatus.WAITING) {
            request.setStatus(RequestStatus.CANCELED);
            save(request);
            eventBus.post(new PeerRequestCancelledEvent(request));
        }
    }

    @Override
    public List<PeerRequest> findByChecklistTemplate(ChecklistTemplate checklistTemplate) {
        return findAll(QPeerRequest.peerRequest.checklistTemplate.eq(checklistTemplate));
    }



}
