package se.su.dsv.scipro.peer;

import com.mysema.query.annotations.QueryInit;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import se.su.dsv.scipro.checklist.ChecklistAnswerEnum;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Data
@Entity
@Table(name="peer_review")
@Cacheable(true)
@EqualsAndHashCode(of = {"id"}, callSuper = false)
@SuppressWarnings("PMD.UnusedPrivateField")
public class PeerReview extends DomainObject implements Commentable {
    public static final int MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS = 100;

    public static enum ReviewStatus {
		IN_PROGRESS, COMPLETED, EXPIRED
	}
	
	@Id
	@GeneratedValue
	private Long id;
	
	@ManyToOne(optional=false)
	@QueryInit("*.*")
	private User reviewer;
	
	@ManyToOne(optional=false)
	@QueryInit({"headSupervisor", "projectType"})
	private Project project;
	
	@ManyToOne(optional=false)
	@QueryInit({"project.headSupervisor","requester.user", "language", "checklistTemplate"})
	private PeerRequest peerRequest;	
	
	@OneToMany(mappedBy="peerReview", orphanRemoval=true, cascade=CascadeType.ALL)
	@OrderBy("id")
	private List<Answer> answers = new ArrayList<>();

	@OneToOne(optional=true, orphanRemoval = true, cascade = CascadeType.ALL)
	private FileDescription file;
	
	@Lob
	private String comment;

    @Enumerated(EnumType.STRING)
    private ReviewStatus status = ReviewStatus.IN_PROGRESS;

	@Override
	public final String getCommentKey() {
		return PeerReview.class.getCanonicalName();
	}

    public List<Member> getMembers() {
        List<Member> members = peerRequest.getMembers();
        members.add(new Member(reviewer, Member.Type.PEER_REVIEWER));
        members.add(new Member(project.getHeadSupervisor(), Member.Type.SUPERVISOR));
        return members;
    }

    public Date getDeadline() {
        DateTime deadline = new DateTime(getDateCreated()).plusDays(getNumDaysToSubmitPeerReview());
        return deadline.toDate();
    }

    private int getNumDaysToSubmitPeerReview() {
        return getProject().getProjectType().getProjectTypeSettings().getNumDaysToSubmitPeerReview();
    }

    public boolean isLate() {
        return new Date().after(getDeadline());
    }

    public ReviewStatus getStatus() {
        return status;
    }

    public boolean isExpired() {
        return status == ReviewStatus.EXPIRED;
    }

    public boolean isCompleted() {
        return status == ReviewStatus.COMPLETED;
    }

    public void submit() {
        for (Answer answer : answers) {
            if (StringUtils.isEmpty(answer.getMotivation()) || answer.getAnswer() == ChecklistAnswerEnum.NO_ANSWER || answer.getAnswer() == null) {
                throw new MissingAnswerException();
            }
        }
        if (answers.isEmpty() && (StringUtils.isEmpty(comment) || comment.length() < MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS)) {
            throw new TooShortCommentException();
        }
        status = isLate() ? ReviewStatus.EXPIRED : ReviewStatus.COMPLETED;
    }

    public void expire() {
        status = ReviewStatus.EXPIRED;
    }

    public void addAnswer(String question) {
        this.answers.add(new Answer(this, question));
    }
}
