package se.su.dsv.scipro.peer;

import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.Predicate;
import com.mysema.query.types.expr.BooleanExpression;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

public class PeerReviewServiceImpl extends AbstractServiceImpl<PeerReview, Long> implements PeerReviewService {

    public static final long EMPTY_RESULT = 0L;

    @Inject
    public PeerReviewServiceImpl(Provider<EntityManager> em) {
        super(em, PeerReview.class, QPeerReview.peerReview);
    }

    @Override
    public List<PeerReview> findAll(Filter params, Pageable pageable) {
        return findAll(fromParams(params), pageable);
    }

    @Override
    public long count(Filter params) {
        return count(fromParams(params));
    }

    @Override
    public List<PeerReview> findByReviewerAndProject(User student, Project project) {
        return from(QPeerReview.peerReview).where(reviewerIs(student)
                .and(reviewProjectIs(project).and(hasStatus(PeerReview.ReviewStatus.EXPIRED).not())))
                .orderBy(QPeerReview.peerReview.dateCreated.desc()).list(QPeerReview.peerReview);
    }

    private BooleanBuilder fromParams(Filter params) {
        BooleanBuilder bb = new BooleanBuilder();
        if (params.getSupervisor() != null) {
            bb.and(supervisorFilter(params.getSupervisor()));
        }
        if (params.getAuthor() != null) {
            bb.and(authorFilter(params.getAuthor()));
        }
        if (params.getStatuses() != null) {
            bb.and(statusFilter(params.getStatuses()));
        }
        if (params.getCreatedAfter() != null) {
            bb.and(createdAfterFilter(params.getCreatedAfter()));
        }
        if (params.getCreatedBefore() != null) {
            bb.and(createdBeforeFilter(params.getCreatedBefore()));
        }
        if (params.getAuthorProject() != null) {
            bb.and(authorProjectFilter(params.getAuthorProject()));
        }
        if (params.getProjectTypes() != null) {
            bb.and(projectTypesFilter(params.getProjectTypes()));
        }
        if (params.getLanguage() != null) {
            bb.and(languageFilter(params.getLanguage()));
        }
        return bb;
    }

    private BooleanExpression authorProjectFilter(Project authorProject) {
        return QPeerReview.peerReview.project.eq(authorProject);
    }

    private BooleanExpression supervisorFilter(User supervisor) {
        return QPeerReview.peerReview.project.headSupervisor.eq(supervisor);
    }

    private BooleanExpression authorFilter(User author) {
        return QPeerReview.peerReview.reviewer.eq(author);
    }

    private BooleanExpression createdAfterFilter(Date createdAfter) {
        return QPeerReview.peerReview.dateCreated.after(createdAfter);
    }

    private BooleanExpression createdBeforeFilter(Date createdBefore) {
        return QPeerReview.peerReview.dateCreated.before(createdBefore);
    }

    @Override
    public List<PeerReview> findAllCompleted(Pageable pageable) {
        return findAll(hasStatus(PeerReview.ReviewStatus.COMPLETED), pageable);
    }

    @Override
    public List<PeerReview> findInProgressReviews(User student, Project project) {
        return findAll(reviewerIs(student).and(reviewProjectIs(project).and(hasStatus(PeerReview.ReviewStatus.IN_PROGRESS))));
    }

    @Override
    public List<PeerReview> findAllReviewsInProgress() {
        return findAll(hasStatus(PeerReview.ReviewStatus.IN_PROGRESS));
    }

    @Override
    public List<PeerReview> findReceivedCompletedReviews(Project project) {
        return findAll(requestProjectIs(project).and(hasStatus(PeerReview.ReviewStatus.COMPLETED)));
    }

    @Override
    public List<PeerReview> findCompletedReviews(Project project) {
        return findAll(reviewProjectIs(project).and(hasStatus(PeerReview.ReviewStatus.COMPLETED)));
    }

    @Override
    public Long countByStatus(PeerReview.ReviewStatus status) {
        return count(hasStatus(status));
    }

    @Override
    public Long countByStatus(PeerReview.ReviewStatus status, Filter filter) {
        if (filter.getProjectTypes().isEmpty()) {
            return EMPTY_RESULT;
        }
        return count(
                QPeerReview.peerReview.status.eq(status)
                        .and(toPredicate(filter)));
    }

    @Override
    public Long countByStatus(PeerReview.ReviewStatus status, ProjectType type, Filter filter) {
        if (filter.getProjectTypes().isEmpty()) {
            return EMPTY_RESULT;
        }
        return count(
                QPeerReview.peerReview.status.eq(status)
                        .and(toPredicate(filter))
                        .and(projectTypesFilter(Collections.singletonList(type))));
    }

    @Override
    public Long countByStatusAndNoCheckListTemplate(Filter filter, PeerReview.ReviewStatus status) {
        QPeerReview peerReview = QPeerReview.peerReview;
        BooleanBuilder booleanBuilder = new BooleanBuilder();
        booleanBuilder.and(toPredicate(filter));
        booleanBuilder.and(peerReview.status.eq(status));
        booleanBuilder.and(peerReview.peerRequest.checklistTemplate.isNull());
        return count(booleanBuilder);
    }

    private static Predicate toPredicate(Filter filter) {
        QPeerReview peerReview = QPeerReview.peerReview;
        BooleanBuilder booleanBuilder = new BooleanBuilder();
        booleanBuilder.and(withProjectType(filter.getProjectTypes()));
        booleanBuilder.and(withSupervisor(filter.getSupervisor()));
        booleanBuilder.and(withAuthor(filter.getAuthor()));
        booleanBuilder.and(withLanguage(filter.getLanguage()));
        startedBeforeFilter(filter, peerReview, booleanBuilder);
        return booleanBuilder;
    }

    private static Predicate withAuthor(User author) {
        if (author == null) {
            return QPeerReview.peerReview.isNotNull();
        } else {
            return QPeerReview.peerReview.reviewer.eq(author);
        }
    }

    private static Predicate withSupervisor(User supervisor) {
        if (supervisor == null) {
            return QPeerReview.peerReview.isNotNull();
        } else {
            return QPeerReview.peerReview.project.headSupervisor.eq(supervisor);
        }
    }

    private static Predicate withLanguage(Language language) {
        if (language == null) {
            return QPeerReview.peerReview.isNotNull();
        } else {
            return QPeerReview.peerReview.peerRequest.language.eq(language);
        }
    }

    private static BooleanExpression withProjectType(Collection<ProjectType> projectTypes) {
        if (projectTypes.isEmpty()) {
            return QPeerReview.peerReview.isNull();
        } else {
            return QPeerReview.peerReview.project.projectType.in(projectTypes);
        }
    }

    private static void startedBeforeFilter(PeerReviewService.Filter filter, QPeerReview peerReview, BooleanBuilder booleanBuilder) {
        if (filter.getCreatedAfter() != null) {
            booleanBuilder.and(peerReview.dateCreated.after(filter.getCreatedAfter()));
        }
        if (filter.getCreatedBefore() != null) {
            booleanBuilder.and(peerReview.dateCreated.before(filter.getCreatedBefore()));
        }
    }

    public static BooleanExpression reviewerIs(User student) {
        if (student != null) {
            return QPeerReview.peerReview.reviewer.eq(student);
        } else {
            return QPeerReview.peerReview.reviewer.isNotNull();
        }
    }

    public static BooleanExpression reviewProjectIs(Project project) {
        return QPeerReview.peerReview.project.eq(project);
    }

    public static BooleanExpression requestProjectIs(Project project) {
        QPeerReview qPeerReview = QPeerReview.peerReview;
        return qPeerReview.peerRequest.project.eq(project);
    }

    private BooleanBuilder statusFilter(Collection<PeerReview.ReviewStatus> statuses) {
        BooleanBuilder e = new BooleanBuilder();
        if (statuses != null && !statuses.isEmpty()) {
            for (PeerReview.ReviewStatus status : statuses) {
                e.or(hasStatus(status));
            }
            return e;
        } else {
            e.and(QPeerReview.peerReview.status.isNull());
            return e;
        }
    }

    private Predicate languageFilter(Language language) {
        if (language != null) {
            return QPeerReview.peerReview.peerRequest.language.eq(language);
        } else return new BooleanBuilder();
    }

    private BooleanBuilder projectTypesFilter(Collection<ProjectType> projectTypes) {
        BooleanBuilder e = new BooleanBuilder();
        if (projectTypes != null && !projectTypes.isEmpty()) {
            for (ProjectType projectType : projectTypes) {
                e.or(hasProjectType(projectType));
            }
            return e;
        } else {
            e.and(QPeerReview.peerReview.project.projectType.isNull());
            return e;
        }
    }

    public static BooleanExpression hasStatus(PeerReview.ReviewStatus status) {
        return QPeerReview.peerReview.status.eq(status);
    }

    public static BooleanExpression hasProjectType(ProjectType projectType) {
        return QPeerReview.peerReview.project.projectType.eq(projectType);
    }


}
