package se.su.dsv.scipro.project;

import com.google.common.eventbus.EventBus;
import com.google.inject.persist.Transactional;
import com.mysema.query.BooleanBuilder;
import com.mysema.query.types.Predicate;
import com.mysema.query.types.expr.BooleanExpression;
import org.joda.time.DateTime;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.reusable.SciProUtilities;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

public class ProjectServiceImpl extends AbstractServiceImpl<Project, Long> implements ProjectService {

    public static final int MIN_TITLE_LENGTH = 3;
    private final ProjectRepo projectRepo;
    private final EventBus eventBus;

    @Inject
    public ProjectServiceImpl(ProjectRepo projectRepo, EventBus eventBus, Provider<EntityManager> em) {
        super(em, Project.class, QProject.project);
        this.projectRepo = projectRepo;
        this.eventBus = eventBus;
    }

    @Override
    @Transactional
    public <S extends Project> S save(S entity) {
        boolean isNew = entity.getId() == null;
        S persisted = projectRepo.save(entity);
        if (isNew) {
            eventBus.post(new ProjectCreatedEvent(persisted));
        }
        return persisted;
    }

    @Override
    public Long countAuthors(Filter params) {
        List<Project> projects = findAll(params);
        if (projects.isEmpty()) {
            return (long) 0;
        }
        return from(QProject.project).where(toPredicate(params)).join(QProject.project.projectParticipants).count();
    }

    @Override
    public List<Project> getProjectsByUser(User user) {
        return findAll(isAuthor(user));
    }

    @Override
    public Project findByExternalIdentifier(Long identifier) {
        return findOne(QProject.project.identifier.eq(identifier));
    }

    @Override
    public boolean isPartOfProject(Project project, User user) {
        return getMemberType(project, user) != null;
    }

    @Override
    public Member.Type getMemberType(Project project, User user) {
        for (Member member : project.getMembers()) {
            if (member.getUser().equals(user)) {
                return member.getType();
            }
        }
        return null;
    }

    @Override
    public boolean isSupervisorOrFollower(Project project, User user) {
        return Member.Type.SUPERVISOR.equals(getMemberType(project, user)) || Member.Type.CO_SUPERVISOR.equals(getMemberType(project, user))
                || Member.Type.REVIEWER.equals(getMemberType(project, user));
    }

    @Override
    public Long countMultiProjectAuthors(Filter params) {
        List<Project> projects = findAll(params);
        if (projects.isEmpty()) {
            return (long) 0;
        }
        return (long) projectRepo.findMultipleAuthors(projects).size();
    }

    @Override
    public List<User> findMultipleAuthors(Filter params) {
        List<Project> projects = findAll(params);
        if (projects.isEmpty()) {
            return Collections.emptyList();
        }
        return projectRepo.findMultipleAuthors(projects);
    }

    @Override
    public List<Project> findByUserAndStatus(User user, ProjectStatus status) {
        Filter filter = new Filter();
        filter.setAuthorUser(user);
        Iterable<Project> all = findAll(toPredicate(filter));

        List<Project> list = new ArrayList<>();

        for (Project project : all) {
            if (project.getProjectStatus().equals(status)) {
                list.add(project);
            }
        }
        return list;
    }

    @Override
    public List<Project> getActiveProjects(User author, ProjectType pc) {
        return findAll(hasStatus(ProjectStatus.ACTIVE).and(hasProjectType(pc)).and(isAuthor(author)));
    }

    @Override
    public List<Project> findAll(Filter params) {
        return findAll(toPredicate(params));
    }

    @Override
    @Transactional
    public Project toggleActiveStatus(Project project, boolean state) {
        if (state) {
            return activate(project);
        }
        else {
            return deactivate(project);
        }
    }

    private Project deactivate(final Project project) {
        project.setProjectStatus(ProjectStatus.INACTIVE);
        final Project deactivated = save(project);
        eventBus.post(new ProjectDeactivatedEvent(deactivated));
        return deactivated;
    }

    private Project activate(final Project project) {
        project.setProjectStatus(ProjectStatus.ACTIVE);
        final Project activated = save(project);
        eventBus.post(new ProjectActivatedEvent(activated));
        return activated;
    }

    @Override
    @Transactional
    public Project complete(final Project project) {
        project.setProjectStatus(ProjectStatus.COMPLETED);
        project.setStateOfMind(StateOfMind.FINE);
        Project completed = save(project);
        eventBus.post(new ProjectCompletedEvent(completed));
        return completed;
    }

    @Override
    public Iterable<Project> findAll(Filter filter, Pageable pageable) {
        return findAll(toPredicate(filter), pageable);
    }

    @Override
    public long count(Filter filter) {
        return count(toPredicate(filter));
    }

    protected static Predicate toPredicate(Filter filter) {
        BooleanBuilder predicate = new BooleanBuilder();
        if (filter.getProjectTypes() != null) {
            predicate.and(levelFilter(filter.getProjectTypes()));
        }
        if (filter.getAuthorUser() != null) {
            predicate.and(isAuthor(filter.getAuthorUser()));
        }

        BooleanBuilder rolesBb = new BooleanBuilder();
        if (filter.getRoles() != null) {
            rolesBb.or(hasAnyRole(filter.getSupervisor(), filter.getRoles()));
        }
        if (filter.getSupervisor() != null && filter.isFilterSupervisor()) {
            rolesBb.or(isHeadSupervisor(filter.getSupervisor()));
        }
        if (rolesBb.hasValue()) {
            predicate.and(rolesBb);
        }

        if (filter.getTitleContains() != null && filter.getTitleContains().length() >= MIN_TITLE_LENGTH) {
            predicate.and(titleContains(filter.getTitleContains()));
        }
        if (filter.getCreatedAfter() != null) {
            predicate.and(createdAfter(filter.getCreatedAfter()));
        }
        if (filter.getCreatedBefore() != null) {
            predicate.and(createdBefore(filter.getCreatedBefore()));
        }
        if (filter.getStatuses() != null) {
            predicate.and(hasAnyStatus(filter.getStatuses()));
        }
        if (filter.isFilterExternal()) {
            predicate.and(hasExternalConnection());
        }

        if (filter.isNoReviewer()) {
            predicate.and(hasNoReviewer());
        }
        if (filter.getUnit() != null) {
            predicate.and(supervisorUnitIs(filter.getUnit()));
        }

        if (filter.getRedStateTime() != null) {
            predicate.and(hasBeenRedForTime(filter.getRedStateTime()));
        }
        return predicate;
    }

    private static Predicate hasNoReviewer() {
        return QProject.project.reviewers.isEmpty();
    }

    private static BooleanExpression isFalse() {
        return QProject.project.isNull();
    }

    private static BooleanBuilder hasAnyStatus(Collection<ProjectStatus> statuses) {
        BooleanBuilder bb = new BooleanBuilder();
        if (statuses != null) {
            for (ProjectStatus status : statuses) {
                bb.or(hasStatus(status));
            }
        }

        if (!bb.hasValue()) {
            bb.and(isFalse());
        }

        return bb;
    }

    private static Predicate hasAnyRole(User employee, Collection<ProjectTeamMemberRoles> roles) {
        BooleanBuilder bb = new BooleanBuilder();
        if (roles != null && !roles.isEmpty() && employee != null) {
            for (ProjectTeamMemberRoles role : roles){
                if (role.equals(ProjectTeamMemberRoles.CO_SUPERVISOR)){
                    bb.or(QProject.project.coSupervisors.any().eq(employee));
                } else if (role.equals(ProjectTeamMemberRoles.REVIEWER)){
                    bb.or(QProject.project.reviewers.any().eq(employee));
                }
            }
            return bb;
        }
        return isFalse();
    }

    private static BooleanBuilder levelFilter(Collection<ProjectType> levels) {
        BooleanBuilder e = new BooleanBuilder();
        if (levels != null && !levels.isEmpty()) {
            for (ProjectType level : levels) {
                e.or(hasProjectType(level));
            }
            return e;
        } else {
            e.and(QProject.project.projectType.isNull());
            return e;
        }
    }

    private static BooleanExpression createdAfter(Date after) {
        Date start = SciProUtilities.setTimeOfDayTo0000(after);
        return QProject.project.dateCreated.after(start);
    }

    private static BooleanExpression createdBefore(Date before) {
        Date end = SciProUtilities.setTimeOfDayTo2359(before);
        return QProject.project.dateCreated.before(end);
    }

    private static BooleanExpression hasStatus(ProjectStatus status) {
        if (status != null) {
            return QProject.project.projectStatus.eq(status);
        } else {
            return null;
        }
    }

    private static BooleanExpression titleContains(String searchTerm) {
        return QProject.project.title.containsIgnoreCase(searchTerm);
    }

    private static BooleanExpression hasBeenRedForTime(Integer redStateTime) {
        if (redStateTime != null) {
            return QProject.project.stateOfMindDate.before(new DateTime().minusDays(redStateTime).toDate()).and(QProject.project.stateOfMind.eq(StateOfMind.NEEDHELP));
        }
        return null;
    }

    private static BooleanExpression hasProjectType(ProjectType pc) {
        if (pc != null) {
            return QProject.project.projectType.eq(pc);
        } else {
            return null;
        }
    }

    private static BooleanExpression isAuthor(User user) {
        return QProject.project.projectParticipants.any().eq(user);
    }

    private static BooleanExpression isHeadSupervisor(User emp) {
        if (emp != null) {
            return QProject.project.headSupervisor.eq(emp);
        } else {
            return null;
        }
    }

    private static BooleanExpression hasExternalConnection() {
        return QProject.project.externalOrganization.isNotNull();
    }

    private static BooleanExpression supervisorUnitIs(Unit unit) {
        return QProject.project.headSupervisor.unit.eq(unit);
    }

}
