package se.su.dsv.scipro.crosscutting;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ReviewerAssignedEvent;
import se.su.dsv.scipro.reviewing.FinalSeminarApproval;
import se.su.dsv.scipro.reviewing.FinalSeminarApprovalService;
import se.su.dsv.scipro.reviewing.RoughDraftApproval;
import se.su.dsv.scipro.reviewing.RoughDraftApprovalService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.Collections;
import java.util.Date;
import java.util.TreeSet;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class ReviewerAssignedNotificationsTest {
    @Mock
    private RoughDraftApprovalService roughDraftApprovalService;
    @Mock
    private FinalSeminarApprovalService finalSeminarApprovalService;
    @Mock
    private NotificationController notificationController;

    private ReviewerAssignedNotifications reviewerAssignedNotifications;

    private Project project;
    private User reviewer;

    @Before
    public void setUp() throws Exception {
        reviewerAssignedNotifications = new ReviewerAssignedNotifications(roughDraftApprovalService, finalSeminarApprovalService, notificationController);

        final ProjectType master = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master");
        reviewer = User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build();
        reviewer.setId(1L);
        TreeSet<User> reviewers = new TreeSet<>(new User.ByNameComparator());
        reviewers.addAll(Collections.singleton(reviewer));
        project = Project.builder().title("My project").projectType(master).reviewers(reviewers).build();

        when(roughDraftApprovalService.findBy(project)).thenReturn(Option.<RoughDraftApproval>empty());
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.<FinalSeminarApproval>empty());
    }

    @Test
    public void does_not_notify_if_there_is_no_approval() throws Exception {
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.<FinalSeminarApproval>empty());

        reviewerAssignedNotifications.reviewerAssigned(new ReviewerAssignedEvent(project, reviewer));

        verify(notificationController, never()).notifyProject(eq(project), eq(ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED), any(NotificationSource.class));
    }

    @Test
    public void notifies_on_undecided_final_seminar_approval() throws Exception {
        final FinalSeminarApproval finalSeminarApproval = new FinalSeminarApproval(project, new FileDescription(), "comment", new Date());
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.apply(finalSeminarApproval));

        reviewerAssignedNotifications.reviewerAssigned(new ReviewerAssignedEvent(project, reviewer));

        verify(notificationController).notifyProject(eq(project), eq(ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED), any(NotificationSource.class));
    }

    @Test
    public void does_not_notify_on_decided_approvals() throws Exception {
        final FinalSeminarApproval finalSeminarApproval = new FinalSeminarApproval(project, new FileDescription(), "comment", new Date());
        finalSeminarApproval.approve("Good", Option.<FileDescription>empty(), Option.<FileDescription>empty());
        when(finalSeminarApprovalService.findBy(project)).thenReturn(Option.apply(finalSeminarApproval));

        reviewerAssignedNotifications.reviewerAssigned(new ReviewerAssignedEvent(project, reviewer));

        verify(notificationController, never()).notifyProject(eq(project), eq(ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED), any(NotificationSource.class));
    }

    @Test
    public void notifies_on_undecided_rough_draft_approval() throws Exception {
        final RoughDraftApproval roughDraftApproval = new RoughDraftApproval(project, new FileDescription(), "comment", new Date());
        when(roughDraftApprovalService.findBy(project)).thenReturn(Option.apply(roughDraftApproval));

        reviewerAssignedNotifications.reviewerAssigned(new ReviewerAssignedEvent(project, reviewer));

        verify(notificationController).notifyProject(eq(project), eq(ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REQUESTED), any(NotificationSource.class));
    }
}