package se.su.dsv.scipro.match;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.*;

import static org.junit.Assert.*;

public class ApplicationPeriodServiceImplTest extends IntegrationTest {

    @Inject
    private ApplicationPeriodServiceImpl applicationPeriodService;
    private ProjectType projectType;
    private User student;

    @Before
    public void setUp() throws Exception {
        projectType = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        student = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
    }

    @Test
    public void getCurrentPeriods() throws Exception {
        List<ApplicationPeriod> expectedList = Collections.singletonList(createPeriod(-1, 1));

        assertEquals(expectedList, applicationPeriodService.getCurrentPeriods());
    }

    @Test
    public void getCurrentPeriodsWithDateAndStudent() throws Exception {
        createPeriod(10, 12);

        assertTrue(applicationPeriodService.getCurrentPeriods(student).isEmpty());
    }

    @Test
    public void getCurrentPeriodsWithDateAndStudentStudentHasExemptions() throws Exception {
        ApplicationPeriod current = createPeriod(-1, 1);
        ApplicationPeriod exempted = createPeriod(-10, -5);
        createExemption(student, exempted);

        List<ApplicationPeriod> actualList = applicationPeriodService.getCurrentPeriods(student);
        assertTrue(actualList.contains(current));
        assertTrue(actualList.contains(exempted));
    }

    @Test
    public void getCurrentPeriodsWithDateAndStudentStudentHasExpiredExemptions() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-10, -1);
        createExemption(student, applicationPeriod, new DateTime().minusYears(2).toDate(), new DateTime().minusYears(1).toDate());

        List<ApplicationPeriod> actualList = applicationPeriodService.getCurrentPeriods(student);
        assertTrue(actualList.isEmpty());
    }

    @Test
    public void getCurrentPeriod() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1, projectType);

        assertEquals(applicationPeriod, applicationPeriodService.getCurrentPeriod(projectType));
    }

    @Test
    public void getCurrentPeriodWithStudentWithOpenPeriod() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1, projectType);

        assertEquals(applicationPeriod, applicationPeriodService.getCurrentPeriod(projectType, student));
    }

    @Test
    public void getCurrentPeriodWithStudentHasExemption() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-10, -5);
        createExemption(student, applicationPeriod, new DateTime().minusYears(1).toDate(), new DateTime().plusYears(1).toDate());

        assertEquals(applicationPeriod, applicationPeriodService.getCurrentPeriod(projectType, student));
    }

    @Test
    public void getCurrentPeriodWithStudentHasExpiredExemption() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-10, -5);
        createExemption(student, applicationPeriod, new DateTime().minusYears(2).toDate(), new DateTime().minusYears(1).toDate());

        assertNull(applicationPeriodService.getCurrentPeriod(projectType, student));
    }

    @Test
    public void getPreviousPeriodByProjectTypeNoPeriods() throws Exception {
        createPeriod(-1, 1, projectType);

        assertNull(applicationPeriodService.getPreviousPeriodByProjectType(projectType));
    }

    @Test
    public void getPreviousPeriodByProjectTypeSecondPeriodIsPrevious() throws Exception {
        ApplicationPeriod applicationPeriod2 = createPeriod(-10, -8, projectType);

        assertEquals(applicationPeriod2, applicationPeriodService.getPreviousPeriodByProjectType(projectType));
    }

    @Test
    public void courseStartHasPassed() throws Exception {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setCourseStartDate(new DateTime().minusDays(1).toDate());

        assertTrue(applicationPeriodService.courseStartHasPassed(applicationPeriod));
    }

    @Test
    public void courseStartHasNotPassed() throws Exception {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setCourseStartDate(new DateTime().plusDays(1).toDate());

        assertFalse(applicationPeriodService.courseStartHasPassed(applicationPeriod));
    }

    @Test
    public void endDateIsReached() throws Exception {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setEndDate(new DateTime().minusDays(1).toDate());

        assertTrue(applicationPeriodService.endDateIsReached(applicationPeriod));
    }

    @Test
    public void endDateIsNotReached() throws Exception {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setEndDate(new DateTime().plusDays(1).toDate());

        assertFalse(applicationPeriodService.endDateIsReached(applicationPeriod));
    }

    @Test
    public void isOpen() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1);

        assertTrue(applicationPeriodService.isOpen(applicationPeriod));
    }

    @Test
    public void isClosedBecauseBeforeStartDate() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(1, 2);

        assertFalse(applicationPeriodService.isOpen(applicationPeriod));
    }

    @Test
    public void isClosedBecauseAfterEndDate() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-2, -1);

        assertFalse(applicationPeriodService.isOpen(applicationPeriod));
    }

    @Test
    public void hasNoOpenExemptions() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1);

        assertFalse(applicationPeriodService.hasOpenExemption(applicationPeriod, student));
    }

    @Test
    public void hasExemptionButStudentHasNoExemption() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-10, -5);
        createExemption(save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build()), applicationPeriod, new DateTime().minusDays(1).toDate(), new Date());

        assertFalse(applicationPeriodService.hasOpenExemption(applicationPeriod, student));
    }

    @Test
    public void hasExemptionButExpired() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1);
        createExemption(student, applicationPeriod, new DateTime().minusDays(3).toDate(), new DateTime().minusDays(1).toDate());

        assertFalse(applicationPeriodService.hasOpenExemption(applicationPeriod, student));
    }

    @Test
    public void hasOpenExemption() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-2, -1);
        createExemption(student, applicationPeriod, new DateTime().minusDays(3).toDate(), new DateTime().plusDays(3).toDate());

        assertTrue(applicationPeriodService.hasOpenExemption(applicationPeriod, student));
    }


    @Test
    public void hasNoExemptions() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1);

        assertFalse(applicationPeriodService.hasExemption(student, applicationPeriod));
    }

    @Test
    public void hasExemption() throws Exception {
        ApplicationPeriod applicationPeriod = createPeriod(-1, 1);
        createExemption(student, applicationPeriod, new DateTime().minusDays(1).toDate(), new Date());

        assertTrue(applicationPeriodService.hasExemption(student, applicationPeriod));
    }


    @Test
    public void get_previous_periods_returns_correct_number() throws Exception {
        int size = 2;
        createPeriod(-2, -1, projectType);
        createPeriod(-4, -3, projectType);
        ApplicationPeriod applicationPeriod = createPeriod(0, 5, projectType);
        assertEquals(size, applicationPeriodService.getPreviousPeriods(applicationPeriod, size).size());
    }

    @Test
    public void does_overlap_editing_period() throws Exception {
        ApplicationPeriod period = createPeriod(0, 5, projectType);
        assertFalse(applicationPeriodService.doesPeriodOverlap(period));
    }

    @Test
    public void does_not_overlap_new_start_before_old_end() throws Exception {
        createPeriod(4, 6, projectType);
        ApplicationPeriod newPeriod = createPeriod(0, 3, projectType);
        assertFalse(applicationPeriodService.doesPeriodOverlap(newPeriod));
    }

    @Test
    public void does_not_overlap_new_start_after_old_end() throws Exception {
        createPeriod(0, 3, projectType);
        ApplicationPeriod newPeriod = createPeriod(4, 6, projectType);
        assertFalse(applicationPeriodService.doesPeriodOverlap(newPeriod));
    }

    @Test
    public void does_not_overlap_different_project_types() throws Exception {
        createPeriod(0, 3, projectType);
        ApplicationPeriod newPeriod = createPeriod(0, 3, save(new ProjectType(DegreeLevel.SECOND_CYCLE, "master", "master")));
        assertFalse(applicationPeriodService.doesPeriodOverlap(newPeriod));
    }

    @Test
    public void does_overlap() throws Exception {
        createPeriod(0, 3, projectType);
        ApplicationPeriod newPeriod = createPeriod(0, 3, projectType);
        assertTrue(applicationPeriodService.doesPeriodOverlap(newPeriod));
    }

    private ApplicationPeriod createPeriod(int start, int end) {
        return createPeriod(start, end, projectType);
    }

    private ApplicationPeriod createPeriod(int start, int end, ProjectType type) {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod("my period");
        applicationPeriod.setStartDate(new DateTime().plusDays(start).toDate());
        applicationPeriod.setEndDate(new DateTime().plusDays(end).toDate());
        applicationPeriod.setCourseStartDate(new DateTime().plus(end).plusDays(1).toDate());
        applicationPeriod.addProjectType(type);
        return save(applicationPeriod);
    }

    private void createExemption(User student, ApplicationPeriod period) {
        createExemption(student, period, new DateTime().minusDays(5).toDate(), new DateTime().plusDays(1).toDate());
    }

    private void createExemption(User student, ApplicationPeriod applicationPeriod, Date from, Date until) {
        ApplicationPeriodExemption applicationPeriodExemption = new ApplicationPeriodExemption(student, applicationPeriod);
        applicationPeriodExemption.setStart(from);
        applicationPeriodExemption.setUntil(until);
        applicationPeriodExemption.setGrantedOn(new Date());
        applicationPeriodExemption.setGrantedBy(student);
        applicationPeriod.setApplicationPeriodExemptions(new HashSet<>(Collections.singletonList(applicationPeriodExemption)));
        save(applicationPeriodExemption);
    }
}
