package se.su.dsv.scipro.match;

import com.google.common.collect.Sets;
import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeMatcher;
import org.hamcrest.core.Every;
import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.system.*;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.Set;

import static com.google.common.collect.Iterators.size;
import static org.hamcrest.CoreMatchers.hasItem;
import static org.junit.Assert.assertThat;

public class IdeaServiceImplIntegrationTest extends IntegrationTest {

    public static final PageRequest ALL = new PageRequest(0, Integer.MAX_VALUE);
    @Inject
    private IdeaService ideaService;

    private ProjectType projectType;
    private User user;
    private User randomUser;
    private Set<Language> english;

    @Before
    public void setUp() throws Exception {
        projectType = createProjectType(DegreeLevel.FIRST_CYCLE);
        randomUser = createUser();
        user = createUser();
        english = Sets.newHashSet(createLanguage());
    }

    @Test
    public void findAll() throws Exception {
        Idea.Status status = Idea.Status.MATCHED;
        ResearchArea researchArea = createResearchArea();

        IdeaService.Filter params = new IdeaService.Filter();
        params.setLevels(Collections.singletonList(projectType));
        params.setSupervisor(randomUser);
        params.setTypes(Collections.singletonList(Idea.Type.SUPERVISOR));
        params.setStatuses(Collections.singletonList(status));
        params.setResearchArea(researchArea);

        User author = createUser();
        Idea idea = setUpIdea(status, author, researchArea, english, createApplicationPeriod());

        Iterable<Idea> expectedIterable = Collections.singletonList(idea);
        Assert.assertEquals(expectedIterable, ideaService.findAll(params));
    }

    @Test
    public void filter_by_status() throws Exception {
        ApplicationPeriod period = save(new ApplicationPeriod("period"));
        Idea unmatched = createSupervisorIdea();
        Idea pending = createSupervisorIdea();
        pending.setApplicationPeriod(period);
        pending.addIdeaParticipation(new IdeaParticipation(user, pending));
        save(pending);

        createMatch(Idea.Status.UNMATCHED, randomUser, unmatched);
        createMatch(Idea.Status.MATCHED, randomUser, pending);

        IdeaService.Filter filter = new IdeaService.Filter();
        filter.setStatuses(Collections.singletonList(Idea.Status.MATCHED));

        Iterable<Idea> ideas = ideaService.findAll(filter, ALL);
        assertThat(ideas, Every.everyItem(hasStatus(Idea.Status.MATCHED)));
        Assert.assertEquals(1, size(ideas.iterator()));
        Assert.assertEquals(1, ideaService.count(filter));
    }

    @Test
    public void find_all_by_project_type() throws Exception {
        Idea expected = createSupervisorIdea();
        createSupervisorIdea(createProjectType(DegreeLevel.FIRST_CYCLE));

        IdeaService.Filter filter = new IdeaService.Filter();
        filter.setLevels(Collections.singletonList(expected.getProjectType()));

        Iterable<Idea> ideas = ideaService.findAll(filter, ALL);
        assertThat(ideas, hasItem(expected));
        Assert.assertEquals(1, size(ideas.iterator()));
    }

    @Test
    public void find_without_first_meeting() throws Exception {
        Idea expected = createSupervisorIdea();

        IdeaService.Filter filter = new IdeaService.Filter();
        filter.setNoFirstMeeting(true);

        Iterable<Idea> ideas = ideaService.findAll(filter);

        assertThat(ideas, hasItem(expected));
    }

    private Matcher<Idea> hasStatus(final Idea.Status status) {
        return new TypeSafeMatcher<Idea>() {
            @Override
            protected boolean matchesSafely(Idea item) {
                return item.getMatchStatus().equals(status);
            }

            @Override
            public void describeTo(Description description) {
                description.appendValue(status);
            }
        };
    }

    private Idea setUpIdea(Idea.Status status, User author, ResearchArea researchArea, Set<Language> languages, ApplicationPeriod applicationPeriod) {
        Idea idea = createIdea(author, researchArea, languages);
        Match match = createMatch(status, randomUser, idea);
        idea.setMatch(match);
        idea.setApplicationPeriod(applicationPeriod);
        idea = save(idea);
        return idea;
    }

    private ProjectType createProjectType(DegreeLevel degreeLevel) {
        ProjectType projectType = new ProjectType(degreeLevel, "Bachelor", "Bachelor");
        return save(projectType);
    }

    private Idea createIdea(User author, ResearchArea researchArea, Set<Language> languages) {
        Idea idea = createSupervisorIdea();
        idea.setResearchArea(researchArea);
        idea.setLanguages(languages);

        IdeaParticipation ideaParticipation = new IdeaParticipation(author, idea);

        idea.addIdeaParticipation(ideaParticipation);

        return save(idea);
    }

    private Idea createSupervisorIdea() {
        return createSupervisorIdea(projectType);
    }

    private Idea createSupervisorIdea(ProjectType projectType) {
        Idea idea = new Idea();
        idea.setProjectType(projectType);
        idea.setType(Idea.Type.SUPERVISOR);
        idea.setCreator(randomUser);
        idea.setTitle("some title");
        return save(idea);
    }

    private ResearchArea createResearchArea() {
        ResearchArea researchArea = new ResearchArea();
        researchArea.setTitle("some title");
        return save(researchArea);
    }

    private Match createMatch(Idea.Status status, User supervisor, Idea idea) {
        Match match = new Match();
        match.setStatus(status);
        match.setSupervisor(supervisor);
        match.setIdea(idea);
        Match save = save(match);
        idea.setMatch(save);
        save(idea);
        return save;
    }

    private User createUser() {
        User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        return save(user);

    }

    private Language createLanguage() {
        Language language = new Language();
        language.setName("English");
        return save(language);
    }

    private ApplicationPeriod createApplicationPeriod() {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod("applicationPeriod");
        applicationPeriod.setStartDate(new Date());
        applicationPeriod.setEndDate(new DateTime().plusMonths(6).toDate());
        applicationPeriod.setCourseStartDate(new Date());
        return save(applicationPeriod);
    }
}
