package se.su.dsv.scipro.milestones.service.impl;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Collections;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

public class MilestoneServiceImplIntegrationTest extends IntegrationTest {
    public static final PageRequest ALL = new PageRequest(0, Long.MAX_VALUE);

    @Inject
    private MilestoneServiceImpl milestoneService;

    private MilestoneActivityTemplate projectMilestone;
    private MilestoneActivityTemplate individualMilestone;
    private Project project;
    private User user;
    private ProjectType bachelor;

    @Before
    public void setUp() throws Exception {
        bachelor = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        project = createProject();
        user = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        MilestonePhaseTemplate phase = save(new MilestonePhaseTemplate("Phase 1", "Description"));
        MilestoneActivityTemplate individualMilestone = new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.STUDENT, "Student milestone", "Description");
        individualMilestone.setMilestonePhaseTemplate(phase);
        individualMilestone.addProjectType(bachelor);
        this.individualMilestone = save(individualMilestone);
        MilestoneActivityTemplate projectMilestone = new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT, "Project milestone", "Description");
        projectMilestone.setMilestonePhaseTemplate(phase);
        projectMilestone.addProjectType(bachelor);
        this.projectMilestone = save(projectMilestone);
    }

    @Test
    public void find_completed_milestones() throws Exception {
        MileStoneService.Filter filter = new MileStoneService.Filter();

        assertTrue(milestoneService.findCompletedMilestones(individualMilestone, filter, ALL).isEmpty());
    }

    @Test
    public void testCountCompletedMileStonesByActivity() throws Exception {
        MileStoneService.Filter filter = new MileStoneService.Filter();

        assertEquals(0, milestoneService.countCompletedMileStones(individualMilestone, filter));
    }

    @Test
    public void testCountCompletedMileStonesByActivityWithProjectType() throws Exception {
        MileStoneService.Filter filter = new MileStoneService.Filter();
        filter.setProjectTypes(Collections.singletonList(bachelor));

        assertEquals(0, milestoneService.countCompletedMileStones(individualMilestone, filter));
    }

    @Test
    public void count_completed_milestones_by_project_type_returns_nothing() throws Exception {
        MileStoneService.Filter filter = new MileStoneService.Filter();
        assertEquals(0, milestoneService.countCompletedMileStones(individualMilestone, filter, bachelor));
    }

    @Test
    public void count_completed_milestones_by_project_type() throws Exception {
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        Milestone milestone = new Milestone(save(Project.builder().title("p").projectType(bachelor).headSupervisor(headSupervisor).build()), individualMilestone);
        milestone.setConfirmed(true);
        save(milestone);
        MileStoneService.Filter filter = new MileStoneService.Filter();
        filter.setProjectStatuses(Collections.singletonList(ProjectStatus.ACTIVE));
        filter.setMileStoneCompletedBefore(new DateTime().plusDays(1).toDate());
        assertEquals(1, milestoneService.countCompletedMileStones(individualMilestone, filter, bachelor));
    }

    @Test
    public void confirm_project_milestone() throws Exception {
        milestoneService.setConfirmed(project, projectMilestone, true);
        assertTrue(milestoneService.isConfirmed(project, projectMilestone));
    }

    @Test
    public void confirm_individual_milestone() throws Exception {
        milestoneService.setConfirmed(project, user, individualMilestone, true);
        assertTrue(milestoneService.isConfirmed(project, user, individualMilestone));
    }

    @Test
    public void testCountMileStones() throws Exception {
        assertEquals(0, milestoneService.countProjectMilestones(project, false));
    }

    @Test
    public void testCountStudentMileStones() throws Exception {
        assertEquals(0, milestoneService.countMileStones(project, user, true));
    }

    @Test
    public void testIsMilestoneConfirmed() throws Exception {
        milestoneService.setConfirmed(project, projectMilestone, false);
        boolean confirmed = milestoneService.isConfirmed(project, projectMilestone);
        assertFalse("Was confirmed", confirmed);
    }

    @Test
    public void testIsStudentMileStoneConfirmed() throws Exception {
        milestoneService.setConfirmed(project, user, individualMilestone, false);
        boolean confirmed = milestoneService.isConfirmed(project, user, individualMilestone);
        assertFalse("Was confirmed", confirmed);
    }

    @Test
    public void testCountCompletedMileStonesWithoutSelectedProjectTypeReturnsZero() throws Exception {
        long expected = 0L;
        MileStoneService.Filter filter = new MileStoneService.Filter();

        assertEquals(expected, milestoneService.countCompletedMileStones(individualMilestone, filter));
    }

    @Test
    public void testGetStudentMileStoneCreatesNewOne() throws Exception {
        Milestone milestone = milestoneService.getMileStone(project, user, individualMilestone);
        assertEquals(user, milestone.getUser());
    }

    @Test
    public void does_not_save_a_new_milestone_if_milestone_module_is_not_enabled_for_the_project() throws Exception {
        bachelor.removeModule(ProjectModule.MILESTONES);

        milestoneService.setConfirmed(project, projectMilestone, true);

        assertFalse(milestoneService.isConfirmed(project, projectMilestone));
    }

    private Project createProject() {
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        return save(Project.builder().title("Title").projectType(bachelor).headSupervisor(headSupervisor).build());
    }
}
