package se.su.dsv.scipro.peer;

import com.google.common.eventbus.EventBus;
import com.mysema.query.types.Predicate;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.runners.MockitoJUnitRunner;
import scala.Option;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeSettings;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class PeerPortalImplTest {

    private static final long SOME_PROJECT_ID = 14L;
    private static final long SOME_REQUEST_ID = 1L;

    @Mock
    private FileDescriptionService fileDescriptionService;
    @Mock
    private PeerReviewRepository peerReviewRepository;
    @Mock
    private PeerRequestRepository peerRequestRepository;
    @Mock
    private ProjectService projectService;
    @Mock
    private EventBus eventBus;
    @InjectMocks
    private PeerPortalImpl peerPortalController;

    @Before
    public void setUp() throws Exception {
        when(peerRequestRepository.save(Matchers.isA(PeerRequest.class))).thenAnswer(new org.mockito.stubbing.Answer<PeerRequest>() {
            @Override
            public PeerRequest answer(InvocationOnMock invocation) throws Throwable {
                PeerRequest peerRequest = (PeerRequest) invocation.getArguments()[0];
                peerRequest.setId(SOME_REQUEST_ID);
                return peerRequest;
            }
        });
        when(peerReviewRepository.save(Matchers.isA(PeerReview.class))).thenAnswer(new org.mockito.stubbing.Answer<PeerReview>() {
            @Override
            public PeerReview answer(InvocationOnMock invocation) throws Throwable {
                return (PeerReview) invocation.getArguments()[0];
            }
        });
    }

    @Test
    public void store_request_stores_file_in_request_folder() throws Exception {
        // Arrange
        PeerRequest request = new PeerRequest();
        Project project = new Project();
        project.setId(SOME_PROJECT_ID);
        request.setProject(project);

        FileUpload fileUpload = mock(FileUpload.class);

        // Act
        peerPortalController.storePeerRequest(fileUpload, request);

        // Assert
        Mockito.verify(fileDescriptionService).storeFile(fileUpload);
    }

    @Test
    public void submitting_late_peer_review_should_mark_review_as_expired() throws Exception {
        peerPortalController.submit(createLateReview(), Option.<FileUpload>empty());

        Assert.assertTrue(submittedReview().isExpired());
    }

    @Test
    public void submitting_peer_review_in_time_should_mark_review_as_completed() throws Exception {
        peerPortalController.submit(createInTimeReview(), Option.<FileUpload>empty());

        Assert.assertTrue(submittedReview().isCompleted());
    }

    @Test
    public void submitting_late_peer_review_should_change_request_status_to_waiting() throws Exception {
        peerPortalController.submit(createLateReview(), Option.<FileUpload>empty());

        Assert.assertThat(reviewedRequest().getStatus(), is(RequestStatus.WAITING));
    }

    @Test
    public void submitting_peer_review_in_time_should_change_request_status_to_finished() throws Exception {
        peerPortalController.submit(createInTimeReview(), Option.<FileUpload>empty());

        Assert.assertThat(reviewedRequest().getStatus(), is(RequestStatus.FINISHED));
    }

    @Test
    public void completing_first_review_should_post_event() throws Exception {
        when(peerReviewRepository.count(any(Predicate.class))).thenReturn(1L);

        peerPortalController.submit(createInTimeReview(), Option.<FileUpload>empty());

        Mockito.verify(eventBus).post(Matchers.isA(FirstPeerReviewCompletedEvent.class));
    }

    @Test
    public void completing_second_review_should_post_event() throws Exception {
        when(peerReviewRepository.count(any(Predicate.class))).thenReturn(2L);

        peerPortalController.submit(createInTimeReview(), Option.<FileUpload>empty());

        Mockito.verify(eventBus).post(Matchers.isA(SecondPeerReviewCompletedEvent.class));
    }

    @Test
    public void handing_in_review_late_should_not_publish_any_event() throws Exception {
        peerPortalController.submit(createLateReview(), Option.<FileUpload>empty());

        Mockito.verify(eventBus, Mockito.never()).post(any(PeerReviewEvent.class));
    }

    @Test
    public void converts_all_checklist_template_questions_to_answers() throws Exception {
        ChecklistTemplate checklistTemplate = new ChecklistTemplate();
        checklistTemplate.addQuestion("Was the background good?");
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(newProject());
        peerRequest.setChecklistTemplate(checklistTemplate);

        PeerReview peerReview = peerPortalController.acceptReview(peerRequest, new User(), newProject());

        assertEquals(peerReview.getAnswers().size(), peerRequest.getChecklistTemplate().getQuestions().size());
    }

    @Test
    public void if_there_is_a_file_it_should_be_stored_in_the_review() throws Exception {
        FileDescription file = mock(FileDescription.class);
        FileUpload upload = mock(FileUpload.class);
        when(fileDescriptionService.storeFile(upload)).thenReturn(file);

        PeerReview peerReview = createInTimeReview();
        peerPortalController.submit(peerReview, Option.apply(upload));

        ArgumentCaptor<PeerReview> captor = ArgumentCaptor.forClass(PeerReview.class);
        verify(peerReviewRepository).save(captor.capture());
        assertEquals(captor.getValue().getFile(), file);
    }

    @Test(expected = MissingAnswerException.class)
    public void every_answer_needs_an_answer_and_motivation() throws Exception {
        PeerReview peerReview = createInTimeReview();
        addSomeAnswers(peerReview);

        peerPortalController.submit(peerReview, Option.<FileUpload>empty());
    }

    @Test(expected = TooShortCommentException.class)
    public void if_there_are_no_answers_a_comment_of_some_length_is_required() throws Exception {
        PeerReview peerReview = createInTimeReview();
        peerReview.setComment("This sucks");

        peerPortalController.submit(peerReview, Option.<FileUpload>empty());
    }

    @Test(expected = PartOfProjectException.class)
    public void trying_to_accept_request_from_own_project() throws Exception {
        Project project = ObjectMother.SOME_PROJECT;
        project.addProjectParticipant(ObjectMother.SOME_USER);
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(project);
        peerPortalController.acceptReview(peerRequest, ObjectMother.SOME_USER, project);
    }

    @Test(expected = RecentlyReviewedException.class)
    public void trying_to_accept_recently_reviewed_project() throws Exception {
        Project project = projectWithSettings();
        PeerReview peerReview = new PeerReview();
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(project);
        peerReview.setPeerRequest(peerRequest);
        when(peerReviewRepository.findAll(any(Predicate.class))).thenReturn(Collections.singletonList(peerReview));
        peerPortalController.acceptReview(peerRequest, ObjectMother.SOME_USER, ObjectMother.SOME_PROJECT);
    }

    @Test(expected = RequestTakenException.class)
    public void trying_to_accept_already_taken_request() throws Exception {
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setStatus(RequestStatus.TAKEN);
        peerRequest.setProject(projectWithSettings());
        peerPortalController.acceptReview(peerRequest, ObjectMother.SOME_USER, ObjectMother.SOME_PROJECT);
    }


    private Project projectWithSettings() {
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "b", "b");
        projectType.setProjectTypeSettings(new ProjectTypeSettings());
        return Project.builder().title("title").projectType(projectType).build();
    }

    private void addSomeAnswers(PeerReview peerReview) {
        peerReview.addAnswer("What is the answer to life, the universe and everything?");
    }

    private PeerReview createInTimeReview() {
        PeerReview peerReview = createPeerReview();
        peerReview.setDateCreated(inTimeDate());
        peerReview.setComment(StringUtils.repeat("a;", PeerReview.MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS));
        return peerReview;
    }

    private PeerReview createLateReview() {
        PeerReview peerReview = createPeerReview();
        peerReview.setDateCreated(lateDate());
        peerReview.setComment(StringUtils.repeat("a;", PeerReview.MINIMUM_COMMENT_LENGTH_IF_NO_ANSWERS));
        return peerReview;
    }

    private PeerReview createPeerReview() {
        PeerReview peerReview = new PeerReview();
        peerReview.setProject(newProject());
        peerReview.setReviewer(ObjectMother.SOME_USER);
        peerReview.setPeerRequest(new PeerRequest());
        return peerReview;
    }

    private Project newProject() {
        ProjectTypeSettings projectTypeSettings = new ProjectTypeSettings();
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        projectType.setProjectTypeSettings(projectTypeSettings);
        return Project.builder().title("Peer project").projectType(projectType).build();
    }

    private Date inTimeDate() {
        return new Date();
    }

    private Date lateDate() {
        return new DateTime().minusDays(ProjectTypeSettings.DEFAULT_NUM_DAYS_TO_SUBMIT_PEER_REVIEW * 2).toDate();
    }

    private PeerReview submittedReview() {
        ArgumentCaptor<PeerReview> captor = ArgumentCaptor.forClass(PeerReview.class);
        Mockito.verify(peerReviewRepository).save(captor.capture());
        return captor.getValue();
    }

    private PeerRequest reviewedRequest() {
        ArgumentCaptor<PeerRequest> captor = ArgumentCaptor.forClass(PeerRequest.class);
        Mockito.verify(peerRequestRepository).save(captor.capture());
        return captor.getValue();
    }
}
