package se.su.dsv.scipro.peer;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Date;
import java.util.EnumSet;
import java.util.List;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;
import static se.su.dsv.scipro.peer.PeerReviewService.Filter;

public class PeerReviewServiceImplTest extends IntegrationTest {

    public static final PageRequest ALL = new PageRequest(0, Long.MAX_VALUE);

    @Inject
    private PeerReviewServiceImpl peerReviewService;

    public static final ProjectType SOME_PROJECT_TYPE = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
    public static final Project SOME_PROJECT = Project.builder().title("Some title").projectType(SOME_PROJECT_TYPE).build();
    private PeerRequest peerRequest;
    private User reviewer;
    private PeerReview completed;
    private PeerReview expired;
    private PeerReview inProgress;
    private Project project;
    private Project requesterProject;

    @Before
    public void setUp() throws Exception {
        reviewer = save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build());
        project = newProject();
        requesterProject = newProject();
        peerRequest = prepareRequest();
        completed = newReview(PeerReview.ReviewStatus.COMPLETED);
        expired = newReview(PeerReview.ReviewStatus.EXPIRED);
        inProgress = newReview(PeerReview.ReviewStatus.IN_PROGRESS);
    }

    @Test
    public void findAll() throws Exception {
        assertThat(peerReviewService.findAll(ALL), hasItems(completed, expired, inProgress));
    }

    @Test
    public void findAllWithParams() throws Exception {
        Filter filter = new Filter();
        filter.setStatuses(EnumSet.of(PeerReview.ReviewStatus.COMPLETED, PeerReview.ReviewStatus.EXPIRED));
        assertThat(peerReviewService.findAll(filter, ALL), hasItems(completed, expired));
        assertThat(peerReviewService.findAll(filter, ALL), not(hasItem(inProgress)));
    }

    @Test
    public void countWithParams() throws Exception {
        Filter filter = new Filter();
        assertEquals(3, peerReviewService.count(filter));
    }

    @Test
    public void findInProgressReviews() throws Exception {
        assertThat(peerReviewService.findInProgressReviews(reviewer, project), hasItem(inProgress));
    }

    @Test
    public void findReceivedCompletedReviews() throws Exception {
        assertThat(peerReviewService.findReceivedCompletedReviews(requesterProject), hasItem(completed));
    }

    @Test
    public void findByReviewerAndProject() throws Exception {
        assertThat(peerReviewService.findByReviewerAndProject(reviewer, project), hasItems(completed, inProgress));
    }

    private PeerRequest prepareRequest() {
        PeerRequest peerRequest = new PeerRequest();
        peerRequest.setProject(requesterProject);
        peerRequest.setRequester(save(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build()));
        peerRequest.setFile(save(new FileDescription()));
        return save(peerRequest);
    }

    private Project newProject() {
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        ProjectType projectType = save(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"));
        return save(Project.builder().title("Title").projectType(projectType).headSupervisor(headSupervisor).build());
    }

    private PeerReview newReview(PeerReview.ReviewStatus status) {
        PeerReview peerReview = new PeerReview();
        peerReview.setStatus(status);
        peerReview.setReviewer(reviewer);
        peerReview.setPeerRequest(peerRequest);
        peerReview.setProject(project);
        peerRequest.getPeerReviews().add(peerReview);
        return save(peerReview);
    }
}
