package se.su.dsv.scipro.project;

import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.Unit;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.IntegrationTest;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;

public class ProjectServiceImplIntegrationTest extends IntegrationTest {

    @Inject
    private ProjectService projectService;
    private Project project;
    private User user;

    @Before
    public void setUp() {
        ProjectType bachelor = createProjectType();
        project = createProject(bachelor, ProjectStatus.ACTIVE);
        user = createUser();
        Unit unit = new Unit();
        unit.setTitle("DSV IT");
        user.setUnit(save(unit));
    }

    @Test
    public void countAuthors() throws Exception {
        User user1 = createUser();

        User user2 = createUser();

        ProjectType projectType = createProjectType();
        Project project = createProject(projectType, ProjectStatus.ACTIVE);
        project.addProjectParticipant(user1);
        project.addProjectParticipant(user2);
        save(project);

        ProjectService.Filter params = new ProjectService.Filter();
        params.setProjectTypes(Collections.singletonList(projectType));

        assertEquals(new Long(2), projectService.countAuthors(params));
    }

    @Test
    public void countAuthorsEmpty() throws Exception {
        User user1 = createUser();

        User user2 = createUser();

        ProjectType projectType = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master");
        save(projectType);

        Project project = createProject(projectType, ProjectStatus.ACTIVE);
        project.addProjectParticipant(user1);
        project.addProjectParticipant(user2);
        save(project);

        ProjectService.Filter params = new ProjectService.Filter();
        params.setProjectTypes(Collections.singletonList(createProjectType()));

        assertEquals(new Long(0), projectService.countAuthors(params));
    }

    @Test
    public void findAll() throws Exception {
        PageRequest pageable = new PageRequest(0, 5);

        List<Project> page = Arrays.asList(project, createProject(createProjectType(), ProjectStatus.ACTIVE));

        assertEquals(page, projectService.findAll(new ProjectService.Filter(), pageable));
    }

    @Test
    public void countAll() throws Exception {
        long expected = 4L;

        ProjectType projectType = createProjectType();
        createProject(projectType, ProjectStatus.ACTIVE);
        createProject(projectType, ProjectStatus.ACTIVE);
        createProject(projectType, ProjectStatus.ACTIVE);

        assertEquals(expected, projectService.count(new ProjectService.Filter()));
    }

    @Test
    public void countAllWithParams() throws Exception {
        ProjectType projectType = createProjectType();
        User author = createUser();
        User employee = createUser();

        ProjectService.Filter params = new ProjectService.Filter();
        params.setStatuses(new HashSet<>(Collections.singletonList(ProjectStatus.ACTIVE)));
        params.setProjectTypes(Collections.singletonList(projectType));
        params.setAuthorUser(author);
        params.setRoles(new HashSet<>(Collections.singletonList(ProjectTeamMemberRoles.CO_SUPERVISOR)));
        params.setSupervisor(employee);
        params.setFilterSupervisor(true);
        params.setTitleContains("Some");
        params.setCreatedAfter(new DateTime().minusDays(1).toDate());
        params.setCreatedBefore(new DateTime().plusDays(1).toDate());

        long expected = 1L;

        Project project = createProject(projectType, ProjectStatus.ACTIVE);
        project.addProjectParticipant(author);
        project.addCoSupervisor(employee);
        project.setHeadSupervisor(employee);

        save(project);

        assertEquals(expected, projectService.count(params));
    }


    @Test
    public void getActiveProjects() throws Exception {
        ProjectType projectType = createProjectType();
        Project project = createProject(projectType, ProjectStatus.ACTIVE);
        User user = createUser();
        project.addProjectParticipant(user);
        project = save(project);
        List<Project> expectedList = Collections.singletonList(project);

        assertEquals(expectedList, projectService.getActiveProjects(user, projectType));
    }

    @Test
    public void count_multi_project_authors() throws Exception {
        User user1 = createUser();
        User user2 = createUser();
        User user3 = createUser();

        ProjectType projectType = createProjectType();
        Project project = createProject(projectType, ProjectStatus.ACTIVE);
        project.addProjectParticipant(user1);
        project.addProjectParticipant(user2);
        save(project);

        Project project2 = createProject(projectType, ProjectStatus.ACTIVE);
        project2.addProjectParticipant(user1);
        project2.addProjectParticipant(user2);
        save(project2);

        Project project3 = createProject(projectType, ProjectStatus.ACTIVE);
        project3.addProjectParticipant(user3);

        ProjectService.Filter params = new ProjectService.Filter();
        params.setProjectTypes(Collections.singletonList(projectType));

        assertEquals(new Long(2), projectService.countMultiProjectAuthors(params));
    }

    @Test
    public void find_multiple_project_authors() throws Exception {
        User user1 = createUser();
        User user2 = createUser();

        ProjectType projectType = createProjectType();
        Project project = createProject(projectType, ProjectStatus.ACTIVE);
        project.addProjectParticipant(user1);
        project.addProjectParticipant(user2);
        save(project);

        Project project2 = createProject(projectType, ProjectStatus.ACTIVE);
        project2.addProjectParticipant(user1);
        save(project);

        ProjectService.Filter params = new ProjectService.Filter();
        params.setProjectTypes(Collections.singletonList(projectType));

        assertEquals(Collections.singletonList(user1), projectService.findMultipleAuthors(params));
    }

    @Test
    public void find_by_user_and_status() throws Exception {
        User user = createUser();

        Project project = createProject(createProjectType(), ProjectStatus.COMPLETED);
        project.addProjectParticipant(user);
        save(project);

        assertEquals(1, projectService.findByUserAndStatus(user, ProjectStatus.COMPLETED).size());
    }

    @Test
    public void find_by_user_and_status_not_matching_status() throws Exception {
        User user = createUser();

        Project project = createProject(createProjectType(), ProjectStatus.INACTIVE);
        project.addProjectParticipant(user);
        save(project);

        assertEquals(0, projectService.findByUserAndStatus(user, ProjectStatus.COMPLETED).size());
    }

    @Test
    public void find_by_user_and_status_not_matching_status_not_matching_user() throws Exception {
        User user = createUser();

        Project project = createProject(createProjectType(), ProjectStatus.INACTIVE);
        project.addProjectParticipant(user);
        save(project);

        assertEquals(0, projectService.findByUserAndStatus(createUser(), ProjectStatus.COMPLETED).size());
    }

    @Test
    public void find_by_external_identifier() {
        project.setIdentifier(234L);

        assertEquals(project, projectService.findByExternalIdentifier(project.getIdentifier()));
    }

    @Test
    public void is_part_of_project() {
        assertFalse(projectService.isPartOfProject(project, user));
        project.addProjectParticipant(user);
        assertTrue(projectService.isPartOfProject(project, user));
    }

    @Test
    public void is_supervisor_or_reviewer() {
        assertFalse(projectService.isSupervisorOrFollower(project, user));
    }

    @Test
    public void is_reviewer() {
        project.addReviewer(user);
        assertTrue(projectService.isSupervisorOrFollower(project, user));
    }

    @Test
    public void is_co_supervisor() {
        project.addCoSupervisor(user);
        assertTrue(projectService.isSupervisorOrFollower(project, user));
    }

    @Test
    public void toggle_status() {
        projectService.toggleActiveStatus(project, false);
        assertEquals(project.getProjectStatus(), ProjectStatus.INACTIVE);
    }

    @Test
    public void completing_project_set_status_to_complete() throws Exception {
        Project completed = projectService.complete(project);
        assertThat(completed.getProjectStatus(), is(ProjectStatus.COMPLETED));
    }

    @Test
    public void completing_project_sets_state_of_mind_to_fine() throws Exception {
        Project completed = projectService.complete(project);
        assertThat(completed.getStateOfMind(), is(StateOfMind.FINE));
    }

    private Project createProject(ProjectType projectType, ProjectStatus active) {
        User headSupervisor = save(User.builder().firstName("John").lastName("Doe").emailAddress("john@example.com").build());
        Project project = new Project();
        project.setTitle("Some title");
        project.setProjectType(projectType);
        project.setProjectStatus(active);
        project.setHeadSupervisor(headSupervisor);
        save(project);
        return project;
    }

    private ProjectType createProjectType() {
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        save(projectType);
        return projectType;
    }

    private User createUser() {
        User user = User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build();
        return save(user);
    }
}
