package se.su.dsv.scipro.report;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeMatcher;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.mockito.Mockito.mock;

public class GradingReportTemplateTest {

    private ProjectType bachelor;
    private ProjectType master;

    @Before
    public void setUp() throws Exception {
        bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        master = new ProjectType(DegreeLevel.SECOND_CYCLE, "master", "master");
    }

    @Test(expected = IllegalArgumentException.class)
    public void creating_a_grading_report_template_with_null_project_type_should_fail() throws Exception {
        new GradingReportTemplate(null, 30);
    }

    @Test(expected = IllegalArgumentException.class)
    public void creating_a_report_for_a_project_on_a_different_project_type_should_fail() throws Exception {
        GradingReportTemplate gradingReportTemplate = createBachelorTemplate();
        Project project = createMasterProject();

        gradingReportTemplate.createSupervisorReport(project, mock(User.class));
    }

    @Test
    public void creating_a_report_for_a_project_on_the_same_project_type_should_work() throws Exception {
        GradingReportTemplate gradingReportTemplate = createBachelorTemplate();
        Project project = createBachelorProject();

        GradingReport gradingReport = gradingReportTemplate.createSupervisorReport(project, mock(User.class));
        assertNotNull(gradingReport);
    }

    @Test
    public void add_criterion() throws Exception {
        String title = "U1 Problem";

        GradingReportTemplate gradingReportTemplate = createBachelorTemplate();
        List<GradingCriterionPointTemplate> gradingCriterionPointTemplates = new ArrayList<>();
        gradingCriterionPointTemplates.add(new GradingCriterionPointTemplate.Builder().point(1).description("Description").build());
        gradingReportTemplate.addProjectCriterion(title, 1, gradingCriterionPointTemplates);

        assertThat(gradingReportTemplate.getCriteria(), hasItem(withTitle(title)));
    }

    private static Matcher<? super GradingCriterionTemplate> withTitle(final String title) {
        return new TypeSafeMatcher<GradingCriterionTemplate>() {
            @Override
            protected boolean matchesSafely(GradingCriterionTemplate item) {
                return item.getTitle().equals(title);
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("a criterion with title ").appendValue(title);
            }
        };
    }

    private Project createBachelorProject() {
        return createProject(bachelor);
    }

    private Project createMasterProject() {
        return createProject(master);
    }

    private Project createProject(ProjectType projectType) {
        return Project.builder().title("Foo").projectType(projectType).build();
    }

    private GradingReportTemplate createBachelorTemplate() {
        return new GradingReportTemplate(bachelor, 30);
    }
}
