package se.su.dsv.scipro.reviewing;

import org.junit.Test;
import scala.Option;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.project.Project;

import java.util.Date;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.mockito.Mockito.mock;
import static se.su.dsv.scipro.reviewing.WithStatus.withStatus;

public class FinalSeminarApprovalTest {
    @Test
    public void new_approval_is_undecided() throws Exception {
        FinalSeminarApproval approval = createUndecided();
        assertThat(approval.getCurrentStatus(), is(Status.UNDECIDED));
    }

    @Test
    public void approve_final_seminar() throws Exception {
        FinalSeminarApproval approval = createApproved();
        assertThat(approval.getCurrentStatus(), is(Status.APPROVED));
    }

    @Test
    public void reject_final_seminar() throws Exception {
        FinalSeminarApproval approval = createRejected();
        assertThat(approval.getCurrentStatus(), is(Status.REJECTED));
    }

    @Test(expected = IllegalStateException.class)
    public void adding_new_thesis_for_approval_only_works_if_decision_is_rejected() throws Exception {
        FinalSeminarApproval approval = createUndecided();
        approval.addNewThesis(createFileDescription(), "test", new Date());
    }

    @Test
    public void adding_new_thesis_for_approval_sets_status_to_undecided() throws Exception {
        FinalSeminarApproval approval = createRejected();
        approval.addNewThesis(createFileDescription(), "test", new Date());
        assertThat(approval.getCurrentStatus(), is(Status.UNDECIDED));
    }

    @Test
    public void current_file_before_any_decision() throws Exception {
        FileDescription fileDescription = createFileDescription();
        FinalSeminarApproval approval = createUndecided(fileDescription);
        assertThat(approval.getCurrentThesis(), is(fileDescription));
    }

    @Test
    public void current_file_after_new_thesis_added() throws Exception {
        FinalSeminarApproval approval = createRejected();
        FileDescription fileDescription = createFileDescription();
        approval.addNewThesis(fileDescription, "test", new Date());
        assertThat(approval.getCurrentThesis(), is(fileDescription));
    }

    @Test
    public void has_history_over_decisions() throws Exception {
        FinalSeminarApproval approval = createRejected();
        approval.addNewThesis(createFileDescription(), "test", new Date());
        assertThat(approval.getHistory(), hasItem(withStatus(Status.REJECTED)));
        assertThat(approval.getHistory(), hasItem(withStatus(Status.UNDECIDED)));
    }

    @Test(expected = IllegalStateException.class)
    public void approve_an_already_made_decision() throws Exception {
        FinalSeminarApproval approval = createApproved();
        approval.approve("Very good", Option.<FileDescription>empty(), Option.<FileDescription>empty());
    }

    @Test(expected = IllegalStateException.class)
    public void reject_an_already_made_decision() throws Exception {
        FinalSeminarApproval approval = createRejected();
        approval.reject("Very bad", Option.<FileDescription>empty(), Option.<FileDescription>empty());
    }

    @Test
    public void stores_reason_when_approving() throws Exception {
        FinalSeminarApproval approval = createUndecided();
        String reason = "my custom reason";
        approval.approve(reason, Option.<FileDescription>empty(), Option.<FileDescription>empty());
        assertThat(approval.getCurrentReason(), is(reason));
    }

    @Test
    public void stores_optional_attachment_when_rejecting() throws Exception {
        FinalSeminarApproval approval = createUndecided();
        FileDescription attachment = createFileDescription();
        approval.reject("Very bad", Option.apply(attachment), Option.<FileDescription>empty());
        assertThat(approval.getCurrentAttachment().get(), is(attachment));
    }

    private FinalSeminarApproval createApproved() {
        FinalSeminarApproval approval = createUndecided();
        approval.approve("Very good", Option.<FileDescription>empty(), Option.<FileDescription>empty());
        return approval;
    }

    private FinalSeminarApproval createRejected() {
        FinalSeminarApproval approval = createUndecided();
        approval.reject("Very bad", Option.<FileDescription>empty(), Option.<FileDescription>empty());
        return approval;
    }

    private FinalSeminarApproval createUndecided() {
        return createUndecided(createFileDescription());
    }

    private FinalSeminarApproval createUndecided(FileDescription fileDescription) {
        return new FinalSeminarApproval(mock(Project.class), fileDescription, "test", new Date());
    }

    private FileDescription createFileDescription() {
        return mock(FileDescription.class);
    }
}
