package se.su.dsv.scipro.test;

import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.Idea;
import se.su.dsv.scipro.match.IdeaParticipation;
import se.su.dsv.scipro.match.Match;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.lang.reflect.Field;
import java.util.*;

public class ObjectMother {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
    public static final ProjectType MASTER = new ProjectType(DegreeLevel.SECOND_CYCLE, ProjectType.MASTER_STRING, ProjectType.MASTER_STRING);
    public static final User SOME_USER = User.builder().firstName("Kalle").lastName("Testare").emailAddress("kalle@dsv.su.se").build();

    public static final Project SOME_PROJECT = Project.builder()
            .title("Some title")
            .projectType(BACHELOR)
            .headSupervisor(User.builder().firstName("Foo").lastName("Bar").emailAddress("foo@dsv.su.se").build())
            .build();

    public static final Project SOME_MASTER_PROJECT = Project.builder().title("Some title").projectType(MASTER).build();

    static {
        DomainObjects.injectId(BACHELOR, 1L);
        DomainObjects.injectId(SOME_PROJECT, 1L);
        DomainObjects.injectId(SOME_USER, 1L);
    }

    private static <T> T get_with_id(T dom, Long id) {
        try {
            Class<?> doClass = dom.getClass();
            Field idField = doClass.getDeclaredField("id");
            idField.setAccessible(true);
            idField.set(dom, id);
        } catch (Exception ex) {
            System.err.println("Could not inject id into: " + dom);
            ex.printStackTrace();
        }
        return dom;
    }

    private static abstract class AbstractBuilder<T> {
        protected Long id;

        protected abstract T build();

        public T create() {
            return get_with_id(build(), id);
        }
    }

    public static class IdeaBuilder extends AbstractBuilder<Idea> {
        public static final String TITLE = "[How A affects B in C]";
        public static final User SUPERVISOR = new UserBuilder().setId(731L).create();
        public static final ProjectType PROJECT_TYPE = new ProjectType(ProjectType.BACHELOR, "Bachelor", "[Bachelor thesis]");
        public static final ApplicationPeriod APPLICATION_PERIOD = new ApplicationPeriod("[Test period]");
        public static final Match MATCH = new Match();

        static {
            APPLICATION_PERIOD.setCourseStartDate(new Date());
            MATCH.setStatus(Idea.Status.MATCHED);
        }

        private String title = TITLE;
        private User supervisor = SUPERVISOR;
        private Set<IdeaParticipation> authors = new HashSet<>();
        private ProjectType projectType = PROJECT_TYPE;
        private ApplicationPeriod applicationPeriod = APPLICATION_PERIOD;
        private User creator = SUPERVISOR;

        @Override
        protected Idea build() {
            Match match = MATCH;
            match.setSupervisor(supervisor);
            Idea idea = new Idea();

            for (IdeaParticipation author : authors) {
                author.setIdea(idea);
            }

            idea.setTitle(title);
            idea.setMatch(match);
            idea.setIdeaParticipations(authors);
            idea.setProjectType(projectType);
            idea.setApplicationPeriod(applicationPeriod);
            idea.setCreator(creator);
            return idea;
        }

        public IdeaBuilder setId(Long id) {
            this.id = id;
            return this;
        }

        public IdeaBuilder setTitle(String title) {
            this.title = title;
            return this;
        }

        public IdeaBuilder setSupervisor(User user) {
            this.supervisor = user;
            return this;
        }
    }

    public static class _User {
        public static final String FIRST_NAME = "TEST_USER_FIRST_NAME";
        public static final String LAST_NAME = "TEST_USER_LAST_NAME";

        public static User create(Long id) {
            return create(id, FIRST_NAME, LAST_NAME);
        }

        public static User create(Long id, String firstName, String lastName) {
            User user = get_with_id(new User(), id);
            user.setFirstName(firstName);
            user.setLastName(lastName);
            return user;
        }
    }

    public static class _Project {
        public static Project create(Long id) {
            Project project = get_with_id(new Project(), id);
            project.setTitle("test project");
            project.setProjectType(new ProjectType(ProjectType.BACHELOR, "Bachelor", "tt"));

            User headSupervisorUser = get_with_id(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build(), 6L);
            project.setHeadSupervisor(headSupervisorUser);

            User author1 = get_with_id(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build(), 2L);
            author1.setFirstName("Kalle");
            author1.setLastName("Testare");
            author1.setEmailAddress("kalle@dsv.su.se");
            User author2 = get_with_id(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build(), 3L);
            SortedSet<User> authors = new TreeSet<>(new User.ByNameComparator());
            authors.add(author1);
            authors.add(author2);
            project.setProjectParticipants(authors);

            User reviewer1 = get_with_id(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build(), 4L);

            User reviewer2 = get_with_id(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build(), 5L);

            User cosupervisor1 = get_with_id(User.builder().firstName("Bob").lastName("Sponge").emailAddress("bob@example.com").build(), 8L);

            project.addReviewer(reviewer1);
            project.addReviewer(reviewer2);
            project.addCoSupervisor(cosupervisor1);

            return project;
        }
    }
}
