package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.activityplan.ActivityPlanTemplate;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.datatables.LabelAndLinkColumn;
import se.su.dsv.scipro.datatables.target.AddTargetLinkPanel;
import se.su.dsv.scipro.match.*;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class AdminApplicationPeriodsPanel extends Panel {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdminApplicationPeriodsPanel.class);
    private final ExportableDataPanel<ApplicationPeriod, String> dp;
    @Inject
    private ApplicationPeriodService applicationPeriodService;
    @Inject
    private ApplicationPeriodFacade applicationPeriodFacade;
    @Inject
    private ApplicationPeriodProjectTypeService applicationPeriodProjectTypeService;

    private ModalWindowPlus modalWindowPlus;
    private final FeedbackPanel feedback;

    public AdminApplicationPeriodsPanel(String id) {
        super(id);

        feedback = new FencedFeedbackPanel("feedback", this);
        feedback.setOutputMarkupId(true);
        add(feedback);

        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);

        add(new Link<Void>("createLink") {
            @Override
            public void onClick() {
                setResponsePage(new AdminEditApplicationPeriodPage(new PageParameters()));
            }
        });
        GenericDataProvider<ApplicationPeriod> provider = new GenericDataProvider<>(applicationPeriodService);
        provider.setSort("startDate", SortOrder.DESCENDING);
        dp = new ExportableDataPanel<>("dp", createColumns(), provider);
        add(dp);
        add(new InfoPanel("infoPanel", new ResourceModel("info.text")));
    }

    private List<? extends IColumn<ApplicationPeriod, String>> createColumns() {
        List<IColumn<ApplicationPeriod, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<ApplicationPeriod>(Model.of("Start"), "startDate", "startDate", DateStyle.DATETIME));
        columns.add(new DateColumn<ApplicationPeriod>(Model.of("End"), "endDate", "endDate", DateStyle.DATETIME));
        columns.add(new DateColumn<ApplicationPeriod>(Model.of("Course start"), "courseStartDate", "courseStartDate", DateStyle.DATETIME));
        addTypesColumn(columns);
        columns.add(new PropertyColumn<ApplicationPeriod, String>(Model.of("Name"), "name", "name"));
        addEditColumn(columns);
        addTotalRemainingColumn(columns);
        addActivityPlanTemplateColumn(columns);
        addExemptionsColumn(columns);
        addDeleteColumn(columns);
        return columns;
    }

    private void addTypesColumn(List<IColumn<ApplicationPeriod, String>> columns) {
        columns.add(new AbstractExportableColumn<ApplicationPeriod, String, String>(Model.of("Types")) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> item, String s, IModel<ApplicationPeriod> iModel) {
                item.add(new DisplayMultiplesPanel<ProjectType>(s, new ListAdapterModel<>(new PropertyModel<Iterable<ProjectType>>(iModel, "projectTypes"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<ProjectType> t) {
                        return new Label(componentId, new PropertyModel<ProjectType>(t, "name"));
                    }
                });
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                StringBuilder s = new StringBuilder();
                Set<ProjectType> projectTypes = rowModel.getObject().getProjectTypes();
                for (ProjectType projectType: projectTypes) {
                    s.append(projectType.getName()).append(", ");
                }
                return Model.of(s.toString());
            }
        });
    }

    private void addEditColumn(List<IColumn<ApplicationPeriod, String>> columns) {
        columns.add(new ClickableIconColumn<ApplicationPeriod>(Model.of("Edit"), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<ApplicationPeriod> applicationPeriodModel, AjaxRequestTarget target) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(ApplicationPeriod.class), applicationPeriodModel.getObject().getId());
                setResponsePage(new AdminEditApplicationPeriodPage(pp));
            }
        });
    }

    private void addTotalRemainingColumn(List<IColumn<ApplicationPeriod, String>> columns) {
        columns.add(new AbstractExportableColumn<ApplicationPeriod, String, String>(Model.of("Targets (total/remaining)")) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> item, String s, IModel<ApplicationPeriod> iModel) {
                item.add(new AddTargetLinkPanel(s, iModel));
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                ApplicationPeriod object = rowModel.getObject();
                StringBuilder s = new StringBuilder();
                Set<ProjectType> projectTypes = object.getProjectTypes();
                for (ProjectType projectType: projectTypes) {
                    s.append(projectType.getName()).append(": ")
                            .append(String.valueOf(applicationPeriodFacade.totalTargets(object, projectType)))
                            .append("/")
                            .append(String.valueOf(applicationPeriodFacade.remainingTargets(object, projectType)))
                            .append(" ");
                }

                return Model.of(s.toString());
            }
        });
    }

    private void addActivityPlanTemplateColumn(List<IColumn<ApplicationPeriod, String>> columns) {
        columns.add(new AbstractExportableColumn<ApplicationPeriod, String, String>(Model.of("Activity plan templates")) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> item, final String s, final IModel<ApplicationPeriod> iModel) {
                item.add(new ApplicationPeriodActivityPlanTemplateColumnPanel(s, iModel) {
                    @Override
                    public void onClick(AjaxRequestTarget target) {
                        iModel.setObject(applicationPeriodService.findOne(iModel.getObject().getId()));
                        modalWindowPlus.setContent(new EditApplicationPeriodActivityPlanTemplatePanel(modalWindowPlus.getContentId(), iModel));
                        modalWindowPlus.setTitle("Edit activity plan templates");
                        modalWindowPlus.show(target);
                    }
                });
            }

            @Override
            public IModel<String> getDataModel(IModel<ApplicationPeriod> rowModel) {
                StringBuilder s = new StringBuilder();
                Set<ProjectType> projectTypes = rowModel.getObject().getProjectTypes();
                for (ProjectType projectType: projectTypes) {
                    ApplicationPeriodProjectType applicationPeriodProjectType =
                            applicationPeriodProjectTypeService.findOne(rowModel.getObject(), projectType);
                    ActivityPlanTemplate activityPlanTemplate = applicationPeriodProjectType.getActivityPlanTemplate();
                    s.append(projectType.getName()).append(": ")
                            .append(activityPlanTemplate != null ? activityPlanTemplate.getTitle() : "None").append(" ");
                }
                return Model.of(s.toString());
            }
        });
    }

    private void addDeleteColumn(List<IColumn<ApplicationPeriod, String>> columns) {
        columns.add(new ClickableIconColumn<ApplicationPeriod>(Model.of("Delete"), null, ClickableIconColumn.DELETE, "Are you sure you want to delete this application period?") {
            @Override
            protected void onClick(IModel<ApplicationPeriod> clicked, AjaxRequestTarget target) {
                try {
                    applicationPeriodService.delete(clicked.getObject().getId());
                    info("Deleted application period");
                } catch (Exception e) {
                    String message = "Could not delete application period";
                    error(message);
                    LOGGER.info(message, e);
                }
                target.add(feedback);
                target.add(dp);
            }
        });
    }

    private void addExemptionsColumn(List<IColumn<ApplicationPeriod, String>> columns) {
        columns.add(new AbstractExportableColumn<ApplicationPeriod, String, Integer>(Model.of("Exemptions")) {
            @Override
            public void populateItem(Item<ICellPopulator<ApplicationPeriod>> item, String componentId, final IModel<ApplicationPeriod> model) {
                item.add(new LabelAndLinkColumn<ApplicationPeriod>(componentId, model, "Edit exemptions", true) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<ApplicationPeriod> model) {
                        PageParameters pp = new PageParameters();
                        pp.add(PageParameterKeys.MAP.get(ApplicationPeriod.class), model.getObject().getId());
                        setResponsePage(AdminEditApplicationPeriodExemptionsPage.class, pp);
                    }

                    @Override
                    public String labelString() {
                        return Integer.toString(model.getObject().noOfApplicationPeriodExemptions()) + " ";
                    }
                });
            }

            @Override
            public IModel<Integer> getDataModel(IModel<ApplicationPeriod> rowModel) {
                return Model.of(rowModel.getObject().noOfApplicationPeriodExemptions());
            }
        });
    }
}