package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.admin.pages.AdminMailPage;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.SupervisorAutoComplete;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminApplicationPeriod;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.target.EditTargetPanel;
import se.su.dsv.scipro.match.*;
import se.su.dsv.scipro.springdata.services.SupervisorService;
import se.su.dsv.scipro.springdata.services.SupervisorService.FilterParams;
import se.su.dsv.scipro.springdata.services.UnitService;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.*;

import static se.su.dsv.scipro.data.enums.DateStyle.DATETIME;

public class AdminEditTargetsPage extends AbstractAdminMatchPage implements MenuHighlightAdminApplicationPeriod {

    public static final boolean EDITABLE = true;
    public static final boolean NOT_EDITABLE = false;
    public static final String EMPLOYEE_LIST = "employeeList";
    public static final String FILTER_FORM = "filterForm";
    public static final String HEADER = "header";
    public static final String FEEDBACK = "feedback";

    @Inject
    private SupervisorService supervisorService;
    @Inject
    private ApplicationPeriodService applicationPeriodService;
    @Inject
    private IdeaService ideaService;
    @Inject
    private UnitService unitService;
    @Inject
    private ResearchAreaService researchAreaService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private LanguageService languageService;
    @Inject
    private TargetService targetService;

    private IModel<ApplicationPeriod> current;
    private ExportableDataPanel<User,String> table;
    private FeedbackPanel feedback;
    private FilteredDataProvider<User, FilterParams> provider;
    private Link sendMailLink;

    public AdminEditTargetsPage(IModel<ApplicationPeriod> model) {

        this.current = model;
        addFeedback();
        addApplicationPeriodInfo();

        IModel<FilterParams> filter = prepareFilter();
        setUpDataTable(filter);

        add(new FilterForm(FILTER_FORM, filter));

        sendMailLink = new Link("sendMail"){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(provider.size() > 0);
            }
            @Override
            public void onClick() {
                Set<User> listOfUsers = new HashSet<>();
                Iterator<?> user = provider.iterator(0, provider.size());
                while (user.hasNext()) {
                    listOfUsers.add((User)user.next());
                }
                setResponsePage(new AdminMailPage(listOfUsers));
            }
        };
        add(sendMailLink);
    }

    private void addFeedback() {
        feedback = new FencedFeedbackPanel(FEEDBACK, this);
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    private void addApplicationPeriodInfo() {
        add(new Label(HEADER, new StringResourceModel("header", current)));
        add(new DateLabel("startDate", new PropertyModel<Date>(current, "startDate"), DATETIME));
        add(new DateLabel("endDate", new PropertyModel<Date>(current, "endDate"), DATETIME));
        add(new DateLabel("courseStartDate", new PropertyModel<Date>(current, "courseStartDate"), DATETIME));
    }

    private void setUpDataTable(IModel<FilterParams> filter) {
        provider = new FilteredDataProvider<>(supervisorService, filter);
        provider.setSort("firstName", SortOrder.ASCENDING);
        table = new ExportableDataPanel<>(EMPLOYEE_LIST, columns(), provider);
        table.setOutputMarkupId(true);
        add(table);
    }

    private Model<FilterParams> prepareFilter() {
        FilterParams filter = new FilterParams();
        filter.setLanguages(languageService.findAll());
        return Model.of(filter);
    }

    private List<? extends IColumn<User, String>> columns() {
        List<IColumn<User, String>> list = new ArrayList<>();
        list.add(new PropertyColumn<User, String>(Model.of(getString("name")), "firstName", "fullName"));
        list.add(new PropertyColumn<User, String>(Model.of(getString("unit")), "unit.title"));
        list.add(new PropertyColumn<User, String>(Model.of(getString("researchAreas")), "researchAreaNames"));
        addColumnsForProjectTypes(list, current, EDITABLE);
        for (ApplicationPeriod applicationPeriod : applicationPeriodService.getPreviousPeriods(current.getObject(), 2)) {
            addColumnsForProjectTypes(list, new DetachableServiceModel<>(applicationPeriodService, applicationPeriod), NOT_EDITABLE);
        }
        return list;
    }

    private void addColumnsForProjectTypes(List<IColumn<User, String>> list, IModel<ApplicationPeriod> applicationPeriod, boolean isEditable) {
        for (final ProjectType projectType : applicationPeriod.getObject().getProjectTypes()) {
            list.add(matchedColumn(new DetachableServiceModel<>(projectTypeService, projectType), applicationPeriod));
            list.add(editTargetColumn(new DetachableServiceModel<>(projectTypeService, projectType), applicationPeriod, isEditable));
        }
    }

    private IColumn<User, String> editTargetColumn(final IModel<ProjectType> projectType, final IModel<ApplicationPeriod> appPeriod, final boolean isEditable) {
        return new AbstractExportableColumn<User, String, Integer>(new StringResourceModel("targetCurrent", appPeriod, null, projectType.getObject().getName()).wrapOnAssignment(this)) {
            @Override
            public void populateItem(Item<ICellPopulator<User>> item, String id, IModel<User> employee) {
                item.add(new EditTargetPanel(id, appPeriod, employee, projectType, isEditable) {
                    @Override
                    public void update(AjaxRequestTarget target, User user) {
                        success(getString("saved", Model.of(user)));
                        target.add(feedback);
                        target.add(table);
                    }
                });
            }
            @Override
            public IModel<Integer> getDataModel(IModel<User> rowModel) {
                return Model.of(targetService.findOne(appPeriod.getObject(), rowModel.getObject(), projectType.getObject()).getTarget());
            }
        };
    }

    private IColumn<User, String> matchedColumn(final DetachableServiceModel<ProjectType> projectType, final IModel<ApplicationPeriod> appPeriod) {
        return new AbstractExportableColumn<User, String, Long>(new StringResourceModel("matchedCurrent", appPeriod, null, projectType.getObject().getName()).wrapOnAssignment(this)) {
            @Override
            public void populateItem(Item<ICellPopulator<User>> item, String id, IModel<User> employee) {
                item.add(new Label(id, Long.toString(ideaService.countMatched(appPeriod.getObject(), employee.getObject(), projectType.getObject()))));
            }

            @Override
            public IModel<Long> getDataModel(IModel<User> rowModel) {
                Long matched = ideaService.countMatched(appPeriod.getObject(), rowModel.getObject(), projectType.getObject());
                return Model.of(matched);
            }
        };
    }

    private class FilterForm extends Form<FilterParams> {
        public FilterForm(String id, final IModel<FilterParams> model) {
            super(id, new CompoundPropertyModel<>(model));
            add(new CheckBoxMultipleChoice<>("languages", languageService.findAll(), new ChoiceRenderer<>("name", "id"))
                    .setLabelPosition(AbstractChoice.LabelPosition.WRAP_AFTER));
            add(new DropDownChoice<>("unit", unitService.findAllAsList(),
                    new ChoiceRenderer<Unit>("title", "id")).setNullValid(true));
            add(new DropDownChoice<>("researchArea", researchAreaService.findNotDeleted(),
                    new ChoiceRenderer<ResearchArea>("title", "id")).setNullValid(true));
            add(new SupervisorAutoComplete("supervisor"));
            add(new CheckBox("hasTargetsInApplicationPeriod", new TargetCheckBoxModel(model)));
        }
        @Override
        public void onSubmit() {
            super.onSubmit();
            sendMailLink.setVisible(provider.size() > 0);
        }

    }

    private class TargetCheckBoxModel extends AbstractCheckBoxModel {
        private IModel<FilterParams> filter;

        public TargetCheckBoxModel(IModel<FilterParams> filter) {
            this.filter = filter;
        }

        @Override
        public boolean isSelected() {
            return filter.getObject().getHasTargetsInApplicationPeriod() != null;
        }

        @Override
        public void select() {
            filter.getObject().setHasTargetsInApplicationPeriod(current.getObject());
        }

        @Override
        public void unselect() {
            filter.getObject().setHasTargetsInApplicationPeriod(null);
        }
    }
}
