package se.su.dsv.scipro.checklists;

import com.googlecode.wickedcharts.highcharts.options.*;
import com.googlecode.wickedcharts.highcharts.options.color.HexColor;
import com.googlecode.wickedcharts.highcharts.options.color.NullColor;
import com.googlecode.wickedcharts.highcharts.options.series.Point;
import com.googlecode.wickedcharts.highcharts.options.series.PointSeries;
import com.googlecode.wickedcharts.wicket6.highcharts.Chart;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.checklist.ChecklistAnswerEnum;
import se.su.dsv.scipro.checklist.ChecklistService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;
import java.awt.*;
import java.util.Map;
import java.util.TreeMap;

@ProjectModuleComponent(ProjectModule.CHECKLIST)
public class ChecklistOverviewPanel extends GenericPanel<Project> {

    @Inject
    private ChecklistService checklistService;
    @Inject
    private UserProfileService profileService;

    private final Map<ChecklistAnswerEnum, Long> map;
    private WebMarkupContainer details;
    private Label expand;
    private Label collapse;

    public ChecklistOverviewPanel(String id, IModel<Project> model) {
        super(id, model);

        addDetails();
        addToggleLink();

        map = new TreeMap<>();
        Long totalAnswered = 0L;
        Long totalUnanswered = 0L;

        for (ChecklistAnswerEnum answer : ChecklistAnswerEnum.values()) {
            if (answer.equals(ChecklistAnswerEnum.NO_ANSWER)) {
                totalUnanswered = countUnanswered().getObject();
                map.put(answer, totalUnanswered);
            } else {
                Long answered = countAnswers(answer).getObject();
                totalAnswered = totalAnswered + answered;
                map.put(answer, answered);
            }
        }

        details.add(new Label("totalQuestions", totalAnswered + totalUnanswered));
        details.add(new Label("totalAnswered", totalAnswered));
        details.add(new Label("totalUnanswered", totalUnanswered));
        details.add(new Chart("chart", new OOptions()));
    }

    private LoadableDetachableModel<Long> countAnswers(final ChecklistAnswerEnum checklistAnswerEnum) {
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return checklistService.countAnswers(getModelObject(), checklistAnswerEnum);
            }
        };
    }

    private LoadableDetachableModel<Long> countUnanswered() {
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return checklistService.countUnanswered(getModelObject());
            }
        };
    }

    private static class COptions extends ChartOptions {
        private static final long serialVersionUID = 1L;

        public COptions() {
            setBackgroundColor(new NullColor());
            setBorderColor(new NullColor());
            setPlotBackgroundColor(new NullColor());
        }
    }

    private class OOptions extends Options {
        private static final long serialVersionUID = 1L;

        public OOptions() {
            setChartOptions(new COptions());
            setExporting(new ExportingOptions().setEnabled(false));
            setCredits(new CreditOptions().setEnabled(false));
            setTitle(new Title(""));

            PointSeries pointSeries = new PointSeries();
            pointSeries.setName("");
            pointSeries.setType(SeriesType.PIE);
            for (Map.Entry<ChecklistAnswerEnum, Long> entry : map.entrySet()) {
                if (entry.getKey().equals(ChecklistAnswerEnum.GREEN)) {
                    pointSeries.addPoint(new Point("OK", entry.getValue(), new HexColor("#4bcb32")));
                } else if (entry.getKey().equals(ChecklistAnswerEnum.YELLOW)) {
                    pointSeries.addPoint(new Point("Minor corrections", entry.getValue(), new HexColor("#ffcc03")));
                } else if (entry.getKey().equals(ChecklistAnswerEnum.RED)) {
                    pointSeries.addPoint(new Point("Not done", entry.getValue(), new HexColor("#eb1f00")));
                } else if (entry.getKey().equals(ChecklistAnswerEnum.NOT_APLICABLE)) {
                    pointSeries.addPoint(new Point("Not applicable", entry.getValue(), Color.DARK_GRAY));
                } else {
                    pointSeries.addPoint(new Point("Unanswered", entry.getValue(), Color.LIGHT_GRAY));
                }
            }
            addSeries(pointSeries);
        }
    }

    private void addDetails() {
        details = new WebMarkupContainer("details") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(profileService.isExpandChecklistOverview(SciProSession.get().getUser()));
            }
        };
        add(details);
        details.setOutputMarkupPlaceholderTag(true);
    }

    private void addToggleLink() {
        AjaxLink<Void> toggleLink = new AjaxLink<Void>("toggle") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                if (profileService.isExpandChecklistOverview(SciProSession.get().getUser())) {
                    profileService.setExpandChecklistOverview(SciProSession.get().getUser(), false);
                } else {
                    profileService.setExpandChecklistOverview(SciProSession.get().getUser(), true);
                }
                target.add(details);
                target.add(expand);
                target.add(collapse);
            }
        };
        add(toggleLink);

        expand = new Label("expand", "Show more") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!profileService.isExpandChecklistOverview(SciProSession.get().getUser()));
            }
        };
        expand.setOutputMarkupPlaceholderTag(true);
        toggleLink.add(expand);

        collapse = new Label("collapse", "Show less") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(profileService.isExpandChecklistOverview(SciProSession.get().getUser()));
            }
        };
        collapse.setOutputMarkupPlaceholderTag(true);
        toggleLink.add(collapse);
    }
}