package se.su.dsv.scipro.components;

import org.apache.wicket.Page;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.io.IClusterable;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlight;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ABC-class for menu's of different types.
 * Subclasses should implement their own data feed mechanics via <code>getItemList</code> and provide a menu type via <code>getMenuType</code>
 * The returned menu-type is used to provide CSS-class mappings for the menu structure, see the MenuType enum for alternatives.
 */
public abstract class AbstractMenuPanel extends Panel {

    private final Class<? extends Page> menuContainerCommonSuperClass;
    private final Class<? extends Page> containerClass;

    private static final Map<MenuType, String> CSS_CLASS = new HashMap<MenuType, String>() {
        {
            put(MenuType.NONE, "");
            put(MenuType.TAB, "tab-menu");
            put(MenuType.TAB_MULTIPLE_ROWS, "multi-tab-menu");  /*lägg in klasser här?*/
            put(MenuType.HORIZONTAL, "horizontal-menu");
            put(MenuType.VERTICAL, "vertical-menu");
            put(MenuType.BLUE, "blue-tab-menu");
            put(MenuType.THIRD, "third-level-menu");
        }
    };

    /**
     * Enumeration of supported menu types.
     */
    public enum MenuType {
        NONE,
        TAB,
        TAB_MULTIPLE_ROWS,
        HORIZONTAL,
        VERTICAL, BLUE, THIRD
    }

    /**
     * Superclass constructor, arguments
     *
     * @param id                            Wicket identifier for the component
     * @param menuContainerCommonSuperClass [Needs documentation]
     * @param containerClass                [Needs documentation]
     */
    public AbstractMenuPanel(final String id, final Class<? extends Page> menuContainerCommonSuperClass, final Class<? extends Page> containerClass) {
        super(id);
        this.menuContainerCommonSuperClass = menuContainerCommonSuperClass;
        this.containerClass = containerClass;
    }

    /**
     * Override provides an acceptable way to use subtype-specific parameters in a safe way.
     */
    @Override
    protected void onInitialize() {
        super.onInitialize();

        final WebMarkupContainer menuContainer = new WebMarkupContainer("menuContainer");

        menuContainer.add(new AttributeAppender("class", Model.of(CSS_CLASS.get(getMenuType())), " "));

        final ListView<MenuItem> tabList = new ListView<MenuItem>("menuList", getItemList()) {
            @Override
            protected void populateItem(ListItem<MenuItem> item) {
                final MenuItem mi = item.getModelObject();
                boolean highLightAsCurrentByInterface = false;
                Class<?> menuItemInterface = mi.getMenuHighlightInterface();
                if (menuItemInterface != null) {
                    for (Class<?> actualPageInterface : containerClass.getInterfaces()) {
                        if (actualPageInterface.equals(menuItemInterface)) {
                            highLightAsCurrentByInterface = true;
                            break;
                        }
                    }
                }
                if (highLightAsCurrentByInterface || mi.getTargetClass().equals(containerClass)  ||
                        (mi.getTargetClass().getSuperclass().isAssignableFrom(containerClass)
                                && !mi.getTargetClass().getSuperclass().equals(menuContainerCommonSuperClass)
                                    && !mi.isPopUp())) {
                    item.add(new AttributeAppender("class", Model.of("current"), " "));
                }


                BookmarkablePageLink<Void> link = new BookmarkablePageLink<>("menuItem", mi.getTargetClass(), mi.getPageParameters());
                link.add(new Label("linkText", mi.getName()));
                if (mi.isPopUp()) {
                    link.add(new AttributeAppender("target", Model.of("_blank")));
                }

                item.add(link);
            }
        };
        menuContainer.add(tabList);
        add(menuContainer);
    }

    /**
     * May not rely on anything in the implementing class's constructor as this will not have been initialized yet.
     * Items are rendered from left to right (or top to bottom depending on type of menu) as you might expect,
     * the first item in the list will be the first menu item on the left/top.
     */
    protected abstract List<MenuItem> getItemList();

    /**
     * Implement in subclass to return the wanted MenuType.
     */
    protected abstract MenuType getMenuType();

    /**
     * Page parameters can safely be set to null if not needed
     *
     */
    protected static class MenuItem implements IClusterable {

        private IModel<String> name;
        private Class<? extends Page> targetClass;
        private PageParameters pp;
        private Class<? extends MenuHighlight> menuHighlightInterface;
        private boolean popUp = false;

        public MenuItem(final String name, final Class<? extends Page> targetClass) {
            this(name, targetClass, null, null, false);
        }

        public MenuItem(final String name, final Class<? extends Page> targetClass, final Class<? extends MenuHighlight> menuHilightInterface) {
            this(name, targetClass, menuHilightInterface, null, false);
        }

        public MenuItem(final String name, final Class<? extends Page> targetClass, PageParameters pp) {
            this(name, targetClass, null, pp, false);
        }

        public MenuItem(final String name, final Class<? extends Page> targetClass, final Class<? extends MenuHighlight> menuHilightInterface, PageParameters pp) {
            this(name, targetClass, menuHilightInterface, pp, false);
        }

        public MenuItem(String name, final Class<? extends Page> targetClass, boolean isPopUp) {
            this(name, targetClass, null, null, isPopUp);
        }

        public MenuItem(final String name, final Class<? extends Page> targetClass, final Class<? extends MenuHighlight> menuHilightInterface, PageParameters pp, boolean popUp) {
            this.name = Model.of(name);
            this.targetClass = targetClass;
            this.menuHighlightInterface = menuHilightInterface;
            this.pp = pp;
            this.popUp = popUp;
        }

        public IModel<String> getName() {
            return name;
        }

        public Class<? extends Page> getTargetClass() {
            return targetClass;
        }

        public PageParameters getPageParameters() {
            return pp;
        }

        public Class<? extends MenuHighlight> getMenuHighlightInterface() {
            return menuHighlightInterface;
        }

        public boolean isPopUp() {
            return popUp;
        }
    }//MenuItem
}
