package se.su.dsv.scipro.datatables.project;

import com.google.common.eventbus.EventBus;
import lombok.Getter;
import lombok.Setter;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.admin.pages.AdminEditProjectPage;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.datatables.AjaxCheckboxWrapper;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.project.ReviewerAssignedEvent;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.*;

import static java.util.Arrays.asList;

@Authorization(authorizedRoles = {Roles.ADMIN})
public class ProjectDataPanel extends Panel {

    public static final int MIN_TITLE_LENGTH = 3;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectService projectService;
    @Inject
    private EventBus eventBus;
    @Inject
    private NotificationController notificationController;

    private ExportableDataPanel<Project, String> exportableDataPanel;
    private ProjectService.Filter filter;
    private FencedFeedbackPanel feedback;

    public ProjectDataPanel(String id) {
        super(id);
        add(new FilterForm("filterForm"));
        addCreateNewProjectLink();
        addFeedbackPanel();
        addDataTable();
    }

    private void addDataTable() {
        SortableDataProvider<Project, String> provider = new FilteredDataProvider<>(projectService, Model.of(filter));
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        exportableDataPanel = new ExportableDataPanel<>("dataTable", createColumns(), provider);
        add(exportableDataPanel);
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();

        columns.add(new DateColumn<Project>(Model.of("Started"), "dateCreated", "dateCreated"));
        columns.add(titleColumn());
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name", "projectType.name"));
        columns.add(authorsColumn());
        columns.add(new PropertyColumn<Project, String>(Model.of("Status"), "projectStatus", "projectStatus"));
        columns.add(supervisorColumn());
        columns.add(reviewerColumn());
        columns.add(new PropertyColumn<Project, String>(Model.of("Research area"), "headSupervisor.researchAreaNames"));
        columns.add(inactivateCheckbox());
        return columns;
    }

    private AbstractColumn<Project, String> supervisorColumn() {
        return new PropertyColumn<Project, String>(Model.of("Head Superv."), "headSupervisor.fullName") {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                User supervisor = rowModel.getObject().getHeadSupervisor();
                if (supervisor != null) {
                    cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "headSupervisor")));
                } else {
                    cellItem.add(new Label(componentId, "-"));
                }
            }
        };
    }

    private AbstractColumn<Project, String> authorsColumn() {
        return new MultipleUsersColumn<Project, String>(Model.of("Authors")) {
            @Override
            public IModel<? extends List<? extends User>> getUsers(final IModel<Project> rowModel) {
                return new ListAdapterModel<>(new PropertyModel<Set<User>>(rowModel, "projectParticipants"));
            }
        };
    }

    private AbstractColumn<Project, String> titleColumn() {
        return new PropertyColumn<Project, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> components, String s, final IModel<Project> model) {
                components.add(new AjaxLinkPanel<Project>(s, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Project> model) {
                        openCRUDPanel(model);
                    }
                });
            }
        };
    }

    private AbstractColumn<Project, String> reviewerColumn() {
        return new PropertyColumn<Project, String>(Model.of("Reviewer"), "reviewer") {
            @Override
            public void populateItem(final Item<ICellPopulator<Project>> cellItem, final String componentId, final IModel<Project> rowModel) {
                cellItem.add(new EmployeeAutoCompleteDivPanel(componentId, new PropertyModel<User>(rowModel, "reviewer")) {
                    @Override
                    public void onNewEmployeeSelection(final AjaxRequestTarget target, final User newSelection) {
                        Project project = rowModel.getObject();
                        if (project.getHeadSupervisor().equals(newSelection)) {
                            error(getString("supervisor.reviewer.error"));
                        } else if (!newSelection.getRoles().contains(Roles.REVIEWER)) {
                            error(getString("not.reviewer.error"));
                        } else {
                            for (User currentReviewer : project.getReviewers()) {
                                project.removeReviewer(currentReviewer);
                            }
                            project.addReviewer(newSelection);
                            notifyReviewersChanged(project);
                            eventBus.post(new ReviewerAssignedEvent(project, newSelection));

                            projectService.save(project);
                            info(getString("reviewer.updated"));
                        }
                        target.add(feedback);
                        target.add(exportableDataPanel);
                    }
                });
            }
        };
    }

    private AbstractColumn<Project, String> inactivateCheckbox() {
        return new AbstractExportableColumn<Project, String, Boolean>(Model.of("Active")) {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, final IModel<Project> model) {
                final Project project = model.getObject();
                final boolean currentState = !project.getProjectStatus().equals(ProjectStatus.INACTIVE);
                cellItem.add(new AjaxCheckboxWrapper(componentId, Model.of(currentState)) {
                    @Override
                    public void onChange(AjaxRequestTarget target, boolean selected) {
                        projectService.toggleActiveStatus(model.getObject(), selected);
                        success(getString("inactive", model));
                        target.add(feedback);
                    }
                });
            }

            @Override
            public IModel<Boolean> getDataModel(IModel<Project> rowModel) {
                return Model.of(!rowModel.getObject().getProjectStatus().equals(ProjectStatus.INACTIVE));
            }
        };
    }

    private void notifyReviewersChanged(Project project) {
        notificationController.notifyProject(project, ProjectEvent.Event.REVIEWERS_CHANGED, new NotificationSource());
    }

    private void addFeedbackPanel() {
        feedback = new FencedFeedbackPanel("feedback", this);
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    private void addCreateNewProjectLink() {
        AjaxLink<Void> newLink = new AjaxLink<Void>("newLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                openCRUDPanel(new DetachableServiceModel<>(projectService, new Project()));
            }

            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(SciProSession.get().authorizedForRole(Roles.SYSADMIN) && SciProSession.get().hasActualRole(Roles.SYSADMIN));
            }
        };
        add(newLink);
    }

    private void openCRUDPanel(final IModel<Project> project) {
        PageParameters pp = new PageParameters();
        pp.set(PageParameterKeys.MAP.get(Project.class), project.getObject().getId());
        setResponsePage(AdminEditProjectPage.class, pp);
    }

    private class FilterForm extends Form<Void> {
        @Getter
        @Setter
        @SuppressWarnings("UnusedDeclaration")
        private String titleString;

        public FilterForm(String id) {
            super(id);
            filter = new ProjectService.Filter();

            filter.setFilterExternal(false);

            add(new AjaxCheckBox("noReviewer", new PropertyModel<Boolean>(filter, "noReviewer")) {
                @Override
                protected void onUpdate(final AjaxRequestTarget target) {
                    target.add(exportableDataPanel);
                }
            });

            addDateFilter();
            addSupervisorFilter();
            addProjectTitleFilter();
            addAuthorFilter();
            addProjectTypeFilter();
            addProjectStatusFilter();
            addRedStateFilter();
        }

        private void addRedStateFilter() {
            TextField<Integer> redStateField = new TextField<>("redStateFilter", new PropertyModel<Integer>(filter, "redStateTime"), Integer.class);
            redStateField.add(new AjaxFormComponentUpdatingBehavior("onkeyup") {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    target.add(exportableDataPanel);
                }
            });
            add(redStateField);
        }

        private void addDateFilter() {
            add(new DeactivatableDatePickerPanel("dateFilter",
                    new PropertyModel<Date>(filter, "createdAfter"),
                    new PropertyModel<Date>(filter, "createdBefore")) {
                @Override
                protected void datesChanged(AjaxRequestTarget target) {
                    target.add(exportableDataPanel);
                }
            });
        }

        private void addSupervisorFilter() {
            add(new EmployeeAutoComplete("supervisorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setSupervisor(newSelection);
                    pTarget.add(exportableDataPanel);
                }
            });
        }

        private void addProjectTitleFilter() {
            TextField<String> titleField = new TextField<>("titleFilter", new PropertyModel<String>(this, "titleString"));
            titleField.add(new AjaxFormComponentUpdatingBehavior("onkeyup") {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    if (titleString != null && titleString.length() >= MIN_TITLE_LENGTH) {
                        filter.setTitleContains(titleString);
                    } else {
                        filter.setTitleContains(null);
                    }
                    target.add(exportableDataPanel);
                }
            });
            add(titleField);
        }

        private void addAuthorFilter() {
            add(new AuthorAutoComplete("authorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setAuthorUser(newSelection);
                    pTarget.add(exportableDataPanel);
                }
            });
        }

        private void addProjectTypeFilter() {
            AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                    projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)),
                    projectTypeService.findAllActive(),
                    new ChoiceRenderer<ProjectType>("name", "id")) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    if (!getModelObject().isEmpty()) {
                        filter.setProjectTypes(getModelObject());
                    }
                    target.add(exportableDataPanel);
                }
            };
            filter.setProjectTypes(projectType.getModelObject());
            add(projectType);
        }

        private void addProjectStatusFilter() {
            AjaxCheckBoxMultipleChoice<ProjectStatus> projectStatus = new AjaxCheckBoxMultipleChoice<ProjectStatus>("statusFilter",
                    new ArrayList<>(asList(ProjectStatus.ACTIVE)),
                    asList(ProjectStatus.values()),
                    new EnumChoiceRenderer<ProjectStatus>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    filter.setStatuses(new HashSet<>(getModelObject()));
                    target.add(exportableDataPanel);
                }
            };
            add(projectStatus);
            filter.setStatuses(new HashSet<>(projectStatus.getModelObject()));
        }
    }
}