package se.su.dsv.scipro.datatables.project;

import lombok.Getter;
import lombok.Setter;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.admin.pages.AdminEditProjectPage;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.date.DeactivatableDatePickerPanel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.ProjectTypeService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.*;

import static java.util.Arrays.asList;

@Authorization(authorizedRoles = {Roles.ADMIN})
public class ProjectExternalOrganisationPanel extends Panel {

    public static final int MIN_TITLE_LENGTH = 3;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private ProjectService projectService;

    private ExportableDataPanel<Project, String> exportableDataPanel;
    private ProjectService.Filter filter;

    public ProjectExternalOrganisationPanel(String id) {
        super(id);
        add(new FilterForm("filterForm"));
        addFeedbackPanel();
        addDataTable();
    }

    private void addDataTable() {
        SortableDataProvider<Project, String> provider = new FilteredDataProvider<>(projectService, Model.of(filter));
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        exportableDataPanel = new ExportableDataPanel<>("dataTable", createColumns(), provider);
        add(exportableDataPanel);
    }

    private List<IColumn<Project, String>> createColumns() {
        List<IColumn<Project, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Project>(Model.of("Started"), "dateCreated", "dateCreated"));
        columns.add(titleColumn());
        columns.add(new PropertyColumn<Project, String>(Model.of("Type"), "projectType.name", "projectType.name"));
        columns.add(new PropertyColumn<Project, String>(Model.of("Status"), "projectStatus", "projectStatus"));
        columns.add(supervisorColumn());
        columns.add(new PropertyColumn<Project, String>(Model.of("External organisation"), "externalOrganization.name", "externalOrganization.name"));
        return columns;
    }

    private AbstractColumn<Project, String> supervisorColumn() {
        return new PropertyColumn<Project, String>(Model.of("Head Superv."), "headSupervisor.fullName") {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> cellItem, String componentId, IModel<Project> rowModel) {
                cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "headSupervisor")));
            }
        };
    }

    private AbstractColumn<Project, String> titleColumn() {
        return new PropertyColumn<Project, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Project>> components, String s, final IModel<Project> model) {
                components.add(new AjaxLinkPanel<Project>(s, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Project> model) {
                        openCRUDPanel(model);
                    }
                });
            }
        };
    }

    private void addFeedbackPanel() {
        FencedFeedbackPanel feedback = new FencedFeedbackPanel("feedback", this);
        feedback.setOutputMarkupId(true);
        add(feedback);
    }

    private void openCRUDPanel(final IModel<Project> project) {
        PageParameters pp = new PageParameters();
        pp.set(PageParameterKeys.MAP.get(Project.class), project.getObject().getId());
        setResponsePage(AdminEditProjectPage.class, pp);
    }

    private class FilterForm extends Form<Void> {
        @Getter
        @Setter
        @SuppressWarnings("UnusedDeclaration")
        private String titleString;

        public FilterForm(String id) {
            super(id);
            filter = new ProjectService.Filter();
            filter.setFilterExternal(true);
            addDateFilter();
            addSupervisorFilter();
            addProjectTitleFilter();
            addAuthorFilter();
            addProjectTypeFilter();
            addProjectStatusFilter();
            addRedStateFilter();
        }

        private void addRedStateFilter() {
            TextField<Integer> redStateField = new TextField<>("redStateFilter", new PropertyModel<Integer>(filter, "redStateTime"), Integer.class);
            redStateField.add(new AjaxFormComponentUpdatingBehavior("onkeyup") {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    target.add(exportableDataPanel);
                }
            });
            add(redStateField);
        }

        private void addDateFilter() {
            add(new DeactivatableDatePickerPanel("dateFilter",
                    new PropertyModel<Date>(filter, "createdAfter"),
                    new PropertyModel<Date>(filter, "createdBefore")) {
                @Override
                protected void datesChanged(AjaxRequestTarget target) {
                    target.add(exportableDataPanel);
                }
            });
        }

        private void addSupervisorFilter() {
            add(new EmployeeAutoComplete("supervisorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setSupervisor(newSelection);
                    pTarget.add(exportableDataPanel);
                }
            });
        }

        private void addProjectTitleFilter() {
            TextField<String> titleField = new TextField<>("titleFilter", new PropertyModel<String>(this, "titleString"));
            titleField.add(new AjaxFormComponentUpdatingBehavior("onkeyup") {
                @Override
                protected void onUpdate(AjaxRequestTarget target) {
                    if (titleString != null && titleString.length() >= MIN_TITLE_LENGTH) {
                        filter.setTitleContains(titleString);
                    } else {
                        filter.setTitleContains(null);
                    }
                    target.add(exportableDataPanel);
                }
            });
            add(titleField);
        }

        private void addAuthorFilter() {
            add(new AuthorAutoComplete("authorFilter") {
                @Override
                protected void action(AjaxRequestTarget pTarget, User newSelection) {
                    filter.setAuthorUser(newSelection);
                    pTarget.add(exportableDataPanel);
                }
            });
        }

        private void addProjectTypeFilter() {
            AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                    projectTypeService.findByDegreeLevels(Arrays.asList(DegreeLevel.FIRST_CYCLE, DegreeLevel.SECOND_CYCLE)),
                    projectTypeService.findAllActive(),
                    new ChoiceRenderer<ProjectType>("name", "id")) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    if (!getModelObject().isEmpty()) {
                        filter.setProjectTypes(getModelObject());
                    }
                    target.add(exportableDataPanel);
                }
            };

            filter.setProjectTypes(projectType.getModelObject());
            add(projectType);
        }

        private void addProjectStatusFilter() {
            AjaxCheckBoxMultipleChoice<ProjectStatus> projectStatus = new AjaxCheckBoxMultipleChoice<ProjectStatus>("statusFilter",
                    new ArrayList<>(asList(ProjectStatus.ACTIVE)),
                    asList(ProjectStatus.values()),
                    new EnumChoiceRenderer<ProjectStatus>(this)) {
                @Override
                public void onUpdate(AjaxRequestTarget target) {
                    filter.setStatuses(new HashSet<>(getModelObject()));
                    target.add(exportableDataPanel);
                }
            };

            add(projectStatus);
            filter.setStatuses(new HashSet<>(projectStatus.getModelObject()));
        }
    }
}