package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.StringResourceModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.UploadFormPanel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.plagiarism.TurnitinSettingsService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;
import java.util.Date;

public class SeminarThesisPanel extends GenericPanel<FinalSeminar> {
    static final String DEADLINE_INFORMATION = "deadlineInformation";
    static final String DATE = "date";
    static final String UPLOAD = "upload";
    static final String NO_THESIS = "noThesis";
    static final String DOWNLOAD_THESIS = "download";
    static final String DELETE_THESIS = "delete";
    public static final String FEEDBACK = "feedback";

    @Inject
    private ProjectService projectService;
    @Inject
    private FinalSeminarService seminarService;
	@Inject
    private TurnitinSettingsService turnitinSettingsService;
    @Inject
    private NotificationController notificationController;

    private final IModel<Date> deadline;
    private final IModel<FinalSeminar> model;
    private final Project seminarProject;

    public SeminarThesisPanel(final String id, final IModel<FinalSeminar> model) {
        super(id, model);

        this.model = model;
        this.seminarProject = projectService.findOne(getSeminar().getProject().getId());

        add(new FencedFeedbackPanel(FEEDBACK, this));

        deadline = new LoadableDetachableModel<Date>() {
            @Override
            protected Date load() {
                return seminarService.thesisUploadDeadline(model.getObject());
            }
        };

        setOutputMarkupPlaceholderTag(true);

        addDeadlineInformation();
        addUploadForm();
        addDownloadLink();
        addDeleteLink();
        addNoThesisText();
    }

    protected void onDelete() {
    }

    private void deleteThesis(AjaxRequestTarget target) {
        FinalSeminar seminar = seminarService.deleteThesis(getSeminar());
        model.setObject(seminar);

        target.add(this);
        onDelete();
    }

    private FinalSeminar getSeminar() {
        return model.getObject();
    }

    private void addNoThesisText() {
        add(new Label(NO_THESIS, new StringResourceModel("noThesis", SeminarThesisPanel.this, null)) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(!seminarService.hasThesis(model.getObject()));
            }
        });
    }

    private void addDeleteLink() {
        AjaxLink<Void> deleteLink = new AjaxConfirmationLink<Void>(DELETE_THESIS, "Are you sure you want to delete the final seminar thesis file?") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                deleteThesis(target);
            }
            @Override
            protected void onConfigure() {
                super.onConfigure();
                boolean isSupervisor = seminarProject.isSupervisor(SciProSession.get().getUser());
                setVisibilityAllowed(seminarService.hasThesis(model.getObject()) && isSupervisor);
            }
        };
        add(deleteLink);
    }

    private void addDownloadLink() {
        add(new ViewAttachmentPanel(DOWNLOAD_THESIS, new PropertyModel<FileDescription>(getDefaultModel(), "document"), DateStyle.DATETIME));
    }

    private void addUploadForm() {
        add(new UploadFormPanel(UPLOAD, new NewSeminarDocumentUploadForm(getSeminar()) {
            @Override
            protected void onFinished(AjaxRequestTarget target) {
                FinalSeminar seminar = seminarService.findOne(getSeminar().getId());
                model.setObject(seminar);

                if (seminar.getDocument().getSize() > turnitinSettingsService.getSettings().getMaxFileSizeInBytes()) {
                    NotificationSource source = new NotificationSource();
                    notificationController.notifyProject(seminarProject, ProjectEvent.Event.FILE_SIZE_TOO_BIG, source);
                }

                SeminarThesisPanel.this.success(getString("upload.success"));

                setResponsePage(ProjectFinalSeminarPage.class);
            }
        }) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(seminarProject.isParticipant(SciProSession.get().getUser()) && !seminarService.hasThesis(model.getObject()));
                if (!model.getObject().getProject().isFinalSeminarRuleExempted()) {
                    setEnabled(!model.getObject().isDeleted() && !seminarService.hasDeadlinePassed(model.getObject()));
                }
            }
        });
    }

    private void addDeadlineInformation() {
        WebMarkupContainer deadlineInformation = new WebMarkupContainer(DEADLINE_INFORMATION) {
            @Override
            protected void onConfigure() {
                setVisible(getModelObject().getId() != null);
            }
        };
        deadlineInformation.add(new DateLabel(DATE, deadline, DateStyle.DATETIME));
        add(deadlineInformation);
    }
}
