package se.su.dsv.scipro.latestevents;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.ProjectFile;
import se.su.dsv.scipro.forum.ProjectForumService;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.pages.threaded.ProjectViewForumThreadPage;
import se.su.dsv.scipro.forum.pages.threaded.SupervisorViewForumThreadPage;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.repository.components.FileDownloadLink;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.*;

public class LatestEventsPanel extends Panel {

    public static final String LATEST_EVENT_CONTAINER = "latestEventsContainer";
    public static final String NO_EVENT = "noEvent";
    public static final String LATEST_EVENTS = "latestEvents";
    public static final String EVENT_TYPE = "eventType";
    public static final String EVENT_DATE = "eventDate";
    public static final String EVENT_CREATOR = "eventCreator";
    public static final String EVENT_LINK = "eventLink";
    public static final String EVENT_TITLE = "eventTitle";
    public static final int AMOUNT = 2;

    @Inject
    private FileDescriptionService fileDescriptionService;
    @Inject
    private ProjectForumService projectForumService;

    private Project project;
    private LoadableDetachableModel<List<LatestEvent>> latestEventModel;

    public LatestEventsPanel(String id, IModel<Project> model) {
        super(id, model);
        project = model.getObject();

        latestEventModel = createLatestEventModel();

        addLatestEventContainer();
        addNoEvent();
    }

    private void addLatestEventContainer() {
        WebMarkupContainer latestEventContainer = new WebMarkupContainer(LATEST_EVENT_CONTAINER) {
            @Override
            protected void onConfigure() {
                setVisible(latestEventModel != null);
            }
        };
        add(latestEventContainer);
        latestEventContainer.add(createLatestEventsListView());
    }

    private ListView<LatestEvent> createLatestEventsListView() {
        return new ListView<LatestEvent>(LATEST_EVENTS, latestEventModel) {

            @Override
            protected void populateItem(ListItem<LatestEvent> item) {
                LatestEvent latestEvent = item.getModelObject();
                item.add(new EnumLabel<>(EVENT_TYPE, new PropertyModel<LatestEvent.LatestEventType>(latestEvent, "type")));
                item.add(new DateLabel(EVENT_DATE, new PropertyModel<Date>(latestEvent, "date"), DateStyle.DATETIME));
                item.add(new Label(EVENT_CREATOR, new PropertyModel<>(latestEvent, "creator.fullName")));
                item.add(createEventLink(latestEvent));
            }
        };
    }

    private Component createEventLink(LatestEvent latestEvent) {
        if (latestEvent.getType().equals(LatestEvent.LatestEventType.FILES)) {
            return getFileLinkComponent(latestEvent);
        }
        return createForumLinkComponent(latestEvent);
    }

    private Component getFileLinkComponent(LatestEvent latestEvent) {
        Model<FileDescription> model = new Model<>((FileDescription) latestEvent.getDomainObject());

        FileDownloadLink<FileDescription> fileDownloadLink = new FileDownloadLink<>(EVENT_LINK, model);
        fileDownloadLink.add(new Label(EVENT_TITLE, new PropertyModel<String>(latestEvent, "title")));
        return fileDownloadLink;
    }

    private Component createForumLinkComponent(LatestEvent latestEvent) {
        ForumThread forumForumThread = ((ForumPost) latestEvent.getDomainObject()).getForumThread();
        final Model<ForumThread> model = new Model<>(forumForumThread);

        AjaxFallbackLink<ForumThread> threadLink = new AjaxFallbackLink<ForumThread>(EVENT_LINK, model) {
            @Override
            public void onClick(AjaxRequestTarget target) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Project.class), project.getId());
                pp.add(PageParameterKeys.MAP.get(ForumThread.class), getModelObject().getId());
                if (project.isParticipant(SciProSession.get().getUser())) {
                    setResponsePage(ProjectViewForumThreadPage.class, pp);
                } else {
                    setResponsePage(SupervisorViewForumThreadPage.class, pp);
                }
            }
        };
        threadLink.add(new Label(EVENT_TITLE, new PropertyModel<String>(latestEvent, "title")));
        return threadLink;
    }

    private LoadableDetachableModel<List<LatestEvent>> createLatestEventModel() {
        return new LoadableDetachableModel<List<LatestEvent>>() {
            @Override
            protected List<LatestEvent> load() {
                List<ProjectFile> projectFiles = findLatestUploadedFile();
                List<ForumPost> forumPosts = findLatestForumPosts();

                List<LatestEvent> latestEvents = new ArrayList<>();
                if (projectFiles != null) {
                    for (ProjectFile pf : projectFiles) {
                        latestEvents.add(new LatestEvent(pf.getDateCreated(), LatestEvent.LatestEventType.FILES, pf.getUploader(), pf.getName(), pf));
                    }
                }
                if (forumPosts != null) {
                    for (ForumPost fp : forumPosts) {
                        latestEvents.add(new LatestEvent(fp.getDateCreated(), LatestEvent.LatestEventType.FORUM, fp.getPostedBy(), fp.getSubject(), fp));
                    }
                }

                Collections.sort(latestEvents, new Comparator<LatestEvent>() {
                    @Override
                    public int compare(final LatestEvent o1, final LatestEvent o2) {
                        return o2.getDate().compareTo(o1.getDate());
                    }
                });
                return latestEvents.subList(0, latestEvents.size() >= AMOUNT ? AMOUNT : latestEvents.size());
            }
        };
    }

    private List<ForumPost> findLatestForumPosts() {
        return projectForumService.latestPost(project, AMOUNT);
    }

    private List<ProjectFile> findLatestUploadedFile() {
        return fileDescriptionService.latestUpload(project, AMOUNT);
    }

    private void addNoEvent() {
        add(new WebMarkupContainer(NO_EVENT) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(latestEventModel.getObject().isEmpty());
            }
        });
    }

}


