package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.IFeedbackMessageFilter;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.PasswordTextField;
import org.apache.wicket.markup.html.form.validation.EqualPasswordInputValidator;
import org.apache.wicket.markup.html.panel.ComponentFeedbackPanel;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.validation.IValidatable;
import org.apache.wicket.validation.IValidator;
import org.apache.wicket.validation.ValidationError;
import se.su.dsv.scipro.basepages.PublicPage;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.Calendar;
import java.util.Date;
import java.util.regex.Pattern;

public class SetPasswordPage extends PublicPage {

    @Inject
    private UserService userService;
    @Inject
    private HashService hashService;
    @Inject
    private PasswordFacade passwordFacade;

    private PasswordTextField password;

    public SetPasswordPage(final PageParameters pp) {
        String hashString = pp.get("hash").toString();

        if (hashString == null || hashString.isEmpty()) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        Hash hash = hashService.findByHash(hashString);

        if (hash==null){
            throw new RestartResponseException(getApplication().getHomePage());
        }

        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.HOUR, -1);
        Date sevenDaysAgo = cal.getTime();

        if (hash.getDateCreated().before(sevenDaysAgo)) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        add(new PasswordForm("form", new DetachableServiceModel<>(userService, hash.getUser())));
    }

    private class PasswordForm extends Form<User> {

        public PasswordForm(String id, IModel<User> model) {
            super(id, model);

            password = new PasswordTextField("password", Model.of("")) {
                @Override
                protected void onComponentTag(ComponentTag tag) {
                    super.onComponentTag(tag);
                    tag.put("placeholder", "Password");
                }
            };
            password.setRequired(true);
            password.add(new PasswordValidator());
            add(password);
            add(new ComponentFeedbackPanel("passwordFeedback", password));

            PasswordTextField repeatPassword = new PasswordTextField("repeatPassword", Model.of("")) {
                @Override
                protected void onComponentTag(ComponentTag tag) {
                    super.onComponentTag(tag);
                    tag.put("placeholder", "Repeat password");
                }
            };
            repeatPassword.setRequired(true);
            add(repeatPassword);
            add(new ComponentFeedbackPanel("repeatPasswordFeedback", repeatPassword));

            add(new EqualPasswordInputValidator(password, repeatPassword));

            add(new FeedbackPanel("feedback", new IFeedbackMessageFilter() {
                @Override
                public boolean accept(FeedbackMessage message) {
                    return !message.isRendered();
                }
            }));
        }

        @Override
        protected void onSubmit() {
            super.onSubmit();
            passwordFacade.setPassword(getModelObject(), password.getInput());
            success(getString("successfully.reset.password"));
        }

        private class PasswordValidator implements IValidator<String> {
            /**
             * disallow non a-zA-Z0-9
             * require at least 1 0-9
             * require at least 1 a-z
             * require at least 1 A-Z
             * require between 8 chars
             */
            private static final String PASSWORD_PATTERN =
//            "((?=^[a-zA-Z0-9]*$)(?=.*\\d)(?=.*[a-z])(?=.*[A-Z]).{8,})"; //using \\d shortcut
            "((?=^[a-zA-Z0-9]*$)(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z]).{8,})"; //avoiding \\d shortcut

            private final Pattern pattern;

            PasswordValidator() {
                pattern = Pattern.compile(PASSWORD_PATTERN);
            }

            @Override
            public void validate(IValidatable<String> validatable) {
                final String value = validatable.getValue();
                if (!pattern.matcher(value).matches()) {
                    validatable.error(new ValidationError(this));
                }
            }
        }
    }
}
