package se.su.dsv.scipro.match;

import org.apache.wicket.WicketRuntimeException;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.AjaxIconLinkPanel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.GenericDataProvider;
import se.su.dsv.scipro.datatables.ClickableIconColumn;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.Program;

import javax.inject.Inject;
import javax.validation.ConstraintViolationException;
import java.util.ArrayList;
import java.util.List;

@Authorization(authorizedRoles = {Roles.SYSADMIN})
public class AdminProgramPage extends AbstractAdminMatchPage {

    private static final Logger LOGGER = LoggerFactory.getLogger(AdminProgramPage.class);
    protected final ExportableDataPanel<Program, String> dp;
    private final FeedbackPanel feedbackPanel;
    private ModalWindowPlus modalWindowPlus;

    @Inject
    private ProgramService programService;

    public AdminProgramPage() {
        feedbackPanel = new FeedbackPanel("feedback");
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        SortableDataProvider<Program, String> provider = new GenericDataProvider<>(programService);

        dp = new ExportableDataPanel<>("dp", createColumns(), provider);
        dp.setOutputMarkupId(true);
        add(dp);

        addDialog();
        addCreateNewLink();
	}

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addCreateNewLink() {
        AjaxLink<Void> createNewLink = new AjaxLink<Void>("createNewLink") {
            @Override
            public void onClick(AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Create new program");
                Program program = new Program();
                modalWindowPlus.setContent(new EditProgramPanel(modalWindowPlus.getContentId(), new DetachableServiceModel<>(programService, program)) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dp);
                    }
                });
                modalWindowPlus.show(target);
            }
        };
        add(createNewLink);
    }

    private List<IColumn<Program, String>> createColumns() {
        List<IColumn<Program, String>> columns = new ArrayList<>();
        columns.add(new PropertyColumn<Program, String>(Model.of("Name"), "name", "name"));
        columns.add(new PropertyColumn<Program, String>(Model.of("Name in English"), "nameEn", "nameEn"));
        columns.add(new AbstractColumn<Program, String>(Model.of("Has external system identifier")) {
            @Override
            public void populateItem(Item<ICellPopulator<Program>> cellItem, String componentId, IModel<Program> rowModel) {
                String hasExternalIdentifier = (rowModel.getObject().getExternalId() == null) ? "No" : "Yes";
                cellItem.add(new Label(componentId, Model.of(hasExternalIdentifier)));
            }
        });
        columns.add(new ClickableIconColumn<Program>(Model.of("Edit program"), null, ClickableIconColumn.EDIT) {
            @Override
            protected void onClick(IModel<Program> clicked, AjaxRequestTarget target) {
                modalWindowPlus.setTitle("Edit program: " + clicked.getObject().getName());
                modalWindowPlus.setContent(new EditProgramPanel(modalWindowPlus.getContentId(), clicked) {
                    @Override
                    public void onSave(AjaxRequestTarget target) {
                        target.add(dp);
                    }
                });
                modalWindowPlus.show(target);
            }
        });
        columns.add(new AbstractColumn<Program, String>(Model.of("Delete")) {
            @Override
            public void populateItem(Item<ICellPopulator<Program>> cellItem, final String componentId, final IModel<Program> rowModel) {
                AjaxIconLinkPanel<Program> delete = new AjaxIconLinkPanel<Program>(componentId, rowModel, Model.of(AjaxIconLinkPanel.DELETE), "Are you sure?") {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Program> model) {
                        try {
                            programService.delete(model.getObject());
                            info("Deleted program");
                        } catch (ConstraintViolationException cve) {
                            String message = "Could not delete program because it is being used";
                            error(message);
                            LOGGER.info(message, cve);
                        } catch (WicketRuntimeException wre) {
                            String message = "Could not delete program";
                            error(message);
                            LOGGER.info(message, wre);
                        }
                        target.add(feedbackPanel);
                        target.add(dp);
                    }
                };
                cellItem.add(delete);
            }
        });
        return columns;
    }
}
