package se.su.dsv.scipro.match;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.dataproviders.PageAdapter;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.*;

public class ProjectAllSupervisorIdeasPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    @Inject
    private IdeaFacade ideaFacade;
    @Inject
    private KeywordService keywordService;
    @Inject
    private ResearchAreaService researchAreaService;

    private ModalWindowPlus modalWindowPlus;
    private IdeaService.Filter params;
    private ExportableDataPanel<Idea, String> dataTable;
    private ApplicationPeriod applicationPeriod;

    public ProjectAllSupervisorIdeasPanel(String id, final ApplicationPeriod applicationPeriod) {
        super(id);
        this.applicationPeriod = applicationPeriod;
        addDialog();
        addFiltering();
        addDataTable();
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addDataTable() {
        SortableDataProvider<Idea, String> provider = new SortableDataProvider<Idea, String>() {
            @Override
            public Iterator<? extends Idea> iterator(long first, long count) {
                return  ideaFacade.findIdeasPossibleToMatch(applicationPeriod, params, new PageAdapter(first, count, getSort())).iterator();
            }

            @Override
            public long size() {
                return ideaFacade.countIdeasPossibleToMatch(applicationPeriod, params);
            }

            @Override
            public IModel<Idea> model(Idea object) {
                return new DetachableServiceModel<>(ideaService, object);
            }
        };
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>("dataTable", createColumns(), provider);
        add(dataTable);
    }

    private List<IColumn<Idea, String>> createColumns() {
        List<IColumn<Idea, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Idea>(Model.of("Created"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Idea, String>(Model.of("Type"), "projectType.name", "projectType.name"));

        columns.add(new AbstractColumn<Idea, String>(Model.of("Languages")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new DisplayMultiplesPanel<Language>(componentId, new ListAdapterModel<>(new PropertyModel<Iterable<Language>>(rowModel, "languages"))) {
                    @Override
                    public Component getComponent(String componentId, IModel<Language> t) {
                        return new Label(componentId, new PropertyModel<Language>(t, "name"));
                    }
                });
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Title"), "title") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> components, String s, final IModel<Idea> model) {
                components.add(new AjaxLinkPanel<Idea>(s, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                        modalWindowPlus.setContent(new AcceptIdeaDialogPanel(modalWindowPlus.getContentId(), model, applicationPeriod));
                        modalWindowPlus.setTitle(model.getObject().getTitle());
                        modalWindowPlus.show(target);
                    }
                });
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Supervisor"), "match.supervisor.lastName") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> cellItem, String componentId, IModel<Idea> rowModel) {
                cellItem.add(new UserLinkPanel(componentId, new PropertyModel<User>(rowModel, "creator")));
            }
        });
        columns.add(new PropertyColumn<Idea, String>(Model.of("Research Area"), "researchArea.title", "researchArea.title"));
        columns.add(new KeywordsColumn<String>(Model.of("Keywords")));

        return columns;
    }

    private void addFiltering() {
        params = new IdeaService.Filter();

        params.setTypes(Collections.singletonList(Type.SUPERVISOR));
        params.setStatuses(Collections.singletonList(Idea.Status.UNMATCHED));

        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                getProjectTypeModel(), getProjectTypes(), new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataTable);
            }
        };
        add(projectType);

        final KeywordFilterPanel keywordFilter = new KeywordFilterPanel("keywordFilter", getKeywords()) {
            @Override
            public void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> keywords) {
                params.setKeywords(keywords);
                target.add(dataTable);
            }
        };
        keywordFilter.setOutputMarkupPlaceholderTag(true);
        keywordFilter.setVisible(false);
        add(keywordFilter);

        researchAreaFilter(keywordFilter);
    }

    private void researchAreaFilter(final KeywordFilterPanel keywordFilter) {
        add(new AjaxDropDownChoice<ResearchArea>("areaFilter", new Model<ResearchArea>(),
                new ListModel<>(researchAreaService.findNotDeleted()),
                new ChoiceRenderer<ResearchArea>("title", "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, ResearchArea objectSelected) {
                params.setResearchArea(getModelObject());
                target.add(dataTable);
                keywordFilter.setVisible(objectSelected != null && !keywordService.findByResearchArea(params.getResearchArea()).isEmpty());
                target.add(keywordFilter);
            }
        });
    }

    private PropertyModel<List<ProjectType>> getProjectTypeModel() {
        params.setLevels(getProjectTypes());
        return new PropertyModel<>(params, "levels");
    }

    private List<ProjectType> getProjectTypes() {
        return new ArrayList<>(applicationPeriod.getProjectTypes());
    }

    private IModel<Set<Keyword>> getKeywords() {
        return new LoadableDetachableModel<Set<Keyword>>() {
            @Override
            protected Set<Keyword> load() {
                if (params.getResearchArea() != null) {
                    return new HashSet<>(keywordService.findByResearchArea(params.getResearchArea()));
                }
                return new HashSet<>();
            }
        };
    }
}
