package se.su.dsv.scipro.match;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.match.Idea.Type;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.*;

public class SupervisorAllSupervisorIdeasPanel extends Panel {

    @Inject
    private IdeaService ideaService;
    @Inject
    private ProjectTypeService projectTypeService;
    @Inject
    private KeywordService keywordService;
    @Inject
    private ResearchAreaService researchAreaService;

    private final IdeaService.Filter params = new IdeaService.Filter();
    private ExportableDataPanel<Idea, String> dataTable;
    private ModalWindowPlus modalWindowPlus;

    public SupervisorAllSupervisorIdeasPanel(String id) {
        super(id);
        addDialog();
        addFiltering();
        addDataTable();
    }

    private void addDialog() {
        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    private void addDataTable() {
        FilteredDataProvider<Idea, IdeaService.Filter> provider = new FilteredDataProvider<>(ideaService, Model.of(params));
        provider.setSort("dateCreated", SortOrder.DESCENDING);
        dataTable = new ExportableDataPanel<>("dataTable", columns(), provider);
        add(dataTable);
    }

    private List<IColumn<Idea, String>> columns() {
        List<IColumn<Idea, String>> columns = new ArrayList<>();
        columns.add(new DateColumn<Idea>(Model.of("Created"), "dateCreated", "dateCreated"));
        columns.add(new PropertyColumn<Idea, String>(Model.of("Type"), "projectType.name", "projectType.name"));

        columns.add(new AbstractColumn<Idea, String>(Model.of("Title")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, final IModel<Idea> model) {
                item.add(new AjaxLinkPanel<Idea>(id, model, new PropertyModel(model, "title")) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<Idea> model) {
                        openDialog(model, target);
                    }
                });
            }
        });

        columns.add(new PropertyColumn<Idea, String>(Model.of("Research Area"), "researchArea.title", "researchArea.title"));
        columns.add(new KeywordsColumn<String>(Model.of("Keywords")));

        columns.add(new AbstractColumn<Idea, String>(Model.of("Supervisor")) {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> model) {
                item.add(new UserLinkPanel(id, new PropertyModel<User>(model, "match.supervisor")));
            }
        });

        columns.add(new AbstractColumn<Idea, String>(Model.of("Status"), "match.status") {
            @Override
            public void populateItem(Item<ICellPopulator<Idea>> item, String id, IModel<Idea> model) {
                item.add(new Label(id, ideaService.getStatusString(model.getObject())));
            }
        });
        return columns;
    }
    
    private void openDialog(IModel<Idea> model, AjaxRequestTarget target) {
        modalWindowPlus.setContent(new IdeaDetailsPanel(modalWindowPlus.getContentId(), model, true));
        modalWindowPlus.setTitle(model.getObject().getTitle());
        modalWindowPlus.show(target);
    }


    private void addFiltering() {
        params.setStatuses(Arrays.asList(Idea.Status.COMPLETED, Idea.Status.MATCHED, Idea.Status.UNMATCHED));
        params.setTypes(Collections.singletonList(Type.SUPERVISOR));

        AjaxCheckBoxMultipleChoice<ProjectType> projectType = new AjaxCheckBoxMultipleChoice<ProjectType>("projectTypeFilter",
                getProjectTypes(), getProjectTypes(), new ChoiceRenderer<ProjectType>("name", "id")) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                if (!getModelObject().isEmpty()) {
                    params.setLevels(getModelObject());
                }
                target.add(dataTable);
            }
        };
        params.setLevels(projectType.getModelObject());
        add(projectType);

        final KeywordFilterPanel keywordFilter = new KeywordFilterPanel("keywordFilter", getKeywords()) {
            @Override
            public void onKeywordsSelected(AjaxRequestTarget target, Set<Keyword> keywords) {
                params.setKeywords(keywords);
                target.add(dataTable);
            }
        };
        keywordFilter.setOutputMarkupPlaceholderTag(true);
        keywordFilter.setVisible(false);
        add(keywordFilter);

        researchAreaFilter(keywordFilter);
    }

    private List<ProjectType> getProjectTypes() {
        return projectTypeService.findWithModule(ProjectModule.MATCH);
    }

    private void researchAreaFilter(final KeywordFilterPanel keywordFilter) {
        add(new AjaxDropDownChoice<ResearchArea>("areaFilter", new Model<ResearchArea>(),
                new ListModel<>(researchAreaService.findNotDeleted()),
                new ChoiceRenderer<ResearchArea>("title", "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, ResearchArea objectSelected) {
                params.setResearchArea(getModelObject());
                target.add(dataTable);
                keywordFilter.setVisible(getModelObject() != null && !keywordService.findByResearchArea(getModelObject()).isEmpty());
                target.add(keywordFilter);
            }
        });
    }

    private IModel<Set<Keyword>> getKeywords() {
        return new LoadableDetachableModel<Set<Keyword>>() {
            @Override
            protected Set<Keyword> load() {
                if (params.getResearchArea() != null) {
                    return new HashSet<>(keywordService.findByResearchArea(params.getResearchArea()));
                }
                return new HashSet<>();
            }
        };
    }
}
