package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.service.MileStoneService;
import se.su.dsv.scipro.milestones.service.MilestoneActivityTemplateService;
import se.su.dsv.scipro.milestones.service.MilestonePhaseTemplateService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

public class MileStoneOverviewPanel extends Panel {

    public static final String MILESTONE_DESCRIPTION = "milestoneDescription";
    @Inject
    private MilestonePhaseTemplateService phaseService;
    @Inject
    private MilestoneActivityTemplateService activityService;
    @Inject
    private MileStoneService mileStoneService;

    private final WebMarkupContainer container;
    private final IModel<Project> projectModel;


    public MileStoneOverviewPanel(String id, IModel<Project> model) {
        super(id, model);

        this.projectModel = model;
        this.container = new WebMarkupContainer(TABLE_CONTAINER);
        add(container.setOutputMarkupId(true));

        addTableHeaderWithStudentNames();
        addMileStonePhaseList();
        addCounter();
    }

    private void addTableHeaderWithStudentNames() {
        container.add(new ListView<User>(STUDENT_HEADER, new ArrayList<>(projectModel.getObject().getProjectParticipants())) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new Label(STUDENT_NAME, item.getModelObject().getFullName()));
            }
        });
    }

    private void addMileStonePhaseList() {
        container.add(new ListView<MilestonePhaseTemplate>(PHASE_LIST, getPhaseListModel()) {
            @Override
            protected void populateItem(ListItem<MilestonePhaseTemplate> item) {
                ListView<MilestoneActivityTemplate> mileStoneList = addMileStoneList(item.getModel());
                item.add(mileStoneList);
                item.add(new Label(TITLE, new PropertyModel<String>(item.getModel(), TITLE)));
                item.setVisible(mileStoneList.getViewSize() > 0);
            }
        });
    }

    private void addCounter() {
        container.add(new MileStoneCounterPanel(PROJECT_COUNTER, countCompleted(null), MilestoneActivityTemplate.Type.PROJECT, projectModel));
        container.add(new ListView<User>(STUDENT_COUNTER_LIST, new ArrayList<>(projectModel.getObject().getProjectParticipants())) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new MileStoneCounterPanel(STUDENT_COUNTER, countCompleted(item.getModel()), MilestoneActivityTemplate.Type.STUDENT, projectModel));
            }
        });
    }

    private ListView<MilestoneActivityTemplate> addMileStoneList(final IModel<MilestonePhaseTemplate> phaseModel) {
        return new ListView<MilestoneActivityTemplate>(MILESTONE_LIST, getActivityListModel(phaseModel)) {

            @Override
            protected void populateItem(ListItem<MilestoneActivityTemplate> item) {
                item.add(new Label(MILESTONE_TITLE, new PropertyModel<String>(item.getModel(), TITLE)));
                item.add(new InfoPanel(MILESTONE_DESCRIPTION, new PropertyModel<String>(item.getModel(), "description")));
                addSupervisorCheckBox(item);
                addAuthorStatusCheck(item);
                addStudentColumns(item);
            }
        };
    }

    private void addSupervisorCheckBox(ListItem<MilestoneActivityTemplate> item) {
        final IModel<MilestoneActivityTemplate> activityModel = item.getModel();
        item.add(new MileStoneCheckBoxPanel(PROJECT_CHECKBOX, projectModel, item.getModel(), MilestoneActivityTemplate.Type.PROJECT) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(isSupervisorView() || activityModel.getObject().isEditableByStudents());
            }
        });
    }

    private void addAuthorStatusCheck(ListItem<MilestoneActivityTemplate> item) {
        final IModel<MilestoneActivityTemplate> activityModel = item.getModel();
        item.add(new MileStoneStatusPanel(PROJECT_STATUS, projectModel, item.getModel(), MilestoneActivityTemplate.Type.PROJECT) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(!isSupervisorView() && !activityModel.getObject().isEditableByStudents());
            }
        });
    }

    private void addStudentColumns(ListItem<MilestoneActivityTemplate> item) {
        final IModel<MilestoneActivityTemplate> activityModel = item.getModel();
        item.add(new ListView<User>(STUDENTS, new ArrayList<>(projectModel.getObject().getProjectParticipants())) {
            @Override
            protected void populateItem(ListItem<User> item) {
                item.add(new MileStoneCheckBoxPanel(STUDENT_CHECKBOX, projectModel, activityModel, item.getModel(), MilestoneActivityTemplate.Type.STUDENT) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(isSupervisorView() || activityModel.getObject().isEditableByStudents());
                    }
                });
                item.add(new MileStoneStatusPanel(STUDENT_STATUS, projectModel, activityModel, item.getModel(), MilestoneActivityTemplate.Type.STUDENT) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(!isSupervisorView() && !activityModel.getObject().isEditableByStudents());
                    }
                });
            }
        });
    }

    private LoadableDetachableModel<List<MilestonePhaseTemplate>> getPhaseListModel() {
        return new LoadableDetachableModel<List<MilestonePhaseTemplate>>() {
            @Override
            protected List<MilestonePhaseTemplate> load() {
                return phaseService.findAllActive(projectModel.getObject().getDateCreated());
            }
        };
    }

    private LoadableDetachableModel<List<MilestoneActivityTemplate>> getActivityListModel(final IModel<MilestonePhaseTemplate> phaseModel) {
        return new LoadableDetachableModel<List<MilestoneActivityTemplate>>() {
            @Override
            protected List<MilestoneActivityTemplate> load() {
                return activityService.findActiveByPhase(phaseModel.getObject(), projectModel.getObject());
            }
        };
    }

    private IModel<Long> countCompleted(final IModel<User> studentModel) {
        return new LoadableDetachableModel<Long>() {
            @Override
            protected Long load() {
                return studentModel == null ?
                        mileStoneService.countProjectMilestones(projectModel.getObject(), true)
                        : mileStoneService.countMileStones(projectModel.getObject(), studentModel.getObject(), true);
            }
        };
    }

    protected boolean isSupervisorView() {
        return false;
    }

    static final String TABLE_CONTAINER = "tableContainer";
    static final String STUDENT_HEADER = "studentHeader";
    static final String STUDENT_NAME = "studentName";
    static final String PHASE_LIST = "phaseList";
    static final String MILESTONE_LIST = "milestoneList";
    static final String TITLE = "title";
    static final String MILESTONE_TITLE = "milestoneTitle";
    static final String PROJECT_CHECKBOX = "projectCheckBox";
    static final String PROJECT_STATUS = "projectStatus";
    static final String STUDENTS = "students";
    static final String STUDENT_CHECKBOX = "studentCheckBox";
    static final String STUDENT_STATUS = "studentStatus";
    static final String PROJECT_COUNTER = "projectCounter";
    static final String STUDENT_COUNTER_LIST = "studentCounterList";
    static final String STUDENT_COUNTER = "studentCounter";
}
