package se.su.dsv.scipro.peer;

import org.apache.wicket.Page;
import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class MyPeerPanel extends GenericPanel<Project> {

    public static final String REQUEST_LIST = "requestList";
    public static final String REVIEW_LIST = "reviewList";
    public static final String REQUEST_DATE = "requestDate";
    public static final String REVIEW_DATE = "reviewDate";
    public static final String PROJECT_TITLE = "projectTitle";
    public static final String REQUEST_STATUS = "requestStatus";
    public static final String REVIEW_STATUS = "reviewStatus";
    public static final String REVIEW_LINK = "reviewLink";
    public static final String REQUEST_LINK = "requestLink";
    public static final String CANCEL = "cancel";
    public static final String NO_REQUESTS = "noRequests";
    public static final String NO_REVIEWS = "noReviews";

    @Inject
    private PeerRequestService peerRequestService;
    @Inject
    private PeerReviewService peerReviewService;

    public MyPeerPanel(String id, IModel<Project> model) {
        super(id, model);
        addRequestsListView();
        addReviewsListView();
    }

    private void addReviewsListView() {
        add(new ListView<PeerReview>(REVIEW_LIST, getPeerReviews()) {
            @Override
            protected void populateItem(ListItem<PeerReview> item) {
                item.add(new DateLabel(REVIEW_DATE, new PropertyModel<Date>(item.getModel(), "dateCreated")));
                BookmarkablePageLink<Object> link = new BookmarkablePageLink<>(REVIEW_LINK, getReviewPage(item.getModelObject()),
                        getReviewParameters(item.getModelObject()));
                link.add(new Label(PROJECT_TITLE, new PropertyModel<>(item.getModel(), "peerRequest.project.title")));
                item.add(link);
                item.add(new EnumLabel<>(REVIEW_STATUS, new PropertyModel<PeerReview.ReviewStatus>(item.getModel(), "status")));
            }
        });
        add(new Label(NO_REVIEWS, new ResourceModel("noReviews")) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getPeerReviews().getObject().isEmpty());
            }
        });
    }

    private Class<? extends Page> getReviewPage(PeerReview modelObject) {
        return modelObject.isCompleted() ? ViewPeerReviewPage.class : PeerPortalPage.class;
    }

    private void addRequestsListView() {
        add(new ListView<PeerRequest>(REQUEST_LIST, getPeerRequests()) {
            @Override
            protected void populateItem(ListItem<PeerRequest> item) {
                item.add(new DateLabel(REQUEST_DATE, new PropertyModel<Date>(item.getModel(), "dateCreated")));
                item.add(new RequestLink(REQUEST_LINK, item.getModel()));
                item.add(new CancelLink(CANCEL, item.getModel()));
            }
        });
        add(new Label(NO_REQUESTS, new ResourceModel("noRequests")) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getPeerRequests().getObject().isEmpty());
            }
        });
    }

    private PageParameters getReviewParameters(PeerReview review) {
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(PeerReview.class), review != null ? review.getId() : "");
        return pageParameters;
    }

    private LoadableDetachableModel<List<? extends PeerReview>> getPeerReviews() {
        return new LoadableDetachableModel<List<? extends PeerReview>>() {
            @Override
            protected List<? extends PeerReview> load() {
                return peerReviewService.findByReviewerAndProject(SciProSession.get().getUser(), getModelObject());
            }
        };
    }

    private LoadableDetachableModel<List<? extends PeerRequest>> getPeerRequests() {
        return new LoadableDetachableModel<List<? extends PeerRequest>>() {
            @Override
            protected List<? extends PeerRequest> load() {
                return peerRequestService.findByRequesterAndProject(SciProSession.get().getUser(), getModelObject());
            }
        };
    }

    private class RequestLink extends BookmarkablePageLink<Void> {

        private IModel<PeerRequest> model;

        public RequestLink(String id, IModel<PeerRequest> model) {
            super(id, ViewPeerReviewPage.class, getReviewParameters(model.getObject().getPeerReview()));
            this.model = model;
            add(new EnumLabel<>(REQUEST_STATUS, new PropertyModel<RequestStatus>(model, "status")));
        }

        @Override
        protected void onConfigure() {
            setEnabled(model.getObject().getStatus().equals(RequestStatus.FINISHED));
        }
    }

    private class CancelLink extends Link<PeerRequest> {

        public CancelLink(String id, IModel<PeerRequest> model) {
            super(id, model);
            add(new JavascriptEventConfirmation("onclick", new ResourceModel("confirmDelete")));
        }

        @Override
        public void onClick() {
            peerRequestService.deleteRequest(getModelObject());
            getSession().success(getString("deleted"));
            setResponsePage(PeerPortalPage.class);
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(getModelObject().getStatus().equals(RequestStatus.WAITING));
        }
    }
}
