package se.su.dsv.scipro.peer;


import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.ajax.markup.html.navigation.paging.AjaxPagingNavigator;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PageableListView;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.checklist.ChecklistTemplateService;
import se.su.dsv.scipro.checklists.ViewChecklistTemplatePanelPeer;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.LanguageService;
import se.su.dsv.scipro.util.AjaxConfirmationSubmitLink;

import javax.inject.Inject;
import java.util.List;

public class PeerRequestSubmissionPanel extends GenericPanel<Project> {

    private final FeedbackPanel feedbackPanel;
    @Inject
    private ChecklistTemplateService checklistTemplateService;
    @Inject
    private PeerPortal peerPortal;
    @Inject
    private PeerRequestService peerRequestService;
    @Inject
    private LanguageService languageService;

    private ModalWindowPlus modalWindowPlus;

    public PeerRequestSubmissionPanel(final String id, final IModel<Project> projectModel) {
        super(id, projectModel);

        feedbackPanel = new FeedbackPanel("feedbackPanel");
        feedbackPanel.setOutputMarkupId(true);
        add(feedbackPanel);

        final PeerRequestForm form = new PeerRequestForm("requestForm");
        add(form);

        modalWindowPlus = new ModalWindowPlus("dialog");
        add(modalWindowPlus);
    }

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(getModelObject().getProjectStatus().equals(ProjectStatus.ACTIVE));
    }

    private final class PeerRequestForm extends Form<PeerRequest> {
        private FileUploadField fileUpload;

        private PeerRequestForm(final String id) {
            super(id, new DetachableServiceModel<>(peerRequestService, new PeerRequest()));
            setOutputMarkupPlaceholderTag(true);
        }

        @Override
        public void onInitialize() {
            super.onInitialize();

            final WebMarkupContainer listContainer = new WebMarkupContainer("listContainer");
            listContainer.setOutputMarkupId(true);

            IModel<List<ChecklistTemplate>> checklistTemplateModel = new LoadableDetachableModel<List<ChecklistTemplate>>() {
                @Override
                protected List<ChecklistTemplate> load() {
                    return checklistTemplateService.findPeerRequestChecklists(PeerRequestSubmissionPanel.this.getModelObject());
                }
            };

            RadioGroup<ChecklistTemplate> templateRadios = new RadioGroup<>("radioGroup", new PropertyModel<ChecklistTemplate>(getModel(), "checklistTemplate"));
            listContainer.add(templateRadios);

            final PageableListView<ChecklistTemplate> templateListView = new PageableListView<ChecklistTemplate>("templateList", checklistTemplateModel, 5) {
                @Override
                protected void populateItem(final ListItem<ChecklistTemplate> item) {
                    item.add(new Radio<>("radio", item.getModel())
                            .setLabel(new PropertyModel<String>(item.getModel(), "name")));

                    AjaxLink<ChecklistTemplate> templateLink = new AjaxLink<ChecklistTemplate>("templateLink", item.getModel()) {
                        @Override
                        public void onClick(AjaxRequestTarget target) {
                            modalWindowPlus.setContent(new ViewChecklistTemplatePanelPeer(modalWindowPlus.getContentId(), getModel()));
                            modalWindowPlus.setTitle("Preview of chosen checklist template");
                            modalWindowPlus.show(target);
                        }
                    };
                    item.add(templateLink);

                }
            };

            listContainer.add(new Label("noTemplatesFoundMsg", "No templates found") {
                @Override
                protected void onConfigure() {
                    setVisible(templateListView.getList().isEmpty());
                }
            });

            Radio<ChecklistTemplate> noTemplate = new Radio<ChecklistTemplate>("noTemp") {
                @Override
                protected void onConfigure() {
                    setVisible(!templateListView.getList().isEmpty());
                }
            };
            templateRadios.add(noTemplate);
            templateRadios.add(templateListView);

            final AjaxPagingNavigator pager = new AjaxPagingNavigator("pager", templateListView) {
                @Override
                protected void onAjaxEvent(AjaxRequestTarget target) {
                    target.add(listContainer);
                }

                @Override
                public boolean isVisible() {
                    return getPageable().getPageCount() > 1;
                }
            };

            listContainer.add(pager);
            add(listContainer);

            add(new TextArea<>("comment", new PropertyModel<String>(getModel(), "comment")));

            fileUpload = new FileUploadField("fileUpload");
            fileUpload.setRequired(true);
            add(fileUpload);

            java.util.List<Language> languages = languageService.findAll();
            add(new DropDownChoice<Language>("language", new PropertyModel<Language>(getModel(),"language"),languages, new ChoiceRenderer<>("name", "id")).setRequired(true));

            add(new AjaxConfirmationSubmitLink("submit", "Please confirm your request for peer review. Once it has been accepted by someone, you cannot withdraw it.") {
                @Override
                protected void onAfterSubmit(AjaxRequestTarget target, Form<?> form) {
                    setResponsePage(getPage());
                }

                @Override
                protected void onError(AjaxRequestTarget target, Form<?> form) {
                    target.add(feedbackPanel);
                }
            });
        }

        @Override
        protected void onSubmit() {
            PeerRequest request = getModelObject();
            request.setProject(PeerRequestSubmissionPanel.this.getModelObject());
            request.setRequester(SciProSession.get().getUser());
            peerPortal.storePeerRequest(WicketFileUpload.from(fileUpload.getFileUpload()), request);
            setModelObject(new PeerRequest());
        }
    }
}
