package se.su.dsv.scipro.settings.pages;

import com.google.common.base.Function;
import lombok.SneakyThrows;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.head.CssHeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.*;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.request.resource.PackageResourceReference;
import org.apache.wicket.validation.validator.RangeValidator;
import org.apache.wicket.validation.validator.StringValidator;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.loginlogout.pages.SetPasswordPage;
import se.su.dsv.scipro.profile.UserProfileImage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.HashService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.imageio.IIOException;
import javax.imageio.ImageIO;
import javax.inject.Inject;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;

public class BasicProfilePage extends AbstractSettingsPage {
    static final String FEEDBACK = "feedback";
    static final String NAME = "name";
    static final String EMAIL = "email";
    static final String FORM = "form";
    static final String SKYPE = "skypeId";
    static final String PHONE = "phoneNumber";
    static final String OTHER = "otherInfo";
    static final int MAX_LENGTH = 200;

    @Inject
    private UserProfileService profileService;
    @Inject
    private HashService hashService;
    @Inject
    private FileDescriptionService fileDescriptionService;

    public BasicProfilePage() {
        add(new FeedbackPanel(FEEDBACK));

        IModel<UserProfile> profile = new LoadableDetachableModel<UserProfile>() {
            @Override
            protected UserProfile load() {
                return profileService.findByUser(getUser());
            }
        };
        add(new UserProfileImage("image", new LoadableDetachableModel<User>() {
            @Override
            public User load() {
                return getUser();
            }
        }, UserProfileImage.Size.LARGE));
        add(new ProfileForm(FORM, new CompoundPropertyModel<>(profile)));
        add(new ProfilePictureForm("profilePictureForm", profile));
    }

    private class ProfileForm extends Form<UserProfile> {

        public ProfileForm(String id, final IModel<UserProfile> model) {
            super(id, model);
            setMultiPart(true);
            add(new Label(NAME, getUser().getFirstName()));
            add(new Label(EMAIL, getUser().getEmailAddress()));
            add(new TextField<String>(SKYPE));
            add(new TextField<String>(PHONE));
            add(new TextArea<String>(OTHER).add(StringValidator.maximumLength(MAX_LENGTH)));
            final WebMarkupContainer pictureFileContainer = new WebMarkupContainer("pictureFileContainer") {
                @Override
                protected void onConfigure() {
                    setVisible(model.getObject().getPicture() == UserProfile.Picture.FILE);
                }
            };
            pictureFileContainer.setOutputMarkupPlaceholderTag(true);
            add(new AjaxDropDownChoice<UserProfile.Picture>("picture", Arrays.asList(UserProfile.Picture.values()), new EnumChoiceRenderer<UserProfile.Picture>(this)) {
                @Override
                public void onNewSelection(final AjaxRequestTarget target, final UserProfile.Picture objectSelected) {
                    target.add(pictureFileContainer);
                }
            });
            pictureFileContainer.add(new AjaxLink<UserProfile>("delete", model) {
                @Override
                public void onClick(final AjaxRequestTarget target) {
                    final UserProfile profile = getModelObject();
                    final FileDescription profilePicture = profile.getProfilePicture();
                    profile.setProfilePicture(null);
                    profileService.save(profile);
                    fileDescriptionService.delete(profilePicture);
                    setResponsePage(BasicProfilePage.class);
                }

                @Override
                protected void onConfigure() {
                    setVisible(getModelObject().getProfilePicture() != null);
                }
            });
            pictureFileContainer.add(new Label("currentPictureFile", new PropertyModel<String>(model, "profilePicture.name")));
            add(pictureFileContainer);

            add(new AjaxConfirmationLink<Void>("changePassword", "Are you sure?") {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    final User user = SciProSession.get().getUser();
                    final Hash hash = hashService.createForUser(user);
                    final PageParameters pp = new PageParameters();
                    pp.add("hash", hash.getHash());
                    setResponsePage(SetPasswordPage.class, pp);
                }

                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(SciProSession.get().getUser().getIdentifier()==null);
                }
            });
        }

        @Override
        protected void onSubmit() {
            profileService.save(getModelObject());
            info(getString("profile.saved"));
        }

    }

    @Override
    public void renderHead(final IHeaderResponse response) {
        super.renderHead(response);
        response.render(CssHeaderItem.forReference(new PackageResourceReference(BasicProfilePage.class, "imgareaselect-default.css")));
        response.render(JavaScriptHeaderItem.forReference(new PackageResourceReference(BasicProfilePage.class, "jquery.imgareaselect.pack.js")));
    }

    private class ProfilePictureForm extends StatelessForm<UserProfile> {
        public static final int SIZE = 128;
        public static final int WIDTH = SIZE;
        public static final int HEIGHT = SIZE;

        private final Model<Integer> xModel = Model.of(0);
        private final Model<Integer> yModel = Model.of(0);
        private final Model<Integer> sizeModel = Model.of(0);
        private final FileUploadField picture = new FileUploadField("picture");

        @Inject
        private FileDescriptionService fileDescriptionService;

        public ProfilePictureForm(final String id, final IModel<UserProfile> model) {
            super(id, model);

            add(new HiddenField<>("x", xModel, Integer.class).setRequired(true));
            add(new HiddenField<>("y", yModel, Integer.class).setRequired(true));
            add(new HiddenField<>("size", sizeModel, Integer.class).setRequired(true).add(RangeValidator.minimum(SIZE)));
            add(picture);
        }

        @Override
        protected void onValidateModelObjects() {
            if (picture.getFileUpload() == null) {
                error(getString("no.picture.selected"));
            }
        }

        @Override
        protected void onSubmit() {
            try {
                final int x = xModel.getObject();
                final int y = yModel.getObject();
                final int size = sizeModel.getObject();
                final BufferedImage image = ImageIO.read(picture.getFileUpload().getInputStream());
                final BufferedImage portrait = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_RGB);

                if (image == null) {
                    throw new IIOException("Invalid original image");
                }

                final boolean done = portrait.createGraphics().drawImage(image, 0, 0, WIDTH, HEIGHT, x, y, x + size, y + size, new ImageObserver() {
                    @Override
                    public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
                        if ((infoflags & ALLBITS) != 0) {
                            store(portrait);
                            return false;
                        }
                        return true;
                    }
                });
                if (done) {
                    store(portrait);
                }
            }
            catch (IOException ignored) {
                error(ignored);
            }
        }

        private void store(final BufferedImage portrait) {
            final WicketFileUpload fileUpload = createFileUpload(portrait);
            final UserProfile profile = getModelObject();
            final FileDescription oldPicture = profile.getProfilePicture();
            final FileDescription newPicture = fileDescriptionService.storeFile(fileUpload);
            profile.setPicture(UserProfile.Picture.FILE);
            profile.setProfilePicture(newPicture);
            profileService.save(profile);
            if (oldPicture != null) {
                fileDescriptionService.delete(oldPicture);
            }
        }

        @SneakyThrows(IOException.class)
        private WicketFileUpload createFileUpload(final BufferedImage portrait) {
            return new WicketFileUpload(picture.getFileUpload()) {
                private byte[] rawPicture = new byte[0];

                {
                    final ByteArrayOutputStream output = new ByteArrayOutputStream();
                    ImageIO.write(portrait, "png", output);
                    rawPicture = output.toByteArray();
                }

                @Override
                public String getContentType() {
                    return "image/png";
                }

                @Override
                public long getSize() {
                    return rawPicture.length;
                }

                @Override
                public <T> T handleData(final Function<InputStream, T> handler) {
                    return handler.apply(new ByteArrayInputStream(rawPicture));
                }
            };
        }
    }
}
