package se.su.dsv.scipro.user;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVRecord;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.Url;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAdminInternalUsers;
import se.su.dsv.scipro.loginlogout.pages.SetPasswordPage;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserFacade;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@Authorization(authorizedRoles = {Roles.SYSADMIN})
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.AvoidCatchingNPE"})
public class AdminImportUserPage extends AbstractAdminUsersPage implements MenuHighlightAdminInternalUsers {

    public static final String FIRSTNAME = "firstname";
    public static final String LASTNAME = "lastname";
    public static final String EMAIL = "email";
    private static final Logger LOGGER = LoggerFactory.getLogger(AdminImportUserPage.class);

    @Inject
    private UserFacade userFacade;
    @Inject
    private MailEventService mailEventService;

    private List<String> failed = new ArrayList<>();
    private int successIndex = 0;
    private boolean indexVisible = false;

    public AdminImportUserPage() {
        add(new FeedbackPanel("feedback"));
        add(new UploadForm("form"));

        AutoHidingListView<String> failList = new AutoHidingListView<String>("failList", new PropertyModel<ArrayList<String>>(this, "failed")) {
            @Override
            protected void populateItem(ListItem<String> item) {
                item.add(new Label("details", item.getModelObject()));
            }
        };
        add(failList);

        add(new Label("successNo", new PropertyModel<>(this, "successIndex")){
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(indexVisible);
            }
        });
    }

    private class UploadForm extends Form {

        private final FileUploadField upload;

        public UploadForm(String id) {
            super(id);
            upload = new FileUploadField("upload");
            add(upload);
        }

        @Override
        protected void onSubmit() {
            failed = new ArrayList<>();
            try {
                Reader in = new InputStreamReader(upload.getFileUpload().getInputStream(), StandardCharsets.UTF_8);
                importUsers(in);
            } catch (IOException e) {
                String message = "Couldn't read file.";
                error(message);
                LOGGER.info(message, e);
            } catch (NullPointerException e){
                String message = "No file selected";
                error(message);
                LOGGER.info(message, e);
            }
        }

        private void importUsers(Reader in) throws IOException {
            Iterable<CSVRecord> recordIterable = CSVFormat.DEFAULT.withHeader().parse(in);

            int index = 0;
            successIndex = 0;
            indexVisible = true;

            for (CSVRecord record : recordIterable) {
                index++;
                try {
                    String firstName = record.get(FIRSTNAME);
                    String lastName = record.get(LASTNAME);
                    String email = record.get(EMAIL);

                    Pair<User, String> userStringPair = userFacade.attemptCreateUser(firstName, lastName, email);
                    if (userStringPair.getHead() != null) {
                        sendActivateAccountMail(userStringPair.getHead(), userStringPair.getTail());
                        successIndex++;
                    } else {
                        failed.add("Row " + index + ": " + userStringPair.getTail() + " " + record.toString());
                    }

                } catch(IllegalArgumentException e) {
                    String message;
                    if (e.getMessage().contains("Index")){
                        message = "Row " + index + ": invalid row " + record.toString();
                        failed.add(message);
                        LOGGER.warn(message, e);
                    } else {
                        message = "Invalid file";
                        error(message);
                        LOGGER.warn(message, e);
                        return;
                    }
                } catch (Exception e){
                    String message = "Row " + index + ": Unknown error: " + record.toString();
                    failed.add(message);
                    LOGGER.warn(message, e);
                }
            }
        }
    }

    private void sendActivateAccountMail(final User user, final String hash) {
        final PageParameters pp = new PageParameters();
        pp.add("hash", hash);
        final String setPasswordURL = RequestCycle.get().getUrlRenderer().renderFullUrl(Url.parse(urlFor(SetPasswordPage.class, pp)));
        final String subject = getString("activate.account.subject");
        final String body = getString("activate.account.content") + " " + setPasswordURL;

        final MailEvent mailEvent = new MailEvent();
        mailEvent.getRecipients().add(user);
        mailEvent.setSubject(subject);
        mailEvent.setMessageBody(body);

        mailEventService.save(mailEvent);
    }

}