package se.su.dsv.scipro.user;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.AjaxCallListener;
import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
import org.apache.wicket.ajax.attributes.ThrottlingSettings;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.form.EnumChoiceRenderer;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.time.Duration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.components.AjaxCheckBoxMultipleChoice;
import se.su.dsv.scipro.components.AjaxLinkPanel;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.components.ModalWindowPlus;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.profile.DisplayUserProfilePanel;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

@Authorization(authorizedRoles={Roles.ADMIN})
public class AdminRolePage extends AbstractAdminUsersPage {

    public static final long DURATION = 500L;
    private static final Logger LOGGER = LoggerFactory.getLogger(AdminRolePage.class);

    @Inject
    private UserService userService;

    private UserService.Filter filter = new UserService.Filter();
    private ExportableDataPanel<User, String> dataPanel;
    private ModalWindowPlus editUserModalWindowPlus;

    public AdminRolePage() {
        add(new FeedbackPanel("feedback"));
        addFilter();
        addDataPanel();
        setupEditUserDialog();
    }

    private void addFilter() {
        addRoleFilter();
        addNameFilter();
        addInternalUserFilter();
    }

    private void addInternalUserFilter() {
        add(new AjaxCheckBox(INTERNAL_USER_FILTER, new PropertyModel<Boolean>(filter, "internalUsersOnly")) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(dataPanel);
            }
        });
    }

    private void addNameFilter() {
        TextField<String> searchField = new TextField<>(NAME_FILTER, new PropertyModel<String>(filter, "name"));

        OnChangeAjaxBehavior userSearchTextBehaviour = new OnChangeAjaxBehavior() {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(dataPanel);
            }

            @Override
            protected void updateAjaxAttributes(AjaxRequestAttributes attributes) {
                super.updateAjaxAttributes(attributes);
                attributes.setThrottlingSettings(new ThrottlingSettings(getId(), Duration.milliseconds(DURATION), true));
            }
        };

        searchField.add(userSearchTextBehaviour);
        add(searchField);
    }

    private void addRoleFilter() {
        add(new AjaxCheckBoxMultipleChoice<Roles>(ROLE_FILTER, new PropertyModel<Collection<Roles>>(filter, "roles"), Arrays.asList(Roles.values()), new EnumChoiceRenderer<Roles>(this)) {
            @Override
            public void onUpdate(AjaxRequestTarget target) {
                target.add(dataPanel);
            }
        });
    }

    private void setupEditUserDialog() {
        editUserModalWindowPlus = new ModalWindowPlus("editUserDialog");
        add(editUserModalWindowPlus);
    }

    private void addDataPanel() {
        SortableDataProvider<User, String> provider = new FilteredDataProvider<>(userService, Model.of(filter));
        provider.setSort("lastName", SortOrder.ASCENDING);
        dataPanel = new ExportableDataPanel<>(USER_DATA_PANEL, createColumns(), provider);
        add(dataPanel);
    }

    private List<IColumn<User, String>> createColumns() {
        List<IColumn<User, String>> columns = new ArrayList<>();
        columns.add(createUserColumn());
        Iterable<Roles> roles = Iterables.filter(Arrays.asList(Roles.values()), new Predicate<Roles>() {
            @Override
            public boolean apply(final Roles input) {
                return SciProSession.get().authorizedForRole(input);
            }
        });
        for (final Roles role : roles) {
            columns.add(createRoleColumn(role));
        }
        return columns;
    }

    private AbstractColumn<User, String> createUserColumn() {
        return new AbstractColumn<User, String>(Model.of("Name"), "lastName") {
            @Override
            public void populateItem(Item<ICellPopulator<User>> cellItem, String componentId, final IModel<User> rowModel) {
                User user = rowModel.getObject();
                IModel<String> label = Model.of(MessageFormat.format("{0} {1} <{2}>", user.getFirstName(), user.getLastName(), user.getEmailAddress()));
                cellItem.add(new AjaxLinkPanel<User>(componentId, rowModel, label) {
                    @Override
                    public void onClick(AjaxRequestTarget target, IModel<User> clickedModel) {
                        editUserModalWindowPlus.setContent(new DisplayUserProfilePanel(editUserModalWindowPlus.getContentId(), clickedModel));
                        editUserModalWindowPlus.setTitle(clickedModel.getObject().getFullName());
                        editUserModalWindowPlus.show(target);
                    }
                });
            }
        };
    }

    private AbstractColumn<User, String> createRoleColumn(final Roles role) {
        return new AbstractColumn<User, String>(Model.of(role.name())) {
            @Override
            public void populateItem(Item<ICellPopulator<User>> cellItem, String componentId, final IModel<User> rowModel) {
                Fragment fragment = new Fragment(componentId, "roleCheckbox", AdminRolePage.this);
	            //noinspection WicketForgeJavaIdInspection
	            fragment.add(new AjaxCheckBox("checkbox", Model.of(rowModel.getObject().hasRole(role))) {
                    @Override
                    protected void onUpdate(AjaxRequestTarget target) {
                        User user = rowModel.getObject();
                        if (getModelObject()) {
                            LOGGER.info("{} gave {} the role {}", SciProSession.get().getUser(), user, role);
                            user.addRole(role);
                        } else {
                            LOGGER.info("{} removed the role {} from {}", SciProSession.get().getUser(), role, user);
                            user.removeRole(role);
                        }
                        userService.save(user);
                    }

                    @Override
                    protected void updateAjaxAttributes(final AjaxRequestAttributes attributes) {
                        super.updateAjaxAttributes(attributes);
                        String message = getModelObject() ? "Are you sure you want to remove " + role.name() + " from " + rowModel.getObject().getFullName() + "?"
                                                          : "Are you sure you want to give the role " + role.name() + " to " + rowModel.getObject().getFullName() + "?";
                        String escapedMessage = message.replaceAll("'", "\\\\'");
                        AjaxCallListener ajaxCallListener = new AjaxCallListener();
                        ajaxCallListener.onPrecondition("if (!confirm('" + escapedMessage + "')) {window.location.reload(); return false;}");
                        attributes.getAjaxCallListeners().add(ajaxCallListener);
                    }
                });
                cellItem.add(fragment);
            }
        };
    }

    static final String INTERNAL_USER_FILTER = "internalUsersOnly";
    static final String NAME_FILTER = "searchField";
    static final String ROLE_FILTER = "roleFilter";
    static final String USER_DATA_PANEL = "userDataView";
}
