package se.su.dsv.scipro.finalseminar

import javax.inject.Inject

import org.apache.commons.lang3.StringUtils
import org.apache.wicket.Component
import org.apache.wicket.feedback.FencedFeedbackPanel
import org.apache.wicket.markup.html.WebMarkupContainer
import org.apache.wicket.markup.html.basic.Label
import org.apache.wicket.markup.html.form._
import org.apache.wicket.markup.html.link.{ExternalLink, Link}
import org.apache.wicket.markup.html.panel.{Fragment, GenericPanel}
import org.apache.wicket.model._
import org.apache.wicket.model.util.SetModel
import org.apache.wicket.request.mapper.parameter.PageParameters
import org.apache.wicket.validation.validator.RangeValidator
import se.su.dsv.scipro.components.AddUsersPanel
import se.su.dsv.scipro.data.enums.DateStyle
import se.su.dsv.scipro.date.DateService
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService
import se.su.dsv.scipro.nonworkperiod.NonWorkDayPeriodService
import se.su.dsv.scipro.notifications.NotificationController
import se.su.dsv.scipro.notifications.dataobject.{NotificationSource, SeminarEvent}
import se.su.dsv.scipro.project.{Project, ProjectService}
import se.su.dsv.scipro.security.auth.roles.Roles
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.supervisor.pages.SupervisorInteractWithReviewerPage
import se.su.dsv.scipro.system.{DomainObject, Language, LanguageService, ProjectType, User}
import se.su.dsv.scipro.util.JavascriptEventConfirmation

import scala.collection.immutable

class SeminarCRUDPanel(id: String, seminar: IModel[FinalSeminar]) extends GenericPanel(id, seminar) {
  panel =>
  import se.su.dsv.scipro.finalseminar.SeminarCRUDPanel._

  @Inject private var languageService: LanguageService = _
  @Inject private var seminarService: FinalSeminarService = _
  @Inject private var projectService: ProjectService = _
  @Inject private var notificationController: NotificationController = _
  @Inject private var nonWorkDays: NonWorkDayPeriodService = _
  @Inject private var dateService: DateService = _
  @Inject private var finalSeminarCreationService: FinalSeminarCreationService = _
  @Inject private var generalSystemSettingsService: GeneralSystemSettingsService = _

  private var opponents: AddUsersPanel = _
  private var participants: AddUsersPanel = _
  private var events: immutable.Set[SeminarEvent.Event] = immutable.Set.empty

  add(new MoveFinalSeminarPanel("moveDateModal", seminar))
  add(new FinalSeminarForm("form", new CompoundPropertyModel[FinalSeminar](seminar)))
  protected override def onConfigure() = {
    val currentUser: User = SciProSession.get.getUser
    setVisible((currentUser == getModelObject.getHeadSupervisor) || SciProSession.get.authorizedForRole(Roles.ADMIN))
    ()
  }

  private class FinalSeminarForm(id: String, seminar: IModel[FinalSeminar]) extends Form(id, seminar) {
    add(new FencedFeedbackPanel("feedback", SeminarCRUDPanel.this))
    
    val maxParticipants = new RequiredTextField(MAX_PARTICIPANTS, classOf[Integer])
    add(maxParticipants.add(RangeValidator.minimum[Integer](getModelObject.getMinActiveParticipants)))
    
    val maxOpponents = new RequiredTextField(MAX_OPPONENTS, classOf[Integer])
    add(maxOpponents.add(RangeValidator.minimum[Integer](getModelObject.getMinOpponents)))
    
    add(new RequiredTextField[String](ROOM) {
      protected override def onModelChanged() = events = events + SeminarEvent.Event.ROOM_CHANGED
    })
    add(new WebMarkupContainer(CANCELLED) {
      protected override def onConfigure() = setVisibilityAllowed(getModelObject.isCancelled)
    })
    add(new FinalSeminarDateTimeComponent(DATE) {
      add(new SeminarDateValidator(seminar.getObject, dateService.getFormat(DateStyle.DATE)))

      override def onConfigure(): Unit = {
        setEnabled(seminar.getObject.getId == null)
        ()
      }
    })
    add(new WebMarkupContainer(MOVE_SEMINAR_LINK){
      override def onConfigure(): Unit = {
        setVisible(seminar.getObject.getId != null)
        ()
      }
    })

    val externalRoomBookingURL = new LoadableDetachableModel[String] {
      override def load(): String = generalSystemSettingsService.getGeneralSystemSettingsInstance.getExternalRoomBookingURL
    }
    add(new ExternalLink("externalRoomBooking", externalRoomBookingURL) {
      override def onConfigure() = setVisible(getDefaultModelObject != null)
    })

    import scala.collection.JavaConverters._

    val languages: java.util.List[Language] = languageService.findAll
    val language = languages.asScala.headOption.orNull

    if (languages.size() == 1) {
      if (getModelObject.getPresentationLanguage == null) {
        getModelObject.setPresentationLanguage(language)
      }
      if (getModelObject.getReportLanguage == null) {
        getModelObject.setReportLanguage(language)
      }
    }

    add(new DropDownChoice[Language](PRESENTATION_LANGUAGE, languages, new ChoiceRenderer[AnyRef]("name", "id")).setRequired(true))
    add(new DropDownChoice[Language](REPORT_LANGUAGE, languages, new ChoiceRenderer[AnyRef]("name", "id")).setRequired(true))

    opponents = new AddUsersPanel("opponents", new SetModel[User](new java.util.HashSet[User]), AddUsersPanel.LookFor.AUTHORS)
    add(opponents)
    participants = new AddUsersPanel("participants", new SetModel[User](new java.util.HashSet[User]), AddUsersPanel.LookFor.AUTHORS)
    add(participants)
    
    add(new Link[FinalSeminar](DELETE, seminar) {
      add(new JavascriptEventConfirmation("onclick", new ResourceModel("confirm")))
      def onClick() = {
        val project: Project = getModelObject.getProject
        seminarService.delete(getModelObject)
        setModelObject(new FinalSeminar(project))
        success(getString("deleted"))
      }

      protected override def onConfigure() = setVisible(getModelObject.getId != null)
    })
    add(new Label(SUBMIT, new CreateUpdateModel(getModel)))

    protected override def onValidate() = {
      import scala.collection.JavaConversions._
      val seminar: FinalSeminar = getModelObject
      for (opponent <- opponents.getModelObject) {
        if (seminar.getProject.getHeadSupervisor == opponent) {
          error("Head supervisor can't be opponent")
        }
        if (seminar.getProject.getReviewers.contains(opponent)) {
          error("Reviewer can't be opponent")
        }
        if (seminar.getProject.isParticipant(opponent)) {
          error("Authors can't be opponents on their own final seminar")
        }
        if (seminar.getOpponents.contains(opponent)) {
          error("Selected opponent is already opponent")
        }
        if (seminar.getActiveParticipants.contains(opponent)) {
          error("Selected opponent is already active participant")
        }
        if (participants.getModelObject.contains(opponent)) {
          error("Can't add same user as both opponent and active participant")
        }
      }
      for (participant <- participants.getModelObject) {
        if (seminar.getProject.getHeadSupervisor == participant) {
          error("Head supervisor can't be active participant")
        }
        if (seminar.getProject.getReviewers.contains(participant)) {
          error("Reviewer can't be active participant")
        }
        if (seminar.getProject.isParticipant(participant)) {
          error("Authors can't be active participants on their own final seminar")
        }
        if (seminar.getOpponents.contains(participant)) {
          error("Selected active participant is already opponent")
        }
        if (seminar.getActiveParticipants.contains(participant)) {
          error("Selected active participant is already active participant")
        }
        if (opponents.getModelObject.contains(participant)) {
          error("Can't add same user as both opponent and active participant")
        }
      }
    }

    protected override def onValidateModelObjects(): Unit = {
      def maybeAddParticipant(userToAdd: User, userProjects: java.util.List[Project], add: => Unit) = {
        if (userProjects.isEmpty) error(getString("opponent.no.project", Model.of(userToAdd)))
        else if (userProjects.size > 1) error(getString("opponent.too.many.projects", Model.of(userToAdd)))
        else add
      }

      import scala.collection.JavaConversions._
      val seminar: FinalSeminar = getModelObject
      if (!seminar.getProject.isFinalSeminarRuleExempted && nonWorkDays.isNonWorkDay(seminar.getStartDate)) {
        error(getString("nonworkday"))
      }
      for (participant <- participants.getModelObject) {
        if (!seminarService.isActiveParticipant(seminar, participant)) {
          val participantProjects: java.util.List[Project] = projectService.getActiveProjects(participant, seminar.getProjectType)
          maybeAddParticipant(participant, participantProjects, {
            val participation: FinalSeminarActiveParticipation = new FinalSeminarActiveParticipation
            participation.setUser(participant)
            participation.setFinalSeminar(seminar)
            participation.setProject(participantProjects.get(0))
            seminar.addActiveParticipant(participation)
            events = events + SeminarEvent.Event.PARTICIPATION_CHANGED
          })
        }
      }
    }

    protected override def onSubmit() = {
      val isCreated: Boolean = getModelObject.getId == null
      if (isCreated) {
        finalSeminarCreationService.isCreationAllowed(getModelObject.getProject) match {
          case RequestNeeded => showApprovalDialog(new SendRequestFragment(_, new ResourceModel("reviewer.approval.explanation")))
          case Denied(approval) => showApprovalDialog(new SendRequestFragment(_,
            new StringResourceModel("reviewer.approval.denied", null, approval.getCurrentDecision.getDecisionDate, approval.getCurrentReason)))
          case AwaitingDecision(approval) =>
            val deadline = approval.getCurrentDeadline
            val requested = approval.getCurrentDecision.getRequested
            showApprovalDialog(new Label(_, new StringResourceModel("reviewer.approval.waiting", null, requested, deadline)))
          case NoReviewer | Granted | GrantedByTimeout(_) => saveSeminar()
        }
      }
      else saveSeminar()
    }

    private class CreateUpdateModel(model: IModel[_ <: DomainObject]) extends AbstractReadOnlyModel[String] {
      def getObject: String = {
        if (model.getObject.getId == null) getString("create") else getString("update")
      }
    }
  }

  private def saveSeminar() = {
    def getPotentialParticipantProject(participant: User, projectType: ProjectType): Option[Project] = {
      val projects = projectService.getActiveProjects(participant, projectType)
      if (projects.size() == 1) Some(projects.get(0))
      else None
    }

    import scala.collection.JavaConverters._
    def addOpponentsTo(finalSeminar: FinalSeminar): Unit = {
      for (potentialOpponent <- opponents.getModelObject.asScala) {
        val maybeProject = getPotentialParticipantProject(potentialOpponent, finalSeminar.getProjectType)

        maybeProject.fold(error(getString("opponent.no.project", Model.of(potentialOpponent)))) {
          project => seminarService.oppose(potentialOpponent, finalSeminar, project) match {
            case Left(AlreadyOpponent) => error(s"Selected opponent ${potentialOpponent.getFullName} is already opponent")
            case Left(AlreadyParticipant) => error(s"Selected opponent ${potentialOpponent.getFullName} is already active participant")
            case Left(NotAuthor) => error(s"Failed to add ${potentialOpponent.getFullName} as an opponent")
            case Right(opponent) =>
              success(s"Added ${potentialOpponent.getFullName} as an opponent")
              events += SeminarEvent.Event.OPPOSITION_CHANGED
          }
        }
      }
    }

    val isCreated: Boolean = getModelObject.getId == null
    var finalSeminar: FinalSeminar = getModelObject
    finalSeminar.setCancelled(false)
    finalSeminar = seminarService.save(finalSeminar)
    setModelObject(finalSeminar)
    success(getString("final.seminar.updated"))
    addOpponentsTo(finalSeminar)
    if (isCreated) {
      notificationController.notifySeminar(finalSeminar, SeminarEvent.Event.CREATED, new NotificationSource)
    }
    else {
      for (event <- events) {
        notificationController.notifySeminar(finalSeminar, event, new NotificationSource)
      }
    }
    events = immutable.Set.empty
  }

  private class SendRequestFragment(id: String, information: IModel[String]) extends Fragment(id, "sendRequest", SeminarCRUDPanel.this) {
    add(new Label("information", information))
    add(new Form("form", new Model(FinalSeminarCreationAction.SEND_REQUEST_FOR_APPROVAL)) {
      import scala.collection.JavaConversions._

      add(new FencedFeedbackPanel("feedback", SendRequestFragment.this))

      private val choice = new RadioChoice[FinalSeminarCreationAction]("choice", getModel, FinalSeminarCreationAction.values().toList, new EnumChoiceRenderer[FinalSeminarCreationAction](this))
      choice.setRequired(true)
      add(choice)

      private val reasonInput = new TextArea[String]("reason", new Model[String])
      add(reasonInput)

      override def onSubmit() = {
        requestDialog.setVisible(false)
        getModelObject match {
          case FinalSeminarCreationAction.SEND_REQUEST_FOR_APPROVAL => setResponsePage(classOf[SupervisorInteractWithReviewerPage], new PageParameters().set("anchor", "#finalSeminarApproval"))
          case FinalSeminarCreationAction.CREATE_ANYWAY =>
            val reason = reasonInput.getModelObject
            if (StringUtils.isNotEmpty(reason)) {
              panel.getModelObject.setCreationReason(reason)
              saveSeminar()
            }
            else {
              requestDialog.setVisible(true)
              error(getString("reviewer.approval.missing.reason"))
            }
        }
      }
    })
  }

  val requestDialog = new WebMarkupContainer("requestDialog")
  requestDialog.setVisible(false)
  add(requestDialog)

  private def showApprovalDialog(content: String => Component): Unit = {
    requestDialog.setVisible(true)
    requestDialog.addOrReplace(content("content"))
    ()
  }
}

object SeminarCRUDPanel {
  private[finalseminar] final val FORM: String = "form"
  private[finalseminar] final val CANCELLED: String = "cancelled"
  private[finalseminar] final val MAX_PARTICIPANTS: String = "maxParticipants"
  private[finalseminar] final val MAX_OPPONENTS: String = "maxOpponents"
  private[finalseminar] final val ROOM: String = "room"
  private[finalseminar] final val DATE: String = "startDate"
  private[finalseminar] final val PRESENTATION_LANGUAGE: String = "presentationLanguage"
  private[finalseminar] final val REPORT_LANGUAGE: String = "reportLanguage"
  private[finalseminar] final val DAYS_AHEAD: String = "daysAheadLabel"
  private[finalseminar] final val DELETE: String = "delete"
  private[finalseminar] final val SUBMIT: String = "submit"
  private[finalseminar] final val MOVE_SEMINAR_LINK: String = "moveSeminarLink"
}