package se.su.dsv.scipro.reviewer

import java.lang.Boolean
import javax.inject.Inject

import org.apache.wicket.feedback.FencedFeedbackPanel
import org.apache.wicket.markup.html.basic.EnumLabel
import org.apache.wicket.markup.html.form.upload.FileUploadField
import org.apache.wicket.markup.html.form.{Button, CheckBox, Form, TextArea}
import org.apache.wicket.model._
import org.apache.wicket.request.mapper.parameter.PageParameters
import se.su.dsv.scipro.StringValueConversions
import se.su.dsv.scipro.components.InfoPanel
import se.su.dsv.scipro.file.{FileDescription, FileDescriptionService}
import se.su.dsv.scipro.files.WicketFileUpload
import se.su.dsv.scipro.forum.panels.threaded.SubmitForumReplyPanel
import se.su.dsv.scipro.grading.{AutoSavingCriteriaPanel, GradingReportPdfResourceFileUpload, ProjectCriteriaGradingReportPdf}
import se.su.dsv.scipro.project.Project
import se.su.dsv.scipro.project.panels.ProjectHeaderPanel
import se.su.dsv.scipro.report.{GradingReportService, ReviewerGradingReport}
import se.su.dsv.scipro.reviewer.timeline.TimelinePanel
import se.su.dsv.scipro.reviewing.ReviewerApproval.Step
import se.su.dsv.scipro.reviewing.{MyReviewService, ReviewerApproval, ReviewerDecisionService, ReviewerInteractionService}
import se.su.dsv.scipro.session.SciProSession
import se.su.dsv.scipro.supervisor.panels.FinalSeminarApprovalProcessPanel

class ReviewerApprovalDecisionPage(pp: PageParameters) extends ReviewerPage(pp) {
  @Inject private var reviewerInteractionService: ReviewerInteractionService = _
  @Inject private var reviewerDecisionService: ReviewerDecisionService = _
  @Inject private var fileDescriptionService: FileDescriptionService = _
  @Inject private var myReviewService: MyReviewService = _
  @Inject private var gradingReportService: GradingReportService = _

  val maybeApproval = for {
    id <- pp.get(ReviewerApprovalDecisionPage.id).asLong
    approval <- Option(myReviewService.findApproval(id))
    currentUser = SciProSession.get().getUser
    if approval.getProject.isReviewer(currentUser)
  }
  yield {
    new LoadableDetachableModel[ReviewerApproval]() {
      override def load(): ReviewerApproval = myReviewService.findApproval(id)
    }
  }

  maybeApproval.fold(setResponsePage(getApplication.getHomePage)) { approval =>
    setDefaultModel(approval)

    val project: IModel[Project] = new PropertyModel[Project](approval, "project")
    add(new ProjectHeaderPanel("projectHeader", project))
    add(new InfoPanel("info", new ResourceModel("info")))
    add(new SubmitForumReplyPanel[Project]("communication", project, new ReviewerInteractionForumThread(reviewerInteractionService, fileDescriptionService)))
    add(new FinalSeminarApprovalProcessPanel("details", approval))
    add(new DecisionForm("decision", approval))
    add(new EnumLabel[Step]("title", new PropertyModel[Step](approval, "step")))
    add(new TimelinePanel("timeline", project))
  }

  private class DecisionForm(id: String, reviewerApproval: IModel[ReviewerApproval]) extends Form[ReviewerApproval](id, reviewerApproval) {
    add(new FencedFeedbackPanel("feedbackPanel", this))
    val feedback = new TextArea[String]("feedback", Model.of(""))
    feedback.setRequired(true)
    add(feedback)
    val attachment = new FileUploadField("attachment")
    add(attachment)

    val gradingReport = new LoadableDetachableModel[ReviewerGradingReport] {
      override def load(): ReviewerGradingReport = gradingReportService.getReviewerGradingReport(reviewerApproval.getObject.getProject)
    }

    add(new AutoSavingCriteriaPanel("gradingReport", gradingReport))
    val attachGradingReport = Model.of[Boolean](false)
    add(new CheckBox("attachGradingReport", attachGradingReport))

    add(new Button("reject") {
      add(new EnumLabel[Step]("label", new PropertyModel[Step](reviewerApproval, "step")) {
        override def resourceKey(value: Step): String = super.resourceKey(value) + ".resend.label"
      })
      override def onSubmit() = {
        reviewerDecisionService.reject(DecisionForm.this.getModelObject, feedback.getModelObject, WicketFileUpload.ofOptional(attachment.getFileUpload), getAttachedGradingReport)
      }
    })
    add(new Button("approve") {
      add(new EnumLabel[Step]("label", new PropertyModel[Step](reviewerApproval, "step")) {
        override def resourceKey(value: Step): String = super.resourceKey(value) + ".approve.label"
      })
      override def onSubmit() = {
        reviewerDecisionService.approve(DecisionForm.this.getModelObject, feedback.getModelObject, WicketFileUpload.ofOptional(attachment.getFileUpload), getAttachedGradingReport)
      }
    })
    add(new EnumLabel[Step]("approveLabel", new PropertyModel[Step](reviewerApproval, "step")) {
      override def resourceKey(value: Step): String = super.resourceKey(value) + ".approve.label"
    })
    add(new EnumLabel[Step]("approveDescription", new PropertyModel[Step](reviewerApproval, "step")) {
      override def resourceKey(value: Step): String = super.resourceKey(value) + ".approve.description"

      override def onConfigure(): Unit = setVisible(!getString(resourceKey(getModelObject)).isEmpty)
    })
    add(new EnumLabel[Step]("resendLabel", new PropertyModel[Step](reviewerApproval, "step")) {
      override def resourceKey(value: Step): String = super.resourceKey(value) + ".resend.label"
    })
    add(new EnumLabel[Step]("resendDescription", new PropertyModel[Step](reviewerApproval, "step")) {
      override def resourceKey(value: Step): String = super.resourceKey(value) + ".resend.description"

      override def onConfigure(): Unit = setVisible(!getString(resourceKey(getModelObject)).isEmpty)
    })

    private def getAttachedGradingReport: Option[FileDescription] = {
      if (attachGradingReport.getObject) {
        val gradingReportPdf = new ProjectCriteriaGradingReportPdf(gradingReport.getObject)
        val upload = new GradingReportPdfResourceFileUpload(gradingReportPdf, gradingReport.getObject.getProject.getReviewer)
        val gradingReportFile = fileDescriptionService.storeFile(upload)
        Some(gradingReportFile)
      }
      else None
    }

    override protected def onConfigure() = {
      setVisible(!getModelObject.isDecided)
    }
  }
}

object ReviewerApprovalDecisionPage {
  val id = "id"

  def pageParametersFor(approval: ReviewerApproval): PageParameters = {
    val parameters = new PageParameters()
    parameters.set(id, approval.getId)
  }
}