package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.Page;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.tester.FormTester;
import org.hamcrest.MatcherAssert;
import org.hamcrest.core.IsCollectionContaining;
import org.joda.time.DateTime;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.applicationperiod.AdminEditApplicationPeriodPage.*;


public class AdminEditApplicationPeriodPageTest extends SciProTest {

    public static final boolean NOT_OVERLAPPING = false;
    public static final boolean OVERLAPPING = true;

    private ProjectType bachelor;

    @Before
    public void setUp() throws Exception {
        bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        bachelor.addModule(ProjectModule.MATCH);
        when(projectTypeService.findWithModule(ProjectModule.MATCH)).thenReturn(Collections.singletonList(bachelor));
        when(applicationPeriodService.findOne(isA(Long.class))).thenReturn(new ApplicationPeriod());
        startPage(NOT_OVERLAPPING);
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(AdminEditApplicationPeriodPage.class);
    }

    @Test
    public void contains_project_type_selection() throws Exception {
        tester.assertListView(path(FORM, PROJECT_TYPES), projectTypeService.findWithModule(ProjectModule.MATCH));
    }

    @Test
    public void title_field() throws Exception {
        tester.assertComponent(path(FORM, TITLE), RequiredTextField.class);
    }

    @Test
    public void submit_saves_application_period() throws Exception {
        String title = "title";

        submitForm(title, 3, 4, 5);

        ArgumentCaptor<ApplicationPeriod> captor = ArgumentCaptor.forClass(ApplicationPeriod.class);
        Mockito.verify(applicationPeriodService).save(captor.capture());

        Assert.assertEquals(captor.getValue().getName(), title);
    }

    @Test
    public void submit_fails_if_dates_are_not_validated() throws Exception {
        submitForm("title", 3, 2, 5);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("startDate.DatesValidator"));
    }

    @Test
    public void feedback_when_submitting_application_period() throws Exception {
        Page page = tester.getLastRenderedPage();
        submitForm("title", 1, 2, 3);
        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        MatcherAssert.assertThat(messages, IsCollectionContaining.hasItem(page.getString("success")));
    }

    @Test
    public void submission_fails_if_period_exists() throws Exception {
        startPage(OVERLAPPING);
        submitForm("title", 1, 2, 3);
        tester.assertErrorMessages(tester.getLastRenderedPage().getString("overlapping"));
    }

    @Test
    public void saves_correct_project_types() throws Exception {
        FormTester formTester = tester.newFormTester(FORM);
        fillInForm("Title", 0, 1, 2, formTester);
        formTester.setValue(path(PROJECT_TYPES, 0, CHECKBOX), true);
        formTester.submit();

        ArgumentCaptor<ApplicationPeriod> captor = ArgumentCaptor.forClass(ApplicationPeriod.class);
        Mockito.verify(applicationPeriodService).save(captor.capture());
        MatcherAssert.assertThat(captor.getValue().getProjectTypes(), IsCollectionContaining.hasItem(bachelor));
    }

    private void submitForm(String title, int startDate, int endDate, int courseStartDate) {
        FormTester formTester = tester.newFormTester(FORM);
        fillInForm(title, startDate, endDate, courseStartDate, formTester);
        formTester.submit();
    }

    private void fillInForm(String title, int startDate, int endDate, int courseStartDate, FormTester formTester) {
        formTester.setValue(TITLE, title);
        setDate(formTester, START_DATE, new DateTime().plusDays(startDate).toDate());
        setDate(formTester, END_DATE, new DateTime().plusDays(endDate).toDate());
        setDate(formTester, COURSE_START_DATE, new DateTime().plusDays(courseStartDate).toDate());
    }

    private void setDate(FormTester formTester, String field, Date date) {
        String dateString = new SimpleDateFormat(dateService.getFormat(DateStyle.DATE)).format(date);
        String timeString = new SimpleDateFormat(dateService.getFormat(DateStyle.TIME)).format(date);
        formTester.setValue(path(field, "date"), dateString);
        formTester.setValue(path(field, "time"), timeString);
    }

    private void startPage(boolean overlapping) {
        when(applicationPeriodService.doesPeriodOverlap(Matchers.any(ApplicationPeriod.class))).thenReturn(overlapping);
        tester.startPage(new AdminEditApplicationPeriodPage(new PageParameters()));
    }
}
