package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.*;

import static org.hamcrest.CoreMatchers.hasItem;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertThat;
import static org.mockito.Matchers.isA;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class AttendingPanelTest extends SciProTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING);

    AttendingPanel panel;
    Project project;
    User user;

    @Before
    public void setUp() throws Exception {
        project = Project.builder().title("some title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor")).build();
        project.setId(9L);

        user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        user.setRoles(new HashSet<>(Collections.singletonList(Roles.AUTHOR)));

        setLoggedIn(true);
        setLoggedInAs(user);
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        assert panel != null;
    }

    @Test
    public void oppositions() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(project);
        finalSeminar.setStartDate(new Date());

        when(finalSeminarService.findUserOpposing(isA(Project.class), isA(User.class))).thenReturn(new ArrayList<>(Collections.singletonList(finalSeminar)));

        startPanel();

        tester.assertListView(path(panel.getId(), "oppositions"), Collections.singletonList(finalSeminar));
        tester.assertComponent(path(panel.getId(), "oppositions", "0", "seminarLink"), BookmarkablePageLink.class);
        tester.assertInvisible(path(panel.getId(), "noOppositions"));
    }

    @Test
    public void no_oppositions() throws Exception {
        startPanel();
        tester.assertInvisible(path(panel.getId(), "oppositions"));
        tester.assertComponent(path(panel.getId(), "noOppositions"), Label.class);
    }

    @Test
    public void participations() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(project);
        finalSeminar.setStartDate(new Date());
        User unregisteredUser = user;

        when(finalSeminarService.findUserParticipating(isA(Project.class), isA(User.class))).thenReturn(new ArrayList<>(Collections.singletonList(finalSeminar)));
        when(finalSeminarActiveParticipationService.findByFinalSeminarUser(finalSeminar, unregisteredUser )).thenReturn(new FinalSeminarActiveParticipation());

        startPanel();

        tester.assertListView(path(panel.getId(), "wmc", "participations"), Collections.singletonList(finalSeminar));
        tester.assertComponent(path(panel.getId(), "wmc", "participations", "0", "seminarLink"), BookmarkablePageLink.class);
        tester.assertInvisible(path(panel.getId(), "noParticipations"));
    }

    @Test
    public void no_participations() throws Exception {
        startPanel();
        tester.assertInvisible(path(panel.getId(), "wmc", "participations"));
        tester.assertComponent(path(panel.getId(), "noParticipations"), Label.class);
    }

    @Test
    public void unregister_as_participant() throws Exception {
        FinalSeminar finalSeminar = new FinalSeminar();
        finalSeminar.setProject(project);
        User unregisteredUser = user;
        FinalSeminarActiveParticipation participation = new FinalSeminarActiveParticipation();
        participation.setUser(unregisteredUser);
        finalSeminar.addActiveParticipant(participation);

        when(finalSeminarService.findUserParticipating(project, user)).thenReturn(Collections.singletonList(finalSeminar));
        when(finalSeminarActiveParticipationService.findByFinalSeminarUser(finalSeminar, user)).thenReturn(participation);

        startPanel();
        tester.clickLink(path(panel, "wmc", "participations", 0, "remove"));

        ArgumentCaptor<FinalSeminar> captor = ArgumentCaptor.forClass(FinalSeminar.class);
        verify(finalSeminarService).save(captor.capture());

        assertThat(captor.getValue().getActiveParticipants(), not(hasItem(unregisteredUser)));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new AttendingPanel("panel", Model.of(project)));
    }

}

