package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Component;
import org.apache.wicket.Page;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.visit.IVisit;
import org.apache.wicket.util.visit.IVisitor;
import org.hamcrest.MatcherAssert;
import org.hamcrest.core.IsCollectionContaining;
import org.hamcrest.core.StringContains;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.PageTest;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;
import se.su.dsv.scipro.util.PageParameterKeys;

import java.io.Serializable;
import java.util.*;

public class ProjectOppositionPageTest extends PageTest {

    private Project project;
    private FinalSeminar finalSeminar;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = new UserBuilder().setFirstName("Rolf").setLastName("Rolfsson").create();
        setLoggedInAs(user);
        finalSeminar = new FinalSeminar();

        User supervisor = new UserBuilder().setFirstName("Hej").setLastName("Svej").create();
        project = Project.builder()
                .title("Wanted Project")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        Project unwantedProject = Project.builder().title("Some unwanted title").projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Some project type", "Some description")).build();
        unwantedProject.setId(9L);
        finalSeminar.setProject(unwantedProject);
        finalSeminar.setStartDate(new Date());
        PageParameters pageParameters = new PageParameters();
        pageParameters.add(PageParameterKeys.MAP.get(Project.class), "1");
        Mockito.when(projectService.findOne(Matchers.any(Long.class))).thenReturn(project);
        Mockito.when(projectService.isPartOfProject(Matchers.any(Project.class), Matchers.any(User.class))).thenReturn(true);
        Mockito.when(finalSeminarService.findAll(Matchers.any(FinalSeminarService.Filter.class), Matchers.any(Pageable.class))).thenReturn(Collections.singleton(finalSeminar));
        Mockito.when(finalSeminarService.count(Matchers.any(FinalSeminarService.Filter.class))).thenReturn(1L);
        Mockito.when(finalSeminarService.canOppose(user, finalSeminar, project)).thenReturn(new CanRegister());
        Mockito.when(finalSeminarService.canParticipate(user, finalSeminar)).thenReturn(true);
        tester.startPage(ProjectOppositionPage.class, pageParameters);
    }

    @Test
    public void clickingOpposeUpdatesCorrectProject() throws Exception {
        final Page page = tester.getLastRenderedPage();
        page.visitChildren(OpposeColumnPanel.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                tester.clickLink(path(component.getPageRelativePath(), OpposeColumnPanel.LINK));
                Mockito.verify(finalSeminarService).oppose(user, finalSeminar, project);
                visit.stop();
            }
        });
    }

    @Test
    public void clicking_oppose_generates_feedback() throws Exception {
        final Page page = tester.getLastRenderedPage();
        page.visitChildren(OpposeColumnPanel.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                tester.clickLink(path(component.getPageRelativePath(), OpposeColumnPanel.LINK));

                List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
                MatcherAssert.assertThat(messages, IsCollectionContaining.hasItem(StringContains.containsString(page.getString("oppose", Model.of(finalSeminar)))));
                visit.stop();
            }
        });
    }

    @Test
    public void clickingParticipateUpdatesCorrectProject() throws Exception {
        final Page page = tester.getLastRenderedPage();
        page.visitChildren(ParticipateColumnPanel.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                tester.clickLink(path(component.getPageRelativePath(), ParticipateColumnPanel.LINK));
                Mockito.verify(finalSeminarService).participate(user, finalSeminar, project);
                visit.stop();
            }
        });
    }

    @Test
    public void clicking_participate_generates_feedback() throws Exception {
        final Page page = tester.getLastRenderedPage();
        page.visitChildren(ParticipateColumnPanel.class, new IVisitor<Component, Object>() {
            @Override
            public void component(Component component, IVisit<Object> visit) {
                tester.clickLink(path(component.getPageRelativePath(), ParticipateColumnPanel.LINK));

                List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
                MatcherAssert.assertThat(messages, IsCollectionContaining.hasItem(StringContains.containsString(page.getString("participate", Model.of(finalSeminar)))));
                visit.stop();
            }
        });
    }

    @Override
    protected Class<? extends Page> getPage() {
        return ProjectOppositionPage.class;
    }

    @Override
    protected PageParameters getPageParameters() {
        PageParameters pp = new PageParameters();
        pp.add(PageParameterKeys.MAP.get(Project.class), 1L);
        return pp;
    }
}
