package se.su.dsv.scipro.finalseminar;

import com.google.common.collect.Lists;
import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.model.Model;
import org.hamcrest.MatcherAssert;
import org.hamcrest.core.IsCollectionContaining;
import org.hamcrest.core.StringContains;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.AjaxRadioChoice;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.io.Serializable;
import java.util.List;

import static se.su.dsv.scipro.finalseminar.SeminarActiveParticipantsPanel.*;

public class SeminarActiveParticipantsPanelTest extends SciProTest {

    private SeminarActiveParticipantsPanel panel;
    private List<FinalSeminarActiveParticipation> participants;
    private FinalSeminar finalSeminar;
    private FinalSeminarActiveParticipation participation;

    @Before
    public void setUp() throws Exception {
        participation = new FinalSeminarActiveParticipation();
        DomainObjects.injectId(participation, 1L);
        FinalSeminarActiveParticipation finalSeminarActiveParticipation = participation;
        finalSeminarActiveParticipation.setUser(new UserBuilder().create());
        participants = Lists.newArrayList(finalSeminarActiveParticipation);
        finalSeminar = new FinalSeminar();
        finalSeminar.setStartDate(new DateTime().minusDays(1).toDate());
        User supervisorUser = new UserBuilder().create();
        finalSeminar.setProject(Project.builder()
                .title("title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING))
                .headSupervisor(supervisorUser)
                .build());
        finalSeminar.setActiveParticipations(participants);
        participation.setFinalSeminar(finalSeminar);

        setLoggedInAs(supervisorUser);
    }

    @Test
    public void has_list_of_participants() throws Exception {
        startPanel();
        tester.assertModelValue(path(panel, PARTICIPANTS), participants);
    }

    @Test
    public void has_user_link_panel() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, PARTICIPANTS, 0, USER), UserLinkPanel.class);
    }

    @Test
    public void has_remove_link_if_head_supervisor() throws Exception {
        startPanel();
        tester.assertVisible(path(panel, PARTICIPANTS, 0, REMOVE_PARTICIPANT));
    }

    @Test
    public void remove_link_invisible_if_not_head_supervisor() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        setRoles(Roles.SUPERVISOR);
        startPanel();
        tester.assertInvisible(path(panel, PARTICIPANTS, 0, REMOVE_PARTICIPANT));
    }

    @Test
    public void remove_link_removes_participant() throws Exception {
        startPanel();
        tester.clickLink(path(panel, PARTICIPANTS, 0, REMOVE_PARTICIPANT));
        Mockito.verify(finalSeminarService).save(finalSeminar);
    }

    @Test
    public void has_radios() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, PARTICIPANTS, 0, RADIOS), AjaxRadioChoice.class);
    }

    @Test
    public void radios_invisible_if_not_supervisor() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        startPanel();
        tester.assertInvisible(path(panel, PARTICIPANTS, 0, RADIOS));
    }

    @Test
    public void radios_invisible_if_seminar_is_in_the_future() throws Exception {
        finalSeminar.setStartDate(new DateTime().plusDays(1).toDate());
        startPanel();
        tester.assertInvisible(path(panel, PARTICIPANTS, 0, RADIOS));
    }

    @Test
    public void clicking_radios_saves_participation() throws Exception {
        startPanel();
        tester.executeAjaxEvent(path(panel, PARTICIPANTS, 0, RADIOS), "onclick");
        Mockito.verify(finalSeminarActiveParticipationService).save(participation);
    }

    @Test
    public void has_fenced_feedback_panel() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, FEEDBACK), FencedFeedbackPanel.class);
    }

    @Test
    public void repaints_feedback_panel() throws Exception {
        startPanel();
        tester.executeAjaxEvent(path(panel, PARTICIPANTS, 0, RADIOS), "onclick");
        tester.assertComponentOnAjaxResponse(path(panel, FEEDBACK));
    }

    @Test
    public void displays_feedback() throws Exception {
        startPanel();
        tester.executeAjaxEvent(path(panel, PARTICIPANTS, 0, RADIOS), "onclick");
        assertSuccessFeedback(panel.getString("feedback.active.participant", Model.of(participation)));
    }

    private void assertSuccessFeedback(String feedbackMessage) {
        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        MatcherAssert.assertThat(messages, IsCollectionContaining.hasItem(StringContains.containsString(feedbackMessage)));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SeminarActiveParticipantsPanel("id", Model.of(finalSeminar)));
    }
}
