package se.su.dsv.scipro.finalseminar;

import com.google.common.collect.Lists;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.model.Model;
import org.joda.time.DateTime;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.OppositionReport;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.TreeSet;

import static se.su.dsv.scipro.finalseminar.SeminarOppositionPanel.*;

public class SeminarOppositionPanelTest extends SciProTest {

    private SeminarOppositionPanel panel;
    private List<FinalSeminarOpposition> opponents;
    private FinalSeminar finalSeminar;
    private FinalSeminarOpposition opposition;
    private User coSupervisorUser;
    private User respondentUser;
    private User opponentUser;
    private User reviewerUser;
    private User supervisorUser;
    private Project project;

    @Before
    public void setUp() throws Exception {
        opposition = new FinalSeminarOpposition();
        opponentUser = new UserBuilder().create();
        opposition.setUser(opponentUser);
        DomainObjects.injectId(opposition, 1L);
        opponents = Lists.newArrayList(opposition);

        respondentUser = new UserBuilder().create();

        finalSeminar = new FinalSeminar();
        opposition.setFinalSeminar(finalSeminar);
        finalSeminar.setStartDate(new DateTime().minusDays(1).toDate());
        finalSeminar.setOppositions(opponents);

        supervisorUser = new UserBuilder().create();
        project = Project.builder().title("title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, ProjectType.BACHELOR_STRING, ProjectType.BACHELOR_STRING))
                .headSupervisor(supervisorUser)
                .build();

        coSupervisorUser = new UserBuilder().create();
        project.addCoSupervisor(coSupervisorUser);

        finalSeminar.setProject(project);


        setLoggedInAs(supervisorUser);
        Mockito.when(finalSeminarOppositionService.findAll(Matchers.isA(FinalSeminarOppositionService.FilterParams.class))).thenReturn(opponents);

        OppositionReport oppositionReport = Mockito.mock(OppositionReport.class);
        Mockito.when(oppositionReportService.findOrCreateReport(opposition)).thenReturn(oppositionReport);
    }

    @Test
    public void has_list_of_opponents() throws Exception {
        startPanel();
        tester.assertListView(path(panel, CONTAINER, OPPONENTS), opponents);
    }

    @Test
    public void has_user_panel() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, CONTAINER, OPPONENTS, 0, USER), UserLinkPanel.class);
    }

    @Test
    public void remove_link_visible_for_supervisor() throws Exception {
        startPanel();
        tester.assertVisible(path(panel, CONTAINER, OPPONENTS, 0, REMOVE));
    }

    @Test
    public void remove_link_invisible_if_not_admin_and_not_supervisor() throws Exception {
        addReviewerToProject(project);
        setLoggedInAs(new UserBuilder().create());
        setRoles(Roles.SUPERVISOR);
        startPanel();
        tester.assertInvisible(path(panel, CONTAINER, OPPONENTS, 0, REMOVE));
    }

    @Test
    public void no_null_pointer_when_no_reviewer() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        setRoles(Roles.SUPERVISOR);
        startPanel();
    }

    @Test
    public void remove_link_removes_opponent() throws Exception {
        startPanel();
        tester.clickLink(path(panel, CONTAINER, OPPONENTS, 0, REMOVE));
        Mockito.verify(finalSeminarService).save(finalSeminar);
    }

    @Test
    public void has_fenced_feedback_panel() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, FEEDBACK), FencedFeedbackPanel.class);
    }

    @Test
    public void feedback_and_points_invisible_before_Grading() throws Exception {
        opposition.setGrade(null);
        startPanel();
        tester.assertInvisible(path(panel, CONTAINER, OPPONENTS, 0, GRADE_CONTAINER));
    }

    @Test
    public void feedback_and_points_visible_if_approved() throws Exception {
        opposition.setGrade(FinalSeminarGrade.APPROVED);
        opposition.setPoints(1);
        opposition.setFeedback("bla bla bla bla bla");
        startPanel();
        tester.assertVisible(path(panel, CONTAINER, OPPONENTS, 0, GRADE_CONTAINER));
    }

    @Test
    public void feedback_and_points_visible_if_not_approved() throws Exception {
        opposition.setGrade(FinalSeminarGrade.NOT_APPROVED);
        opposition.setPoints(0);
        opposition.setFeedback("bla bla bla bla bla");
        startPanel();
        tester.assertVisible(path(panel, CONTAINER, OPPONENTS, 0, GRADE_CONTAINER));
    }

    @Test
    public void feedback_and_points_invisible_if_not_employee() throws Exception {
        addReviewerToProject(project);
        setLoggedInAs(new UserBuilder().create());
        setRoles(Roles.AUTHOR);
        opposition.setGrade(FinalSeminarGrade.APPROVED);
        startPanel();
        tester.assertInvisible(path(panel, CONTAINER, OPPONENTS, 0, GRADE_CONTAINER));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new SeminarOppositionPanel("id", Model.of(finalSeminar)));
    }

    private void addReviewerToProject(Project project) {
        reviewerUser = new UserBuilder().create();
        project.addReviewer(reviewerUser);
    }
}
