package se.su.dsv.scipro.finalthesis;

import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.file.File;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.UserBuilder;

import java.net.URI;
import java.net.URISyntaxException;

import static se.su.dsv.scipro.finalthesis.FinalThesisUploadFormPanel.FILE;
import static se.su.dsv.scipro.finalthesis.FinalThesisUploadFormPanel.FORM;

public class FinalThesisUploadFormPanelTest extends SciProTest {

    public static final Project SOME_PROJECT = new Project();
    private FinalThesisUploadFormPanel panel;
    private User user;

    @Before
    public void setUp() throws Exception {
        user = new UserBuilder().create();
        Mockito.when(userService.findOne(Matchers.isA(Long.class))).thenReturn(user);
        Mockito.when(finalSeminarService.hasHadFinalSeminar(Matchers.any(Project.class))).thenReturn(true);
        Mockito.when(finalThesisService.isUploadAllowed(Matchers.any(Project.class))).thenReturn(true);

    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void has_file_upload_field() throws Exception {
        startPanel();
        tester.assertComponent(path(panel, FORM, FILE), FileUploadField.class);
    }

    @Test
    public void file_field_is_required() throws Exception {
        startPanel();
        tester.assertRequired(path(panel, FORM, FILE));
    }

    @Test
    public void creates_a_final_thesis_from_the_uploaded_file() throws Exception {
        File file = getTestFile();
        String contentType = "application/pdf";

        setLoggedInAs(user);

        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, FORM));
        formTester.setFile(FILE, file, contentType);
        formTester.submit();

        ArgumentCaptor<FileUpload> captor = ArgumentCaptor.forClass(FileUpload.class);
        Mockito.verify(finalThesisService).upload(captor.capture(), Matchers.eq(SOME_PROJECT));

        FileUpload fileUpload = captor.getValue();
        Assert.assertEquals(contentType, fileUpload.getContentType());
        Assert.assertEquals(user, fileUpload.getUploader());
        Assert.assertEquals(file.getName(), fileUpload.getFileName());
    }

    @Test
    public void hides_upload_when_no_final_seminar() throws Exception {
        Mockito.when(finalSeminarService.hasHadFinalSeminar(Matchers.any(Project.class))).thenReturn(false);

        startPanel();

        tester.assertInvisible(path(panel, FORM, FILE));
    }

    @Test
    public void shows_upload_if_uploading_is_allowed() throws Exception {
        Mockito.when(finalThesisService.isUploadAllowed(SOME_PROJECT)).thenReturn(true);

        startPanel();

        tester.assertVisible(path(panel, FORM));
    }

    @Test
    public void hides_upload_if_uploading_is_not_allowed() throws Exception {
        Mockito.when(finalThesisService.isUploadAllowed(SOME_PROJECT)).thenReturn(false);

        startPanel();

        tester.assertInvisible(path(panel, FORM));
    }

    @Test
    public void feedback_when_no_file_is_selected() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, FORM));
        formTester.submit();

        tester.assertErrorMessages(panel.getString(FILE + ".Required"));
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new FinalThesisUploadFormPanel("id", Model.of(SOME_PROJECT)));
    }

    private File getTestFile() throws URISyntaxException {
        URI resource = Thread.currentThread().getContextClassLoader().getResource("final_thesis.txt").toURI();
        return new File(resource);
    }

}
