package se.su.dsv.scipro.forum.panels.threaded;

import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.model.Model;
import org.apache.wicket.protocol.http.mock.MockHttpServletRequest;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.forum.dataobjects.ForumPost;
import se.su.dsv.scipro.forum.dataobjects.ForumThread;
import se.su.dsv.scipro.forum.dataobjects.ProjectThread;
import se.su.dsv.scipro.forum.dataobjects.ReviewerThread;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewer.ReviewerInteractionForumThread;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ForumBuilder;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Arrays;
import java.util.Collections;
import java.util.TreeSet;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.*;
import static org.mockito.Mockito.when;

public class ViewForumThreadPanelTest extends SciProTest {

    ViewForumThreadPanel panel;
    ProjectThread thread;

    @Before
    public void setUp() throws Exception {
        thread = new ForumBuilder().createThread();
        ForumPost post = new ForumBuilder().createPost(thread.getForumThread());
        thread.getForumThread().addPost(post);

        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-mm-dd");

        User student = new UserBuilder().setFirstName("author").setLastName("1").create();
        User supervisor = new UserBuilder().setFirstName("head").setLastName("supervisor").create();
        Project project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .headSupervisor(supervisor)
                .build();
        DomainObjects.injectId(project, 1L);

        when(projectService.findOne(isA(Long.class))).thenReturn(project);

        when(projectService.isPartOfProject(isA(Project.class), isA(User.class))).thenReturn(true);
        when(projectService.getMemberType(isA(Project.class), isA(User.class))).thenReturn(Member.Type.SUPERVISOR);
        when(projectForumService.findOne(isA(Long.class))).thenReturn(thread);
        when(projectForumService.createReply(any(ProjectThread.class), any(User.class), any(String.class), anySetOf(FileDescription.class))).thenReturn(post);
        when(projectForumService.getPosts(isA(ProjectThread.class))).thenReturn(Arrays.asList(post, post));
    }

    @Test
    public void testRenders() throws Exception {
        startPanel();
        assert panel != null;
    }

    @Test
    public void testListItemContainsPostDetailsPanel() throws Exception {
        startPanel();
        tester.assertComponent(
                path(panel.getId(), ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.POST_LIST, 0, ViewForumThreadPanel.POST), ForumPostPanel.class);
    }

    @Test
    public void testPanelContainsTopReplyLink() throws Exception {
        startPanel();
        tester.assertComponent(path(panel.getId(), ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.TOP_REPLY_LINK), AjaxLink.class);
    }

    @Test
    public void testReplyLinkVisibility() throws Exception {
        startPanel();
        boolean isTopLinkVisible = panel.get(path(ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.TOP_REPLY_LINK)).isVisibleInHierarchy();
        assertTrue(isTopLinkVisible);
    }

    @Test
    public void testSubmittingReplyThroughUpperReplyPanelRendersWrapper() throws Exception {
        startPanel();
        tester.clickLink(path(panel.getId(), ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.TOP_REPLY_LINK));
        testFormSubmission(ViewForumThreadPanel.TOP_REPLY_PANEL);
    }

    @Test
    public void testClickTopReplyLinkShowsReplyPanelAndHidesLink() throws Exception {
        startPanel();

        tester.clickLink(path(panel.getId(), ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.TOP_REPLY_LINK));
        tester.assertComponentOnAjaxResponse(path(panel.getId(), ViewForumThreadPanel.WRAPPER));

        boolean isPanelVisible = panel.get(path(ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.TOP_REPLY_PANEL)).isVisibleInHierarchy();
        assertTrue(isPanelVisible);

        boolean isLinkVisible = panel.get(path(ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.TOP_REPLY_LINK)).isVisibleInHierarchy();
        assertFalse(isLinkVisible);
    }

    @Test
    public void back_link_not_visible_if_not_forum_thread() throws Exception {
        ReviewerThread reviewerThread = mockReviewerThread();
        panel = tester.startComponentInPage(new ViewForumThreadPanel<>("panel", Model.of(reviewerThread.getProject()), new ReviewerInteractionForumThread(reviewerInteractionService, fileDescriptionService)));
        tester.assertInvisible(path(panel.getId(), ViewForumThreadPanel.WRAPPER, ViewForumThreadPanel.BACK_LINK));
    }

    private ReviewerThread mockReviewerThread() {
        ForumThread forumThread = new ForumThread();
        forumThread.setId(77L);
        ReviewerThread reviewerThread = new ReviewerThread();
        Project project = new Project();
        project.setId(55L);
        reviewerThread.setProject(project);
        reviewerThread.setForumThread(forumThread);
        return reviewerThread;
    }

    private void testFormSubmission(final String panelId) {
        SubmitForumReplyPanel replyPanel = (SubmitForumReplyPanel) panel.get(path(ViewForumThreadPanel.WRAPPER, panelId));
        FormTester formTester = tester.newFormTester(replyPanel.getPageRelativePath()+ ":form");
        formTester.setValue("content", "content");
        MockHttpServletRequest servletRequest = tester.getRequest();
        servletRequest.setUseMultiPartContentType(true);
        formTester.submit();
        assertFalse(panel.get(path(ViewForumThreadPanel.WRAPPER, panelId)).isVisibleInHierarchy());
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new ViewForumThreadPanel<>("panel", Model.of(thread), new ProjectForumThread(projectForumService, fileDescriptionService)));
    }
}
