package se.su.dsv.scipro.grading;

import org.apache.wicket.Page;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReport;
import se.su.dsv.scipro.report.GradingReportTemplate;
import se.su.dsv.scipro.report.ReviewerGradingReport;
import se.su.dsv.scipro.report.SupervisorGradingReport;
import se.su.dsv.scipro.supervisor.pages.SupervisorPageTest;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.TreeSet;

import static org.mockito.Mockito.when;

public class SupervisorGradingPageTest extends SupervisorPageTest {

    public static final ProjectType BACHELOR = new ProjectType(DegreeLevel.FIRST_CYCLE, "bachelor", "bachelor");
    public static final Project SOME_PROJECT = Project.builder()
            .title("some title")
            .projectType(BACHELOR)
            .build();

    @Override
    protected Class<? extends Page> getPage() {
        return SupervisorGradingPage.class;
    }

    @Before
    public void setUp() throws Exception {
        User emp = new UserBuilder().setId(3434L).create();
        SOME_PROJECT.setHeadSupervisor(emp);
        setLoggedInAs(emp);
        DomainObjects.injectId(SOME_PROJECT, 1L);
        Mockito.when(projectService.findOne(Matchers.anyLong())).thenReturn(SOME_PROJECT);

        List<SupervisorGradingReport> reports = new ArrayList<>();
        SupervisorGradingReport report = new SupervisorGradingReport() {
            @Override
            public boolean isFinished() {
                return false;
            }
        };
        reports.add(report);
        Mockito.when(gradingReportService.getSupervisorGradingReports(SOME_PROJECT)).thenReturn(reports);

        createFinalThesis();
        mockReviewerGradingReport(SOME_PROJECT);
    }

    @Test
    public void has_fill_out_grading_report_panel_if_supervisor() throws Exception {
        startPage();
        tester.assertComponent(SupervisorGradingPage.FILL_OUT_PANEL, FillOutGradingReportPanel.class);
    }

    @Test
    public void shows_reviewer_report_for_comparison_if_the_initial_assessment_is_done_by_both_parties() throws Exception {
        reviewerHasDoneInitialAssessment();
        supervisorHasDoneInitialAssessment();
        startPage();
        tester.assertVisible("viewReviewerReport");
    }

    @Test
    public void does_not_show_reviewer_report_if_initial_assessment_is_not_done() throws Exception {
        startPage();
        tester.assertInvisible("viewReviewerReport");
    }

    private void supervisorHasDoneInitialAssessment() {
        for (SupervisorGradingReport supervisorGradingReport : gradingReportService.getSupervisorGradingReports(project)) {
            supervisorGradingReport.setState(GradingReport.State.REVIEWING);
        }
    }

    private void reviewerHasDoneInitialAssessment() {
        final ReviewerGradingReport reviewerGradingReport = mockReviewerGradingReport(SOME_PROJECT);
        reviewerGradingReport.setState(GradingReport.State.FINALIZED);
    }

    private ReviewerGradingReport mockReviewerGradingReport(final Project project) {
        ReviewerGradingReport reviewerGradingReport = new GradingReportTemplate(project.getProjectType(), 0).createReviewerReport(project);
        when(gradingReportService.getReviewerGradingReport(project)).thenReturn(reviewerGradingReport);
        return reviewerGradingReport;
    }

    private void createFinalThesis() {
        FinalThesis finalThesis = new FinalThesis();
        finalThesis.setId(1L);
        FileDescription fileDescription = new FileDescription();
        fileDescription.setId(1L);
        finalThesis.setFileDescription(fileDescription);
        when(finalThesisService.findByProject(Matchers.<Project>any())).thenReturn(finalThesis);
    }
}
