package se.su.dsv.scipro.milestones.panels;

import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.milestones.dataobjects.Milestone;
import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.*;

import static org.mockito.Matchers.*;
import static org.mockito.Mockito.when;

public class MilestoneOverviewPanelTest extends SciProTest {

    private MileStoneOverviewPanel panel;
    private List<MilestoneActivityTemplate> activityList;
    private List<MilestonePhaseTemplate> phaseList;
    private Project project;

    @Before
    public void setUp() throws Exception {
        User user = new UserBuilder().setFirstName("author").setLastName("1").create();
        project = Project.builder()
                .title("some title")
                .projectType(new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor"))
                .build();
        DomainObjects.injectId(project, 1L);

        MilestoneActivityTemplate projectActivity = new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.PROJECT);
        MilestoneActivityTemplate studentActivity = new MilestoneActivityTemplate(MilestoneActivityTemplate.Type.STUDENT);
        activityList = Arrays.asList(projectActivity, studentActivity);
        MilestonePhaseTemplate phase = new MilestonePhaseTemplate("This is a title", "This is a description");
        phaseList = Collections.singletonList(phase);

        Milestone milestone = new Milestone(project, new MilestoneActivityTemplate());
        DomainObjects.injectId(milestone, 2L);

        when(mileStonePhaseTemplateService.findAllActive(project.getDateCreated())).thenReturn(phaseList);
        when(dateService.getFormat(isA(DateStyle.class))).thenReturn("yyyy-MM-dd");
        when(milestoneActivityTemplateService.findActiveByPhase(isA(MilestonePhaseTemplate.class), eq(project))).thenReturn(activityList);
        when(mileStoneService.getMileStone(isA(Project.class), isA(MilestoneActivityTemplate.class))).thenReturn(milestone);
        when(mileStoneService.getMileStone(isA(Project.class), isA(User.class), isA(MilestoneActivityTemplate.class))).thenReturn(milestone);
        startPanel();
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new MileStoneOverviewPanel("panel", Model.of(project)));
    }

    @Test
    public void testRenders() throws Exception {
        assert panel != null;
    }

    @Test
    public void testPanelContainsListOfPhases() throws Exception {
        String listPath = path(panel.getId(), MileStoneOverviewPanel.TABLE_CONTAINER, MileStoneOverviewPanel.PHASE_LIST);
        tester.assertComponent(listPath, ListView.class);
        tester.assertListView(listPath, phaseList);
    }

    @Test
    public void testEachPhaseContainsListOfMileStones() throws Exception {
        String activityListPath = phasePath(MileStoneOverviewPanel.MILESTONE_LIST);
        tester.assertComponent(activityListPath, ListView.class);
        tester.assertListView(activityListPath, activityList);
    }

    @Test
    public void testMileStoneContainsStudentList() throws Exception {
        String studentListPath = activityPath(MileStoneOverviewPanel.STUDENTS);
        tester.assertComponent(studentListPath, ListView.class);
        tester.assertListView(studentListPath, new ArrayList<>(project.getProjectParticipants()));
    }

    @Test
    public void phase_is_hidden_if_no_activities() throws Exception {
        when(milestoneActivityTemplateService.findActiveByPhase(any(MilestonePhaseTemplate.class), any(Project.class))).thenReturn(Collections.<MilestoneActivityTemplate>emptyList());
        startPanel();

        tester.assertInvisible(phasePath());
    }

    private String phasePath(String... paths) {
        return path(panel.getId(), MileStoneOverviewPanel.TABLE_CONTAINER, MileStoneOverviewPanel.PHASE_LIST, 0, path((Object[]) paths));
    }

    private String activityPath(String... paths) {
        return path(panel.getId(), MileStoneOverviewPanel.TABLE_CONTAINER, MileStoneOverviewPanel.PHASE_LIST, 0, MileStoneOverviewPanel.MILESTONE_LIST, 0, path((Object[]) paths));
    }

}
