package se.su.dsv.scipro.peer;

import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.file.File;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import scala.Option;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.checklists.TrafficLightPanel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import java.net.URL;

import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.verify;
import static se.su.dsv.scipro.peer.PerformReviewPanel.*;

public class PerformReviewPanelTest extends SciProTest {

    private PeerReview peerReview;
    private PerformReviewPanel panel;

    @Before
    public void setUp() throws Exception {
        ProjectType projectType = new ProjectType(DegreeLevel.FIRST_CYCLE, "Name", "Description");
        Project project = Project.builder().title("Title").projectType(projectType).build();
        PeerRequest peerRequest = new PeerRequest();
        User reviewer = User.builder().firstName("Kalle").lastName("Testare").emailAddress("kalle@dsv.su.se").build();
        DomainObjects.injectId(reviewer, 23432L);
        peerReview = new PeerReview();
        peerReview.setReviewer(reviewer);
        peerReview.setPeerRequest(peerRequest);
        peerReview.setProject(project);
        peerReview.getAnswers().add(new Answer(peerReview, "My question?"));
        peerReview.setComment("My awesome comment");
        setLoggedInAs(reviewer);
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void lists_all_the_reviews_answers() throws Exception {
        startPanel();
        tester.assertListView(path(panel, REVIEW_FORM, ANSWERS), peerReview.getAnswers());
    }

    @Test
    public void lists_every_question_text() throws Exception {
        int answer = 0;
        startPanel();
        tester.assertLabel(path(panel, REVIEW_FORM, ANSWERS, answer, QUESTION), peerReview.getAnswers().get(answer).getQuestion());
    }

    @Test
    public void has_input_to_answer_every_question() throws Exception {
        int answer = 0;
        startPanel();

        String path = path(panel, REVIEW_FORM, ANSWERS, answer, MOTIVATION);
        tester.assertComponent(path, TextArea.class);
        tester.assertModelValue(path, peerReview.getAnswers().get(0).getMotivation());
    }

    @Test
    public void shows_traffic_light_panel() throws Exception {
        int answer = 0;
        startPanel();

        String path = path(panel, REVIEW_FORM, ANSWERS, answer, ANSWER);
        tester.assertComponent(path, TrafficLightPanel.class);
        tester.assertModelValue(path, peerReview.getAnswers().get(0).getAnswer());
    }

    @Test
    public void submit_review_shows_error_if_not_all_questions_are_answered() throws Exception {
        doThrow(MissingAnswerException.class).when(performReviewService).submit(peerReview, Option.<FileUpload>empty());

        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, REVIEW_FORM));
        formTester.submit(SUBMIT_BUTTON);

        tester.assertErrorMessages(panel.getString("missing.answer"));
    }

    @Test
    public void submit_review_shows_error_if_no_comment_provided() throws Exception {
        doThrow(TooShortCommentException.class).when(performReviewService).submit(peerReview, Option.<FileUpload>empty());

        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, REVIEW_FORM));
        formTester.submit(SUBMIT_BUTTON);

        tester.assertErrorMessages(panel.getString("too.short.comment"));
    }

    @Test
    public void save_review_throw_no_error() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, REVIEW_FORM));
        formTester.submit();

        tester.assertNoErrorMessage();
    }

    @Test
    public void submits_optional_file_alongside_review() throws Exception {
        startPanel();
        FormTester formTester = tester.newFormTester(path(panel, REVIEW_FORM));
        formTester.setFile(FILE, getTestFile(), "text/plain");
        formTester.submit(SUBMIT_BUTTON);

        ArgumentCaptor<Option<FileUpload>> captor = optionalFileUpload();
        verify(performReviewService).submit(eq(peerReview), captor.capture());

        Option<FileUpload> upload = captor.getValue();
        assertTrue(upload.isDefined());
    }

    @Test
    public void has_text_area_for_peer_review_comment_if_there_are_no_answer() throws Exception {
        peerReview.getAnswers().clear();
        startPanel();
        String path = path(panel, REVIEW_FORM, COMMENT);
        tester.assertComponent(path, TextArea.class);
        tester.assertModelValue(path, peerReview.getComment());
    }

    @Test
    public void text_area_for_comment_is_hidden_if_there_are_answers() throws Exception {
        startPanel();
        tester.assertInvisible(path(panel, REVIEW_FORM, COMMENT));
    }

    @Test
    public void panel_is_hidden_if_current_user_has_no_in_progress_review() throws Exception {
        startPanelWithoutInProgressReview();

        tester.assertInvisible(path(panel));
    }

    @Test
    public void shows_deadline() throws Exception {
        startPanel();

        tester.assertComponent(path(panel, REVIEW_FORM, DEADLINE), DateLabel.class);
        tester.assertModelValue(path(panel, REVIEW_FORM, DEADLINE), peerReview.getDeadline());
    }

    @Test
    public void shows_details_about_the_request() throws Exception {
        startPanel();

        tester.assertComponent(path(panel, REQUEST_DETAILS), ReviewPageRequestDetailsPanel.class);
        tester.assertModelValue(path(panel, REQUEST_DETAILS), peerReview.getPeerRequest());
    }

    @SuppressWarnings("unchecked")
    private static ArgumentCaptor<Option<FileUpload>> optionalFileUpload() {
        ArgumentCaptor captor = ArgumentCaptor.forClass(Option.class);
        return (ArgumentCaptor<Option<FileUpload>>) captor;
    }

    private static File getTestFile() throws Exception {
        URL resource = Thread.currentThread().getContextClassLoader().getResource("peer_review.txt");
        assert resource != null;
        return new File(resource.toURI());
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new PerformReviewPanel("id", Model.of(peerReview)));
    }

    private void startPanelWithoutInProgressReview() {
        panel = tester.startComponentInPage(new PerformReviewPanel("id", new Model<PeerReview>()));
    }
}
