package se.su.dsv.scipro.phd;

import org.apache.wicket.feedback.FeedbackMessage;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.*;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsCollectionContaining.hasItem;
import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class DepartmentServiceByUserPanelTest extends SciProTest {

    private DepartmentServiceByUserPanel panel;

    @Before
    public void setUp() throws Exception {
        when(departmentServiceService.getCurrent(any(User.class))).thenReturn(getCurrent());
        when(departmentServiceService.getPrevious(any(User.class))).thenReturn(getPrevious());
        startPanel();
    }

    @Test
    public void submit_new_department_service() throws Exception {
        String hours = "10";
        submitForm(hours);

        ArgumentCaptor<DepartmentService> captor = ArgumentCaptor.forClass(DepartmentService.class);
        verify(departmentServiceService).save(captor.capture());

        assertEquals(String.valueOf(captor.getValue().getHours()), hours);
    }

    @Test
    public void submit_generates_feedback() throws Exception {
        submitForm("10");

        List<Serializable> messages = tester.getMessages(FeedbackMessage.SUCCESS);
        assertThat(messages, hasItem(panel.getString("saved")));
    }

    @Test
    public void submit_fails_without_valid_fields() throws Exception {
        submitForm(null);
        tester.assertErrorMessages(panel.getString("hours.Required"));
    }

    @Test
    public void lists_current_department_services() throws Exception {
        tester.assertListView(path("panel", "current"), getCurrent());
    }

    @Test
    public void lists_all_department_services() throws Exception {
        tester.clickLink(path("panel", "show"));
        tester.assertListView(path("panel", "previous", "previousList"), getPrevious());
    }

    @Test
    public void no_current_label_visible_if_list_is_empty() throws Exception {
        when(departmentServiceService.getCurrent(any(User.class))).thenReturn(new ArrayList<DepartmentService>());
        startPanel();
        tester.assertVisible(path("panel", "noCurrent"));
    }

    @Test
    public void no_current_label_invisible_with_list_populated() throws Exception {
        tester.assertInvisible(path("panel", "noCurrent"));
    }

    @Test
    public void delete_link_deletes_thesis() throws Exception {
        int index = 0;
        tester.clickLink(path("panel", "current", index, "delete"));
        verify(departmentServiceService).delete(getCurrent().get(index));
    }

    @Test
    public void show_all_link_visible_if_previous_exists() throws Exception {
        tester.assertVisible(path("panel", "show"));
    }

    @Test
    public void show_all_link_invisible_if_no_previous_exists() throws Exception {
        when(departmentServiceService.getPrevious(any(User.class))).thenReturn(new ArrayList<DepartmentService>());
        startPanel();
        tester.assertInvisible(path("panel", "show"));
    }

    private List<DepartmentService> getPrevious() {
        return Collections.singletonList(new DepartmentService());
    }

    private List<DepartmentService> getCurrent() {
        return Collections.singletonList(new DepartmentService());
    }

    private void submitForm(String hours) {
        FormTester form = tester.newFormTester(path("panel", "form"));
        form.setValue("hours", hours);
        setDate(form, "dateStarted", new Date());
        setDate(form, "dateEnded", new Date());
        form.submit();
    }

    private void setDate(FormTester formTester, String field, Date date) {
        String dateString = new SimpleDateFormat(dateService.getFormat(DateStyle.DATE)).format(date);
        formTester.setValue(field, dateString);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new DepartmentServiceByUserPanel("panel", Model.of(SciProSession.get().getUser())));
    }
}
