package se.su.dsv.scipro.workerthreads;

import com.google.common.eventbus.EventBus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.activityplan.ActivityPlanFacade;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.match.*;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class IdeaExportWorkerTest {

    @Mock
    private IdeaService ideaService;
    @Mock
    private MailEventService mailService;
    @Mock
    private IdeaCreationJudge ideaCreationJudge;
    @Mock
    private ProjectService projectService;
    @Mock
    private EventBus eventBus;
    @Mock
    private ActivityPlanFacade activityPlanFacade;

    private IdeaExportWorker worker;

    private Idea idea;

    @Before
    public void setUp() throws Exception {
        worker = new IdeaExportWorker(ideaService, mailService, projectService, Collections.singleton(ideaCreationJudge), eventBus, activityPlanFacade) {
            @Override
            protected void beginTransaction() {
            }

            @Override
            protected void commitTransaction() {
            }

            @Override
            protected void rollbackTransaction() {
            }
        };

        when(ideaCreationJudge.ruling(any(Idea.class))).thenReturn(Decision.allowed());

        idea = prepareIdea();
        setupDatabase(idea);
    }

    @Test
    public void if_judge_says_no_then_do_nothing() throws Exception {
        denyStart();

        worker.doWork();

        verify(projectService, never()).save(any(Project.class));
    }

    @Test
    public void if_no_judge_says_no_then_create_project() throws Exception {
        worker.doWork();

        ArgumentCaptor<Project> captor = ArgumentCaptor.forClass(Project.class);
        verify(projectService).save(captor.capture());
        assertEquals(idea.getTitle(), captor.getValue().getTitle());
    }

    @Test
    public void if_idea_start_is_allowed_post_idea_started_event() throws Exception {
        worker.doWork();
        verify(eventBus).post(isA(ProjectStartedEvent.class));
    }

    @Test
    public void if_idea_start_is_denied_post_idea_start_denied_event() throws Exception {
        denyStart();
        worker.doWork();
        verify(eventBus).post(isA(ProjectStartDeniedEvent.class));
    }

    private void denyStart() throws Exception {
        denyStart("No!");
    }

    private void denyStart(String reason) throws Exception {
        denyStart(any(Idea.class), reason);
    }

    private void denyStart(final Idea idea, final String reason) {
        when(ideaCreationJudge.ruling(idea)).thenReturn(Decision.denied(reason));
    }

    private void setupDatabase(Idea... ideas) {
        when(ideaService.findReadyToStart()).thenReturn(Arrays.asList(ideas));
    }

    private Idea prepareIdea() {
        User student = User.builder().firstName("Kalle").lastName("Tester").emailAddress("kalle@dsv.su.se").build();
        DomainObjects.injectId(student, 234L);
        User supervisor = User.builder().firstName("Henrik").lastName("Tester").emailAddress("henrik@dsv.su.se").build();
        DomainObjects.injectId(supervisor, 34L);
        Match match = new Match();
        match.setSupervisor(supervisor);
        Idea idea = new Idea();
        idea.setMatch(match);
        idea.setTitle("Foobar");
        idea.setApplicationPeriod(newApplicationPeriod());
        idea.setProjectType(newProjectType());
        idea.addIdeaParticipation(new IdeaParticipation(student, idea));
        return idea;
    }

    private ProjectType newProjectType() {
        return new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master");
    }

    private ApplicationPeriod newApplicationPeriod() {
        ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setCourseStartDate(new Date());
        return applicationPeriod;
    }
}
